@@ -18,19 +18,45 @@
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 from sat.core.i18n import _
-from bridge_frontend import BridgeFrontend
+from bridge_frontend import BridgeFrontend, BridgeException
 import dbus
-from logging import debug, error
+from sat.core.log import getLogger
+log = getLogger(__name__)
 from sat.core.exceptions import BridgeExceptionNoService, BridgeInitError
 
 from dbus.mainloop.glib import DBusGMainLoop
 DBusGMainLoop(set_as_default=True)
 
+import ast
+
 const_INT_PREFIX = "org.goffi.SAT"  # Interface prefix
 const_ERROR_PREFIX = const_INT_PREFIX + ".error"
 const_OBJ_PATH = '/org/goffi/SAT/bridge'
 const_CORE_SUFFIX = ".core"
 const_PLUGIN_SUFFIX = ".plugin"
+const_TIMEOUT = 120
+
+
+def dbus_to_bridge_exception(dbus_e):
+    """Convert a DBusException to a BridgeException.
+
+    @param dbus_e (DBusException)
+    @return: BridgeException
+    """
+    full_name = dbus_e.get_dbus_name()
+    if full_name.startswith(const_ERROR_PREFIX):
+        name = dbus_e.get_dbus_name()[len(const_ERROR_PREFIX) + 1:]
+    else:
+        name = full_name
+    # XXX: dbus_e.args doesn't contain the original DBusException args, but we
+    # receive its serialized form in dbus_e.args[0]. From that we can rebuild
+    # the original arguments list thanks to ast.literal_eval (secure eval).
+    message = dbus_e.get_dbus_message()  # similar to dbus_e.args[0]
+    try:
+        message, condition = ast.literal_eval(message)
+    except (SyntaxError, ValueError, TypeError):
+        condition = ''
+    return BridgeException(name, message, condition)
 
 
 class DBusBridgeFrontend(BridgeFrontend):
@@ -47,7 +73,7 @@ class DBusBridgeFrontend(BridgeFrontend):
             if e._dbus_error_name == 'org.freedesktop.DBus.Error.ServiceUnknown':
                 raise BridgeExceptionNoService
             elif e._dbus_error_name == 'org.freedesktop.DBus.Error.NotSupported':
-                print u"D-Bus is not launched, please see README to see instructions on how to launch it"
+                log.error(_(u"D-Bus is not launched, please see README to see instructions on how to launch it"))
                 raise BridgeInitError
             else:
                 raise e
@@ -59,7 +85,7 @@ class DBusBridgeFrontend(BridgeFrontend):
         elif iface == "plugin":
             self.db_plugin_iface.connect_to_signal(functionName, handler)
         else:
-            error(_('Unknown interface'))
+            log.error(_('Unknown interface'))
 
     def __getattribute__(self, name):
         """ usual __getattribute__ if the method exists, else try to find a plugin method """
@@ -89,8 +115,9 @@ class DBusBridgeFrontend(BridgeFrontend):
                 method = getattr(self.db_plugin_iface, name)
 
                 if async:
+                    kwargs['timeout'] = const_TIMEOUT
                     kwargs['reply_handler'] = _callback
-                    kwargs['error_handler'] = lambda err: _errback(err._dbus_error_name[len(const_ERROR_PREFIX) + 1:])
+                    kwargs['error_handler'] = lambda err: _errback(dbus_to_bridge_exception(err))
 
                 return method(*args, **kwargs)
 
@@ -98,29 +125,29 @@ class DBusBridgeFrontend(BridgeFrontend):
     def addContact(self, entity_jid, profile_key="@DEFAULT@"):
         return self.db_core_iface.addContact(entity_jid, profile_key)
 
-    def asyncConnect(self, profile_key="@DEFAULT@", callback=None, errback=None):
-        return self.db_core_iface.asyncConnect(profile_key, reply_handler=callback, error_handler=lambda err:errback(err._dbus_error_name[len(const_ERROR_PREFIX)+1:]))
+    def asyncConnect(self, profile_key="@DEFAULT@", password='', callback=None, errback=None):
+        return self.db_core_iface.asyncConnect(profile_key, password, timeout=const_TIMEOUT, reply_handler=callback, error_handler=lambda err:errback(dbus_to_bridge_exception(err)))
 
-    def asyncCreateProfile(self, profile, callback=None, errback=None):
-        return self.db_core_iface.asyncCreateProfile(profile, reply_handler=callback, error_handler=lambda err:errback(err._dbus_error_name[len(const_ERROR_PREFIX)+1:]))
+    def asyncCreateProfile(self, profile, password='', callback=None, errback=None):
+        return self.db_core_iface.asyncCreateProfile(profile, password, timeout=const_TIMEOUT, reply_handler=callback, error_handler=lambda err:errback(dbus_to_bridge_exception(err)))
+
+    def asyncDeleteProfile(self, profile, callback=None, errback=None):
+        return self.db_core_iface.asyncDeleteProfile(profile, timeout=const_TIMEOUT, reply_handler=callback, error_handler=lambda err:errback(dbus_to_bridge_exception(err)))
 
     def asyncGetParamA(self, name, category, attribute="value", security_limit=-1, profile_key="@DEFAULT@", callback=None, errback=None):
-        return unicode(self.db_core_iface.asyncGetParamA(name, category, attribute, security_limit, profile_key, reply_handler=callback, error_handler=lambda err:errback(err._dbus_error_name[len(const_ERROR_PREFIX)+1:])))
+        return unicode(self.db_core_iface.asyncGetParamA(name, category, attribute, security_limit, profile_key, timeout=const_TIMEOUT, reply_handler=callback, error_handler=lambda err:errback(dbus_to_bridge_exception(err))))
 
     def confirmationAnswer(self, id, accepted, data, profile):
         return self.db_core_iface.confirmationAnswer(id, accepted, data, profile)
 
-    def connect(self, profile_key="@DEFAULT@"):
-        return self.db_core_iface.connect(profile_key)
-
-    def createProfile(self, profile):
-        return self.db_core_iface.createProfile(profile)
-
     def delContact(self, entity_jid, profile_key="@DEFAULT@"):
         return self.db_core_iface.delContact(entity_jid, profile_key)
 
-    def deleteProfile(self, profile):
-        return self.db_core_iface.deleteProfile(profile)
+    def discoInfos(self, entity_jid, profile_key, callback=None, errback=None):
+        return self.db_core_iface.discoInfos(entity_jid, profile_key, timeout=const_TIMEOUT, reply_handler=callback, error_handler=lambda err:errback(dbus_to_bridge_exception(err)))
+
+    def discoItems(self, entity_jid, profile_key, callback=None, errback=None):
+        return self.db_core_iface.discoItems(entity_jid, profile_key, timeout=const_TIMEOUT, reply_handler=callback, error_handler=lambda err:errback(dbus_to_bridge_exception(err)))
 
     def disconnect(self, profile_key="@DEFAULT@"):
         return self.db_core_iface.disconnect(profile_key)
@@ -138,7 +165,7 @@ class DBusBridgeFrontend(BridgeFrontend):
         return self.db_core_iface.getEntityData(jid, keys, profile)
 
     def getHistory(self, from_jid, to_jid, limit, between=True, profile="@NONE@", callback=None, errback=None):
-        return self.db_core_iface.getHistory(from_jid, to_jid, limit, between, profile, reply_handler=callback, error_handler=lambda err:errback(err._dbus_error_name[len(const_ERROR_PREFIX)+1:]))
+        return self.db_core_iface.getHistory(from_jid, to_jid, limit, between, profile, timeout=const_TIMEOUT, reply_handler=callback, error_handler=lambda err:errback(dbus_to_bridge_exception(err)))
 
     def getLastResource(self, contact_jid, profile_key="@DEFAULT@"):
         return unicode(self.db_core_iface.getLastResource(contact_jid, profile_key))
@@ -153,19 +180,19 @@ class DBusBridgeFrontend(BridgeFrontend):
         return unicode(self.db_core_iface.getParamA(name, category, attribute, profile_key))
 
     def getParams(self, security_limit=-1, app='', profile_key="@DEFAULT@", callback=None, errback=None):
-        return unicode(self.db_core_iface.getParams(security_limit, app, profile_key, reply_handler=callback, error_handler=lambda err:errback(err._dbus_error_name[len(const_ERROR_PREFIX)+1:])))
+        return unicode(self.db_core_iface.getParams(security_limit, app, profile_key, timeout=const_TIMEOUT, reply_handler=callback, error_handler=lambda err:errback(dbus_to_bridge_exception(err))))
 
     def getParamsCategories(self, ):
         return self.db_core_iface.getParamsCategories()
 
     def getParamsForCategory(self, category, security_limit=-1, app='', profile_key="@DEFAULT@", callback=None, errback=None):
-        return unicode(self.db_core_iface.getParamsForCategory(category, security_limit, app, profile_key, reply_handler=callback, error_handler=lambda err:errback(err._dbus_error_name[len(const_ERROR_PREFIX)+1:])))
+        return unicode(self.db_core_iface.getParamsForCategory(category, security_limit, app, profile_key, timeout=const_TIMEOUT, reply_handler=callback, error_handler=lambda err:errback(dbus_to_bridge_exception(err))))
 
     def getParamsUI(self, security_limit=-1, app='', profile_key="@DEFAULT@", callback=None, errback=None):
-        return unicode(self.db_core_iface.getParamsUI(security_limit, app, profile_key, reply_handler=callback, error_handler=lambda err:errback(err._dbus_error_name[len(const_ERROR_PREFIX)+1:])))
+        return unicode(self.db_core_iface.getParamsUI(security_limit, app, profile_key, timeout=const_TIMEOUT, reply_handler=callback, error_handler=lambda err:errback(dbus_to_bridge_exception(err))))
 
-    def getPresenceStatus(self, profile_key="@DEFAULT@"):
-        return self.db_core_iface.getPresenceStatus(profile_key)
+    def getPresenceStatuses(self, profile_key="@DEFAULT@"):
+        return self.db_core_iface.getPresenceStatuses(profile_key)
 
     def getProfileName(self, profile_key="@DEFAULT@"):
         return unicode(self.db_core_iface.getProfileName(profile_key))
@@ -176,6 +203,9 @@ class DBusBridgeFrontend(BridgeFrontend):
     def getProgress(self, id, profile):
         return self.db_core_iface.getProgress(id, profile)
 
+    def getReady(self, callback=None, errback=None):
+        return self.db_core_iface.getReady(timeout=const_TIMEOUT, reply_handler=callback, error_handler=lambda err:errback(dbus_to_bridge_exception(err)))
+
     def getVersion(self, ):
         return unicode(self.db_core_iface.getVersion())
 
@@ -189,22 +219,25 @@ class DBusBridgeFrontend(BridgeFrontend):
         return self.db_core_iface.isConnected(profile_key)
 
     def launchAction(self, callback_id, data, profile_key="@DEFAULT@", callback=None, errback=None):
-        return self.db_core_iface.launchAction(callback_id, data, profile_key, reply_handler=callback, error_handler=lambda err:errback(err._dbus_error_name[len(const_ERROR_PREFIX)+1:]))
+        return self.db_core_iface.launchAction(callback_id, data, profile_key, timeout=const_TIMEOUT, reply_handler=callback, error_handler=lambda err:errback(dbus_to_bridge_exception(err)))
+
+    def loadParamsTemplate(self, filename):
+        return self.db_core_iface.loadParamsTemplate(filename)
 
     def paramsRegisterApp(self, xml, security_limit=-1, app=''):
         return self.db_core_iface.paramsRegisterApp(xml, security_limit, app)
 
-    def registerNewAccount(self, login, password, email, host, port=5222):
-        return unicode(self.db_core_iface.registerNewAccount(login, password, email, host, port))
+    def saveParamsTemplate(self, filename):
+        return self.db_core_iface.saveParamsTemplate(filename)
 
     def sendMessage(self, to_jid, message, subject='', mess_type="auto", extra={}, profile_key="@NONE@", callback=None, errback=None):
-        return self.db_core_iface.sendMessage(to_jid, message, subject, mess_type, extra, profile_key, reply_handler=callback, error_handler=lambda err:errback(err._dbus_error_name[len(const_ERROR_PREFIX)+1:]))
+        return self.db_core_iface.sendMessage(to_jid, message, subject, mess_type, extra, profile_key, timeout=const_TIMEOUT, reply_handler=callback, error_handler=lambda err:errback(dbus_to_bridge_exception(err)))
 
     def setParam(self, name, value, category, security_limit=-1, profile_key="@DEFAULT@"):
         return self.db_core_iface.setParam(name, value, category, security_limit, profile_key)
 
-    def setPresence(self, to_jid='', show='', priority=0, statuses={}, profile_key="@DEFAULT@"):
-        return self.db_core_iface.setPresence(to_jid, show, priority, statuses, profile_key)
+    def setPresence(self, to_jid='', show='', statuses={}, profile_key="@DEFAULT@"):
+        return self.db_core_iface.setPresence(to_jid, show, statuses, profile_key)
 
     def subscription(self, sub_type, entity, profile_key="@DEFAULT@"):
         return self.db_core_iface.subscription(sub_type, entity, profile_key)
