@@ -22,7 +22,8 @@ import urwid
 from urwid_satext import sat_widgets
 from sat_frontends.tools.games import TarotCard
 from sat_frontends.quick_frontend.quick_card_game import QuickCardGame
-from sat_frontends.primitivus.xmlui import XMLUI
+from sat_frontends.primitivus import xmlui
+from sat_frontends.primitivus.keys import action_key_map as a_key
 
 
 class CardDisplayer(urwid.Text):
@@ -38,7 +39,7 @@ class CardDisplayer(urwid.Text):
         return True
 
     def keypress(self, size, key):
-        if key == ' ':
+        if key == a_key['CARD_SELECT']:
             self.select(not self.__selected)
             self._emit('click')
         return key
@@ -95,7 +96,7 @@ class Hand(urwid.WidgetWrap):
             return self.columns.keypress(size, key)
         else:
             #No card displayed, we still have to manage the clicks
-            if key == ' ':
+            if key == a_key['CARD_SELECT']:
                 self._emit('click', None)
             return key
 
@@ -122,7 +123,7 @@ class Hand(urwid.WidgetWrap):
             self.columns.column_types.append(('fixed', 3))
             urwid.connect_signal(widget, 'click', self.__onClick)
         self.columns.contents.append((urwid.Text(''), ('weight', 1, False)))
-        self.columns.set_focus(1)
+        self.columns.focus_position = 1
 
     def __onClick(self, card_wid):
         self._emit('click', card_wid)
@@ -269,7 +270,7 @@ class CardGame(QuickCardGame, urwid.WidgetWrap):
     def chooseContrat(self, xml_data):
         """Called when the player has to select his contrat
         @param xml_data: SàT xml representation of the form"""
-        form = XMLUI(self.parent.host, xml_data, title=_('Please choose your contrat'), flags=['NO_CANCEL'])
+        form = xmlui.create(self.parent.host, xml_data, title=_('Please choose your contrat'), flags=['NO_CANCEL'])
         form.show(valign='top')
 
     def showCards(self, game_stage, cards, data):
@@ -288,7 +289,7 @@ class CardGame(QuickCardGame, urwid.WidgetWrap):
             title = _("Draw game")
         else:
             title = _('You win \o/') if self.player_nick in winners else _('You loose :(')
-        form = XMLUI(self.parent.host, xml_data, title=title, flags=['NO_CANCEL'])
+        form = xmlui.create(self.parent.host, xml_data, title=title, flags=['NO_CANCEL'])
         form.show()
 
     def invalidCards(self, phase, played_cards, invalid_cards):
