@@ -18,15 +18,18 @@
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 from sat.core.i18n import _
+from sat.core import log as logging
+log = logging.getLogger(__name__)
 import urwid
 from urwid_satext import sat_widgets
 from urwid_satext.files_management import FileDialog
 from sat_frontends.quick_frontend.quick_chat import QuickChat
 from sat_frontends.primitivus.card_game import CardGame
 from sat_frontends.quick_frontend.quick_utils import escapePrivate, unescapePrivate
-from sat_frontends.primitivus.xmlui import XMLUI
+from sat_frontends.primitivus.constants import Const as C
+from sat_frontends.primitivus.keys import action_key_map as a_key
 import time
-from sat.tools.jid  import JID
+from sat_frontends.tools.jid  import JID
 
 
 class ChatText(urwid.FlowWidget):
@@ -71,7 +74,10 @@ class ChatText(urwid.FlowWidget):
             else:
                 render_txt.append(('my_nick' if self.my_mess else 'other_nick',"[%s] " % self.nick))
         render_txt.append(self.message)
-        return urwid.Text(render_txt, align=self.align)
+        txt_widget = urwid.Text(render_txt, align=self.align)
+        if self.is_info:
+            return urwid.AttrMap(txt_widget, 'info_msg')
+        return txt_widget
 
 
 class Chat(urwid.WidgetWrap, QuickChat):
@@ -94,26 +100,26 @@ class Chat(urwid.WidgetWrap, QuickChat):
         self.subject = None
 
     def keypress(self, size, key):
-        if key == "meta p": #user wants to (un)hide the presents panel
+        if key == a_key['OCCUPANTS_HIDE']: #user wants to (un)hide the presents panel
             if self.type == 'group':
                 widgets = [widget for (widget, options) in self.chat_colums.contents]
                 if self.present_panel in widgets:
                     self.__removePresentPanel()
                 else:
                     self.__appendPresentPanel()
-        elif key == "meta t": #user wants to (un)hide timestamp
+        elif key == a_key['TIMESTAMP_HIDE']: #user wants to (un)hide timestamp
             self.show_timestamp = not self.show_timestamp
             for wid in self.content:
                 wid._invalidate()
-        elif key == "meta n": #user wants to (not) use short nick
+        elif key == a_key['SHORT_NICKNAME']: #user wants to (not) use short nick
             self.show_short_nick = not self.show_short_nick
             for wid in self.content:
                 wid._invalidate()
-        elif key == "meta l": #user wants to (un)hide widget decoration
+        elif key == a_key['DECORATION_HIDE']: #user wants to (un)hide widget decoration
             show = not isinstance(self._w, sat_widgets.LabelLine)
             self.showDecoration(show)
             self._invalidate()
-        elif key == "meta s": #user wants to (un)hide group's subject or change its apperance
+        elif key == a_key['SUBJECT_SWITCH']: #user wants to (un)hide group's subject or change its apperance
             if self.subject:
                 self.show_title = (self.show_title + 1) % 3
                 if self.show_title == 0:
@@ -131,11 +137,11 @@ class Chat(urwid.WidgetWrap, QuickChat):
         """Return Menu bar"""
         menu = sat_widgets.Menu(self.host.loop)
         if self.type == 'group':
+            self.host.addMenus(menu, C.MENU_ROOM, {'room_jid': self.target.bare})
             game = _("Game")
-            muc = _("MUC")
             menu.addMenu(game, "Tarot", self.onTarotRequest)
-            menu.addMenu(muc, _("Configure room"), self.onConfigureRoom)
         elif self.type == 'one2one':
+            self.host.addMenus(menu, C.MENU_SINGLE, {'jid': unescapePrivate(self.target)})
             menu.addMenu(_("Action"), _("Send file"), self.onSendFileRequest)
         return menu
 
@@ -175,21 +181,32 @@ class Chat(urwid.WidgetWrap, QuickChat):
             main_widget = self.pile
         self._w = main_widget
 
-    def updateChatState(self, state):
-        """Update the chat state of the contact.
-        @param state: new state to set
+    def updateChatState(self, state, nick=None):
+        """Set the chat state (XEP-0085) of the contact. Leave nick to None
+        to set the state for a one2one conversation, or give a nickname or
+        C.ALL_OCCUPANTS to set the state of a participant within a MUC.
+        @param state: the new chat state
+        @param nick: None for one2one, the MUC user nick or C.ALL_OCCUPANTS
         """
-        if (self.type == 'one2one'):
+        if nick:
+            assert(self.type == 'group')
+            occupants = self.occupants if nick == C.ALL_OCCUPANTS else [nick]
+            options = self.present_wid.getAllValues()
+            for index in xrange(0, len(options)):
+                nick = options[index].value
+                if nick in occupants:
+                    options[index] = (nick, '%s %s' % (C.MUC_USER_STATES[state], nick))
+            self.present_wid.changeValues(options)
+            self.host.redraw()
+        else:
+            assert(self.type == 'one2one')
             self.__setSurrendedText(state)
             self.showDecoration()
             self.host.redraw()
-        elif (self.type == 'group'):
-            # TODO: chat state for groupchat
-            pass
 
     def _presentClicked(self, list_wid, clicked_wid):
         assert(self.type == 'group')
-        nick = clicked_wid.getValue()
+        nick = clicked_wid.getValue().value
         if nick == self.getUserNick():
             #We ignore click on our own nick
             return
@@ -197,7 +214,7 @@ class Chat(urwid.WidgetWrap, QuickChat):
         full_jid = JID("%s/%s" % (self.target.bare, nick))
         new_jid = escapePrivate(full_jid)
         if new_jid not in self.host.contact_list:
-            self.host.contact_list.add(new_jid)
+            self.host.contact_list.add(new_jid, [C.GROUP_NOT_IN_ROSTER])
 
         #now we select the new window
         self.host.contact_list.setFocus(full_jid, True)
@@ -250,9 +267,9 @@ class Chat(urwid.WidgetWrap, QuickChat):
         nick = unicode(param_nick) #FIXME: should be done in DBus bridge
         QuickChat.replaceUser(self, nick, show_info)
         presents = self.present_wid.getAllValues()
-        if nick not in presents:
+        if nick not in [present.value for present in presents]:
             presents.append(nick)
-            presents.sort()
+            presents.sort(cmp=lambda a, b: cmp(a.value if hasattr(a, 'value') else a, b.value if hasattr(b, 'value') else b))
             self.present_wid.changeValues(presents)
         self.host.redraw()
 
@@ -263,6 +280,16 @@ class Chat(urwid.WidgetWrap, QuickChat):
         self.present_wid.deleteValue(nick)
         self.host.redraw()
 
+    def clearHistory(self):
+        """Clear the content of this chat."""
+        del self.content[:]
+
+    def afterHistoryPrint(self):
+        """Refresh or scroll down the focus after the history is printed"""
+        if len(self.content):
+            self.text_list.focus_position = len(self.content) - 1  # scroll down
+        self.host.redraw()
+
     def printMessage(self, from_jid, msg, profile, timestamp=""):
         assert isinstance(from_jid, JID)
         try:
@@ -275,7 +302,8 @@ class Chat(urwid.WidgetWrap, QuickChat):
         if timestamp and self.content:
             for idx in range(len(self.content) - 1, -1, -1):
                 current_text = self.content[idx]
-                if new_text.timestamp < current_text.timestamp and idx > 0:
+                older = new_text.timestamp < current_text.timestamp
+                if older and idx > 0:
                     continue  # the new message is older, we need to insert it upper
 
                 #we discard double messages, to avoid backlog / history conflict
@@ -284,11 +312,15 @@ class Chat(urwid.WidgetWrap, QuickChat):
                     (idx < len(self.content) - 2 and self.content[idx + 1].message)):
                     return
 
-                self.content.insert(idx + 1, new_text)
+                self.content.insert(0 if older else idx + 1, new_text)
                 break
         else:
             self.content.append(new_text)
-        self._notify(from_jid, msg)
+        if not timestamp:
+            # XXX: do not send notifications for each line of the history being displayed
+            # FIXME: this must be changed in the future if the timestamp is passed with
+            # all messages and not only with the messages coming from the history.
+            self._notify(from_jid, msg)
 
     def printInfo(self, msg, type_='normal', timestamp=""):
         """Print general info
@@ -311,7 +343,7 @@ class Chat(urwid.WidgetWrap, QuickChat):
         if self.text_list.get_focus()[1] == len(self.content) - 2:
             #we don't change focus if user is not at the bottom
             #as that mean that he is probably watching discussion history
-            self.text_list.set_focus(len(self.content) - 1)
+            self.text_list.focus_position = len(self.content) - 1
         self.host.redraw()
         if not self.host.x_notify.hasFocus():
             if self.type == "one2one":
@@ -333,25 +365,26 @@ class Chat(urwid.WidgetWrap, QuickChat):
 
     #MENU EVENTS#
     def onTarotRequest(self, menu):
+        # TODO: move this to plugin_misc_tarot with dynamic menu
         if len(self.occupants) != 4:
             self.host.showPopUp(sat_widgets.Alert(_("Can't start game"), _("You need to be exactly 4 peoples in the room to start a Tarot game"), ok_cb=self.host.removePopUp))
         else:
             self.host.bridge.tarotGameCreate(self.id, list(self.occupants), self.host.profile)
 
-    def onConfigureRoom(self, menu):
-        def gotUI(xmlui):
-            self.host.addWindow(XMLUI(self.host, xmlui))
-        def configureError(failure):
-            self.host.showPopUp(sat_widgets.Alert(_("Error"), unicode(failure), ok_cb=self.host.removePopUp))
-        self.host.bridge.configureRoom(self.id, self.host.profile, callback=gotUI, errback=configureError)
-
     def onSendFileRequest(self, menu):
+        # TODO: move this to core with dynamic menus
         dialog = FileDialog(ok_cb=self.onFileSelected, cancel_cb=self.host.removePopUp)
         self.host.showPopUp(dialog, 80, 80)
 
     #MISC EVENTS#
     def onFileSelected(self, filepath):
         self.host.removePopUp()
+        try:
+            filepath = filepath.decode('utf-8') # FIXME: correctly manage unicode
+        except UnicodeError:
+            log.error("FIXME: filepath with unicode error are not managed yet")
+            self.host.showDialog(_(u"File has a unicode error in its name, it's not yet managed by SàT"), title=_("Can't send file"), type_="error")
+            return
         #FIXME: check last_resource: what if self.target.resource exists ?
         last_resource = self.host.bridge.getLastResource(unicode(self.target.bare), self.host.profile)
         if last_resource:
@@ -360,3 +393,4 @@ class Chat(urwid.WidgetWrap, QuickChat):
             full_jid = self.target
         progress_id = self.host.bridge.sendFile(full_jid, filepath, {}, self.host.profile)
         self.host.addProgress(progress_id,filepath)
+        self.host.showDialog(_(u"You file request has been sent, we are waiting for your contact answer"), title=_("File request sent"))
