@@ -35,6 +35,7 @@ class Const(constants.Const):
                ('date', 'light gray', 'default'),
                ('my_nick', 'dark red,bold', 'default'),
                ('other_nick', 'dark cyan,bold', 'default'),
+               ('info_msg', 'yellow', 'default', 'bold'),
                ('menubar', 'light gray,bold', 'dark red'),
                ('menubar_focus', 'light gray,bold', 'dark green'),
                ('selected_menu', 'light gray,bold', 'dark green'),
@@ -79,4 +80,8 @@ class Const(constants.Const):
                 "dnd": (u'✖', "show_dnd"),
                 "xa": (u'☄', "show_xa")
                 }
+    LOG_OPT_SECTION = APP_NAME.lower()
+    LOG_OPT_OUTPUT = ('output', constants.Const.LOG_OPT_OUTPUT_SEP + constants.Const.LOG_OPT_OUTPUT_MEMORY)
 
+    CONFIG_SECTION = APP_NAME.lower()
+    CONFIG_OPT_KEY_PREFIX = "KEY_"
