@@ -38,7 +38,7 @@ class Notify(object):
         #Now we try to connect to Freedesktop D-Bus API
         try:
             bus = dbus.SessionBus()
-            db_object = bus.get_object('org.freedesktop.Notifications', '/org/freedesktop/Notifications')
+            db_object = bus.get_object('org.freedesktop.Notifications', '/org/freedesktop/Notifications', follow_name_owner_changes=True)
             self.freedesktop_int = dbus.Interface(db_object, dbus_interface='org.freedesktop.Notifications')
         except:
             self.freedesktop_int = None
