@@ -22,4 +22,4 @@ from sat_frontends import constants
 
 class Const(constants.Const):
 
-    PRIVATE_PREFIX = "@PRIVATE@"
+    PRIVATE_PREFIX = "@private@"
