@@ -17,8 +17,9 @@
 # You should have received a copy of the GNU Affero General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
-from logging import debug, info, error
-from sat.tools.jid  import JID
+from sat.core.log import getLogger
+log = getLogger(__name__)
+from sat_frontends.tools.jid  import JID
 
 
 
@@ -153,7 +154,7 @@ class QuickCardGame(object):
         elif phase == "ecart":
             self.state = "ecart"
         else:
-            error ('INTERNAL ERROR: unmanaged game phase')
+            log.error ('INTERNAL ERROR: unmanaged game phase')
 
         for suit, value in played_cards:
             self.hand.append(self.cards[suit, value])
