@@ -18,14 +18,15 @@
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 from sat.core.i18n import _
-from logging import debug
+from sat.core.log import getLogger
+log = getLogger(__name__)
 
 
 class QuickContactList(object):
     """This class manage the visual representation of contacts"""
 
     def __init__(self):
-        debug(_("Contact List init"))
+        log.debug(_("Contact List init"))
         self._cache = {}
         self.specials={}
 
@@ -54,7 +55,19 @@ class QuickContactList(object):
         self.specials.clear()
 
     def replace(self, jid, groups=None, attributes=None):
-        """add a contact to the list if doesn't exist, else update it"""
+        """Add a contact to the list if doesn't exist, else update it.
+
+        This method can be called with groups=None for the purpose of updating
+        the contact's attributes (e.g. nickname). In that case, the groups
+        attribute must not be set to the default group but ignored. If not,
+        you may move your contact from its actual group(s) to the default one.
+
+        None value for 'groups' has a different meaning than [None] which is for the default group.
+
+        @param jid (JID)
+        @param groups (list): list of groups or None to ignore the groups membership.
+        @param attributes (dict)
+        """
         if attributes and 'name' in attributes:
             self.setCache(jid, 'name', attributes['name'])
 
