@@ -17,7 +17,7 @@
 # You should have received a copy of the GNU Affero General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
-from sat.tools.jid  import JID
+from sat_frontends.tools.jid  import JID
 from os.path import exists, splitext
 from sat_frontends.quick_frontend.constants import Const
 
