@@ -18,14 +18,11 @@
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 from sat.core.i18n import _
-import sys
 import os.path
-import gettext
 import sat_frontends.wix
 from sat_frontends.quick_frontend import constants
 
 
-_ = gettext.gettext
 wix_root = os.path.dirname(sat_frontends.wix.__file__)
 
 
@@ -42,4 +39,4 @@ class Const(constants.Const):
                 ("dnd", _("DND"), "red"),
                 ("xa", _("Away"), "red")
                 ]
-
+    LOG_OPT_SECTION = APP_NAME.lower()
