@@ -19,17 +19,19 @@
 
 
 from sat.core.i18n import _
+from sat_frontends.wix.constants import Const as C
 from sat_frontends.quick_frontend.quick_chat_list import QuickChatList
 from sat_frontends.quick_frontend.quick_app import QuickApp
 import wx
 from sat_frontends.wix.contact_list import ContactList
 from sat_frontends.wix.chat import Chat
-from sat_frontends.wix.xmlui import XMLUI
+from sat_frontends.wix import xmlui
 from sat_frontends.wix.profile import Profile
 from sat_frontends.wix.profile_manager import ProfileManager
 import os.path
-from sat.tools.jid  import JID
-from logging import debug, info, warning, error
+from sat_frontends.tools.jid  import JID
+from sat.core.log import getLogger
+log = getLogger(__name__)
 from sat_frontends.wix.constants import Const
 
 idCONNECT,\
@@ -37,11 +39,9 @@ idDISCONNECT,\
 idEXIT,\
 idABOUT,\
 idPARAM,\
-idADD_CONTACT,\
-idREMOVE_CONTACT,\
 idSHOW_PROFILE,\
 idJOIN_ROOM,\
-= range(9)
+ = range(7)
 
 class ChatList(QuickChatList):
     """This class manage the list of chat windows"""
@@ -84,17 +84,14 @@ class MainWindow(wx.Frame, QuickApp):
         #tray icon
         ticon = wx.Icon(os.path.join(self.media_dir, 'icons/crystal/32/tray_icon.xpm'), wx.BITMAP_TYPE_XPM)
         self.tray_icon = wx.TaskBarIcon()
-        self.tray_icon.SetIcon(ticon, _("Wix jabber client"))
+        if ticon.IsOk():
+            self.tray_icon.SetIcon(ticon, _("Wix jabber client"))
         wx.EVT_TASKBAR_LEFT_UP(self.tray_icon, self.onTrayClick)
 
 
         #events
         self.Bind(wx.EVT_CLOSE, self.onClose, self)
 
-        #menus
-        self.createMenus()
-        for i in range(self.menuBar.GetMenuCount()):
-            self.menuBar.EnableTop(i, False)
 
         #profile panel
         self.profile_pan = ProfileManager(self)
@@ -104,18 +101,49 @@ class MainWindow(wx.Frame, QuickApp):
 
         self.Show()
 
-    def plug_profile(self, profile_key='@DEFAULT@'):
+    def plug_profile_1(self, profile_key='@DEFAULT@'):
         """Hide profile panel then plug profile"""
-        debug (_('plugin profile %s' % profile_key))
+        log.debug (_('plugin profile %s' % profile_key))
         self.profile_pan.Hide()
         self.contact_list.Show()
         self.sizer.Layout()
-        for i in range(self.menuBar.GetMenuCount()):
-            self.menuBar.EnableTop(i, True)
-        super(MainWindow, self).plug_profile(profile_key)
+        super(MainWindow, self).plug_profile_1(profile_key)
+        #menus
+        self.createMenus()
+
+    def addMenus(self, menubar, type_, menu_data=None):
+        """Add cached menus to instance
+        @param menu: wx.MenuBar instance
+        @param type_: menu type like is sat.core.sat_main.importMenu
+        @param menu_data: data to send with these menus
+
+        """
+        menus = self.profiles[self.profile]['menus'].get(type_,[])
+        for id_, path, path_i18n  in menus:
+            if len(path) != 2:
+                raise NotImplementedError("Menu with a path != 2 are not implemented yet")
+            category = path_i18n[0] # TODO: manage path with more than 2 levels
+            name = path_i18n[1]
+            menu_idx = menubar.FindMenu(category)
+            current_menu = None
+            if menu_idx == wx.NOT_FOUND:
+                #the menu is new, we create it
+                current_menu = wx.Menu()
+                menubar.Append(current_menu, category)
+            else:
+                current_menu = menubar.GetMenu(menu_idx)
+            assert(current_menu != None)
+            item_id = wx.NewId()
+            help_string = self.bridge.getMenuHelp(id_, '')
+            current_menu.Append(item_id, name, help=help_string)
+            #now we register the event
+            def event_answer(e, id_=id_):
+                self.launchAction(id_, menu_data, profile_key = self.profile)
+
+            wx.EVT_MENU(menubar.Parent, item_id, event_answer)
 
     def createMenus(self):
-        info(_("Creating menus"))
+        log.info(_("Creating menus"))
         connectMenu = wx.Menu()
         connectMenu.Append(idCONNECT, _("&Connect	CTRL-c"),_(" Connect to the server"))
         connectMenu.Append(idDISCONNECT, _("&Disconnect	CTRL-d"),_(" Disconnect from the server"))
@@ -124,10 +152,6 @@ class MainWindow(wx.Frame, QuickApp):
         connectMenu.Append(idABOUT, _("A&bout"), _(" About %s") % Const.APP_NAME)
         connectMenu.Append(idEXIT,_("E&xit"),_(" Terminate the program"))
         contactMenu = wx.Menu()
-        contactMenu.Append(idADD_CONTACT, _("&Add contact"),_(" Add a contact to your list"))
-        contactMenu.Append(idREMOVE_CONTACT, _("&Remove contact"),_(" Remove the selected contact from your list"))
-        contactMenu.AppendSeparator()
-        contactMenu.Append(idSHOW_PROFILE, _("&Show profile"), _(" Show contact's profile"))
         communicationMenu = wx.Menu()
         communicationMenu.Append(idJOIN_ROOM, _("&Join Room"),_(" Join a Multi-User Chat room"))
         self.menuBar = wx.MenuBar()
@@ -138,31 +162,11 @@ class MainWindow(wx.Frame, QuickApp):
 
         #additionals menus
         #FIXME: do this in a more generic way (in quickapp)
-        add_menus = self.bridge.getMenus('', Const.NO_SECURITY_LIMIT)
-        for id_, type_, path, path_i18n  in add_menus:
-            assert(type_=="NORMAL") #TODO: manage other types
-            if len(path) != 2:
-                raise NotImplementedError("Menu with a path != 2 are not implemented yet")
-            category = path_i18n[0] # TODO: manage path with more than 2 levels
-            name = path_i18n[1]
-            menu_idx = self.menuBar.FindMenu(category)
-            current_menu = None
-            if menu_idx == wx.NOT_FOUND:
-                #the menu is new, we create it
-                current_menu = wx.Menu()
-                self.menuBar.Append(current_menu, category)
-            else:
-                current_menu = self.menuBar.GetMenu(menu_idx)
-            assert(current_menu != None)
-            item_id = wx.NewId()
-            help_string = self.bridge.getMenuHelp(id_, '')
-            current_menu.Append(item_id, name, help=help_string)
-            #now we register the event
-            def event_answer(e, id_=id_):
-                self.launchAction(id_, None, profile_key = self.profile)
-
-            wx.EVT_MENU(self, item_id, event_answer)
+        self.addMenus(self.menuBar, C.MENU_GLOBAL)
 
+        # menu items that should be displayed after the automatically added ones
+        contactMenu.AppendSeparator()
+        contactMenu.Append(idSHOW_PROFILE, _("&Show profile"), _(" Show contact's profile"))
 
         #events
         wx.EVT_MENU(self, idCONNECT, self.onConnectRequest)
@@ -170,13 +174,11 @@ class MainWindow(wx.Frame, QuickApp):
         wx.EVT_MENU(self, idPARAM, self.onParam)
         wx.EVT_MENU(self, idABOUT, self.onAbout)
         wx.EVT_MENU(self, idEXIT, self.onExit)
-        wx.EVT_MENU(self, idADD_CONTACT, self.onAddContact)
-        wx.EVT_MENU(self, idREMOVE_CONTACT, self.onRemoveContact)
         wx.EVT_MENU(self, idSHOW_PROFILE, self.onShowProfile)
         wx.EVT_MENU(self, idJOIN_ROOM, self.onJoinRoom)
 
-    def newMessage(self, from_jid, to_jid, msg, _type, extra, profile):
-        QuickApp.newMessage(self, from_jid, to_jid, msg, _type, extra, profile)
+    def newMessageHandler(self, from_jid, to_jid, msg, _type, extra, profile):
+        QuickApp.newMessageHandler(self, from_jid, to_jid, msg, _type, extra, profile)
 
     def showAlert(self, message):
         # TODO: place this in a separate class
@@ -191,16 +193,16 @@ class MainWindow(wx.Frame, QuickApp):
         popup.Show()
         wx.CallLater(5000,popup.Destroy)
 
-    def showDialog(self, message, title="", type="info", answer_cb = None, answer_data = None):
-        if type == 'info':
+    def showDialog(self, message, title="", type_="info", answer_cb = None, answer_data = None):
+        if type_ == 'info':
             flags = wx.OK | wx.ICON_INFORMATION
-        elif type == 'error':
+        elif type_ == 'error':
             flags = wx.OK | wx.ICON_ERROR
-        elif type == 'yes/no':
+        elif type_ == 'yes/no':
             flags = wx.YES_NO | wx.ICON_QUESTION
         else:
             flags = wx.OK | wx.ICON_INFORMATION
-            error(_('unmanaged dialog type: %s'), type)
+            log.error(_('unmanaged dialog type: %s'), type_)
         dlg = wx.MessageDialog(self, message, title, flags)
         answer = dlg.ShowModal()
         dlg.Destroy()
@@ -241,8 +243,18 @@ class MainWindow(wx.Frame, QuickApp):
                 # action was a one shot, nothing to do
                 pass
             elif "xmlui" in data:
-                debug (_("XML user interface received"))
-                XMLUI(self, xml_data = data['xmlui'])
+                log.debug (_("XML user interface received"))
+                ui = xmlui.create(self, xml_data = data['xmlui'])
+                ui.show()
+            elif "authenticated_profile" in data:
+                assert("caller" in data)
+                if data["caller"] == "profile_manager":
+                    assert(self.profile_pan.IsShown())
+                    self.profile_pan.getXMPPParams(data['authenticated_profile'])
+                elif data["caller"] == "plug_profile":
+                    self.plug_profile_1(data['authenticated_profile'])
+                else:
+                    raise NotImplementedError
             else:
                 dlg = wx.MessageDialog(self, _(u"Unmanaged action result"),
                                        _('Error'),
@@ -251,8 +263,8 @@ class MainWindow(wx.Frame, QuickApp):
                 dlg.ShowModal()
                 dlg.Destroy()
         def action_eb(failure):
-            dlg = wx.MessageDialog(self, unicode(failure),
-                                   _('Error'),
+            dlg = wx.MessageDialog(self, failure.message,
+                                   failure.fullname,
                                    wx.OK | wx.ICON_ERROR
                                   )
             dlg.ShowModal()
@@ -260,14 +272,14 @@ class MainWindow(wx.Frame, QuickApp):
 
         self.bridge.launchAction(callback_id, data, profile_key, callback=action_cb, errback=action_eb)
 
-    def askConfirmation(self, confirmation_id, confirmation_type, data, profile):
+    def askConfirmationHandler(self, confirmation_id, confirmation_type, data, profile):
         #TODO: refactor this in QuickApp
         if not self.check_profile(profile):
             return
-        debug (_("Confirmation asked"))
+        log.debug (_("Confirmation asked"))
         answer_data={}
         if confirmation_type == "FILE_TRANSFER":
-            debug (_("File transfer confirmation asked"))
+            log.debug (_("File transfer confirmation asked"))
             dlg = wx.MessageDialog(self, _("The contact %(jid)s wants to send you the file %(filename)s\nDo you accept ?") % {'jid':data["from"], 'filename':data["filename"]},
                                    _('File Request'),
                                    wx.YES_NO | wx.ICON_QUESTION
@@ -287,7 +299,7 @@ class MainWindow(wx.Frame, QuickApp):
             dlg.Destroy()
 
         elif confirmation_type == "YES/NO":
-            debug (_("Yes/No confirmation asked"))
+            log.debug (_("Yes/No confirmation asked"))
             dlg = wx.MessageDialog(self, data["message"],
                                    _('Confirmation'),
                                    wx.YES_NO | wx.ICON_QUESTION
@@ -300,56 +312,56 @@ class MainWindow(wx.Frame, QuickApp):
 
             dlg.Destroy()
 
-    def actionResult(self, type, id, data, profile):
+    def actionResultHandler(self, type_, id_, data, profile):
         if not self.check_profile(profile):
             return
-        debug (_("actionResult: type = [%(type)s] id = [%(id)s] data = [%(data)s]") % {'type':type, 'id':id, 'data':data})
-        if not id in self.current_action_ids:
-            debug (_('unknown id, ignoring'))
+        log.debug (_("actionResult: type_ = [%(type_)s] id_ = [%(id_)s] data = [%(data)s]") % {'type_':type_, 'id_':id_, 'data':data})
+        if not id_ in self.current_action_ids:
+            log.debug (_('unknown id_, ignoring'))
             return
-        if type == "SUPPRESS":
-            self.current_action_ids.remove(id)
-        elif type == "SUCCESS":
-            self.current_action_ids.remove(id)
+        if type_ == "SUPPRESS":
+            self.current_action_ids.remove(id_)
+        elif type_ == "SUCCESS":
+            self.current_action_ids.remove(id_)
             dlg = wx.MessageDialog(self, data["message"],
                                    _('Success'),
                                    wx.OK | wx.ICON_INFORMATION
                                   )
             dlg.ShowModal()
             dlg.Destroy()
-        elif type == "ERROR":
-            self.current_action_ids.remove(id)
+        elif type_ == "ERROR":
+            self.current_action_ids.remove(id_)
             dlg = wx.MessageDialog(self, data["message"],
                                    _('Error'),
                                    wx.OK | wx.ICON_ERROR
                                   )
             dlg.ShowModal()
             dlg.Destroy()
-        elif type == "XMLUI":
-            self.current_action_ids.remove(id)
-            debug (_("XML user interface received"))
+        elif type_ == "XMLUI":
+            self.current_action_ids.remove(id_)
+            log.debug (_("XML user interface received"))
             misc = {}
             #FIXME FIXME FIXME: must clean all this crap !
             title = _('Form')
-            if data['type'] == _('registration'):
+            if data['type_'] == _('registration'):
                 title = _('Registration')
                 misc['target'] = data['target']
                 misc['action_back'] = self.bridge.gatewayRegister
-            XMLUI(self, title=title, xml_data = data['xml'], misc = misc)
-        elif type == "RESULT":
-            self.current_action_ids.remove(id)
-            if self.current_action_ids_cb.has_key(id):
-                callback = self.current_action_ids_cb[id]
-                del self.current_action_ids_cb[id]
+            xmlui.create(self, title=title, xml_data = data['xml'], misc = misc)
+        elif type_ == "RESULT":
+            self.current_action_ids.remove(id_)
+            if self.current_action_ids_cb.has_key(id_):
+                callback = self.current_action_ids_cb[id_]
+                del self.current_action_ids_cb[id_]
                 callback(data)
-        elif type == "DICT_DICT":
-            self.current_action_ids.remove(id)
-            if self.current_action_ids_cb.has_key(id):
-                callback = self.current_action_ids_cb[id]
-                del self.current_action_ids_cb[id]
+        elif type_ == "DICT_DICT":
+            self.current_action_ids.remove(id_)
+            if self.current_action_ids_cb.has_key(id_):
+                callback = self.current_action_ids_cb[id_]
+                del self.current_action_ids_cb[id_]
                 callback(data)
         else:
-            error (_("FIXME FIXME FIXME: type [%s] not implemented") % type)
+            log.error (_("FIXME FIXME FIXME: type_ [%s] not implemented") % type_)
             raise NotImplementedError
 
 
@@ -379,14 +391,14 @@ class MainWindow(wx.Frame, QuickApp):
     ### events ###
 
     def onContactActivated(self, jid):
-        debug (_("onContactActivated: %s"), jid)
+        log.debug (_("onContactActivated: %s"), jid)
         if self.chat_wins[jid.bare].IsShown():
             self.chat_wins[jid.bare].Hide()
         else:
             self.chat_wins[jid.bare].Show()
 
     def onConnectRequest(self, e):
-        self.bridge.connect(self.profile)
+        QuickApp.asyncConnect(self, self.profile)
 
     def onDisconnectRequest(self, e):
         self.bridge.disconnect(self.profile)
@@ -397,17 +409,17 @@ class MainWindow(wx.Frame, QuickApp):
         self.bridge.setPresence(show=show, statuses={'default': status}, profile_key=self.profile)  #FIXME: manage multilingual statuses
 
     def onStatusChange(self, e):
-        debug(_("Status change request"))
+        log.debug(_("Status change request"))
         self.__updateStatus()
 
     def onParam(self, e):
-        debug(_("Param request"))
+        log.debug(_("Param request"))
         def success(params):
-            XMLUI(self, xml_data=params, title=_("Configuration"))
+            xmlui.create(self, xml_data=params, title=_("Configuration"))
 
         def failure(error):
-            dlg = wx.MessageDialog(self, unicode(error),
-                                   _('Error'),
+            dlg = wx.MessageDialog(self, error.message,
+                                   error.fullname,
                                    wx.OK | wx.ICON_ERROR
                                   )
             dlg.ShowModal()
@@ -434,47 +446,8 @@ class MainWindow(wx.Frame, QuickApp):
     def onExit(self, e):
         self.Close()
 
-    def onAddContact(self, e):
-        debug(_("Add contact request"))
-        dlg = wx.TextEntryDialog(
-                self, _('Please enter new contact JID'),
-                _('Adding a contact'), _('name@server.tld'))
-
-        if dlg.ShowModal() == wx.ID_OK:
-            jid=JID(dlg.GetValue())
-            if jid.is_valid():
-                self.bridge.addContact(jid.bare, profile_key=self.profile)
-            else:
-                error (_("'%s' is an invalid JID !"), jid)
-                #TODO: notice the user
-
-        dlg.Destroy()
-
-    def onRemoveContact(self, e):
-        debug(_("Remove contact request"))
-        target = self.contact_list.getSelection()
-        if not target:
-            dlg = wx.MessageDialog(self, _("You haven't selected any contact !"),
-                                   _('Error'),
-                                   wx.OK | wx.ICON_ERROR
-                                  )
-            dlg.ShowModal()
-            dlg.Destroy()
-            return
-
-        dlg = wx.MessageDialog(self, _("Are you sure you want to delete %s from your roster list ?") % target.bare,
-                               _('Contact suppression'),
-                               wx.YES_NO | wx.ICON_QUESTION
-                              )
-
-        if dlg.ShowModal() == wx.ID_YES:
-            info(_("Unsubscribing %s presence"), target.bare)
-            self.bridge.delContact(target.bare, profile_key=self.profile)
-
-        dlg.Destroy()
-
     def onShowProfile(self, e):
-        debug(_("Show contact's profile request"))
+        log.debug(_("Show contact's profile request"))
         target = self.contact_list.getSelection()
         if not target:
             dlg = wx.MessageDialog(self, _("You haven't selected any contact !"),
@@ -484,17 +457,17 @@ class MainWindow(wx.Frame, QuickApp):
             dlg.ShowModal()
             dlg.Destroy()
             return
-        _id = self.bridge.getCard(target.bare, profile_key=self.profile)
+        _id = self.bridge.getCard(target.bare, self.profile)
         self.current_action_ids.add(_id)
         self.current_action_ids_cb[_id] = self.onProfileReceived
 
     def onProfileReceived(self, data):
         """Called when a profile is received"""
-        debug (_('Profile received: [%s]') % data)
-        profile=Profile(self, data)
+        log.debug (_('Profile received: [%s]') % data)
+        Profile(self, data)
 
     def onJoinRoom(self, e):
-        warning('FIXME: temporary menu, must be improved')
+        log.warning('FIXME: temporary menu, must be improved')
         #TODO: a proper MUC room joining dialog with nickname etc
         dlg = wx.TextEntryDialog(
                 self, _("Please enter MUC's JID"),
@@ -505,30 +478,21 @@ class MainWindow(wx.Frame, QuickApp):
             if room_jid.is_valid():
                 self.bridge.joinMUC(room_jid, self.profiles[self.profile]['whoami'].node, {}, self.profile)
             else:
-                error (_("'%s' is an invalid JID !"), room_jid)
+                log.error (_("'%s' is an invalid JID !"), room_jid)
 
     def onClose(self, e):
         QuickApp.onExit(self)
-        info(_("Exiting..."))
+        log.info(_("Exiting..."))
         for win in self.chat_wins:
             self.chat_wins[win].Destroy()
         self.tray_icon.Destroy()
         e.Skip()
 
     def onTrayClick(self, e):
-        debug(_("Tray Click"))
+        log.debug(_("Tray Click"))
         if self.IsShown():
             self.Hide()
         else:
             self.Show()
             self.Raise()
         e.Skip()
-
-    def chatStateReceived(self, from_jid_s, state, profile):
-        """Signal observer to display a contact chat state
-        @param from_jid_s: contact who sent his new state
-        @state: state
-        @profile: current profile
-        """
-        print "TODO: chatStateReceived not implemented yet"
-        pass
