@@ -20,11 +20,13 @@
 
 
 from sat.core.i18n import _
+from sat_frontends.primitivus.constants import Const as C
 import wx
-import pdb
-from logging import debug, info, error
-from sat.tools.jid  import JID
-import pdb
+from sat.core.log import getLogger
+log = getLogger(__name__)
+
+
+NO_SELECTION_ENTRY = ' '
 
 
 class ProfileManager(wx.Panel):
@@ -36,7 +38,7 @@ class ProfileManager(wx.Panel):
         self.sizer = wx.BoxSizer(wx.VERTICAL)
         self.SetSizer(self.sizer)
 
-        profiles = self.host.bridge.getProfilesList()
+        self.selected_profile = NO_SELECTION_ENTRY  # allow to reselect the previous selection until the profile is authenticated
         self.profile_name = wx.ComboBox(self, -1, style=wx.CB_READONLY|wx.CB_SORT)
         self.__refillProfiles()
         self.Bind(wx.EVT_COMBOBOX, self.onProfileChange)
@@ -75,9 +77,8 @@ class ProfileManager(wx.Panel):
         #Now we can set the default value
         self.__setDefault()
 
-
     def __setDefault(self):
-        profile_default = self.host.bridge.getProfileName("@DEFAULT@")
+        profile_default = NO_SELECTION_ENTRY if self.host.options.profile else self.host.bridge.getProfileName("@DEFAULT@")
         if profile_default:
             self.profile_name.SetValue(profile_default)
             self.onProfileChange(None)
@@ -87,10 +88,10 @@ class ProfileManager(wx.Panel):
         self.profile_name.Clear()
         profiles = self.host.bridge.getProfilesList()
         profiles.sort()
+        self.profile_name.Append(NO_SELECTION_ENTRY)
         for profile in profiles:
             self.profile_name.Append(profile)
 
-
     def onNewProfile(self, event):
         dlg = wx.TextEntryDialog(self, _("Please enter the new profile name"), _("New profile"), style = wx.OK | wx.CANCEL)
         if dlg.ShowModal() == wx.ID_OK:
@@ -99,9 +100,12 @@ class ProfileManager(wx.Panel):
                 if name[0]=='@':
                     wx.MessageDialog(self, _("A profile name can't start with a @"), _("Bad profile name"), wx.ICON_ERROR).ShowModal()
                 else:
-                    profile = self.host.bridge.createProfile(name)
-                    self.__refillProfiles()
-                    self.profile_name.SetValue(name)
+                    def cb():
+                        self.__refillProfiles()
+                        self.profile_name.SetValue(name)
+                        self.selected_profile = name
+                        self.getXMPPParams(name)
+                    self.host.bridge.asyncCreateProfile(name, callback=cb)
         dlg.Destroy()
 
     def onDeleteProfile(self, event):
@@ -110,24 +114,46 @@ class ProfileManager(wx.Panel):
             return
         dlg = wx.MessageDialog(self, _("Are you sure to delete the profile [%s]") % name, _("Confirmation"), wx.ICON_QUESTION | wx.YES_NO)
         if dlg.ShowModal() == wx.ID_YES:
-            self.host.bridge.deleteProfile(name)
-            self.__refillProfiles()
-            self.__setDefault()
+            def cb():
+                self.__refillProfiles()
+                self.__setDefault()
+            self.host.bridge.asyncDeleteProfile(name, callback=cb)
         dlg.Destroy()
 
+    def getXMPPParams(self, profile):
+        """This is called from MainWindow.launchAction when the profile has been authenticated.
+
+        @param profile: %(doc_profile)s
+        """
+        def setJID(jabberID):
+            self.login_jid.SetValue(jabberID)
+
+        def setPassword(password):
+            self.login_pass.SetValue(password)
+
+        self.profile_name.SetValue(profile)
+        self.selected_profile = profile
+        self.host.bridge.asyncGetParamA("JabberID", "Connection", profile_key=profile, callback=setJID, errback=self.getParamError)
+        self.host.bridge.asyncGetParamA("Password", "Connection", profile_key=profile, callback=setPassword, errback=self.getParamError)
+
     def onProfileChange(self, event):
         """Called when a profile is choosen in the combo box"""
-        def setJID(jabberID):
-            self.login_jid.SetValue(jabberID)
-        def setPassword(password):
-            self.login_pass.SetValue(password)
-        self.host.bridge.asyncGetParamA("JabberID", "Connection", profile_key=self.profile_name.GetValue(), callback=setJID, errback=self.getParamError)
-        self.host.bridge.asyncGetParamA("Password", "Connection", profile_key=self.profile_name.GetValue(), callback=setPassword, errback=self.getParamError)
+        profile_name = self.profile_name.GetValue()
+        if not profile_name or profile_name == self.selected_profile:
+            return  # avoid infinite loop
+        if profile_name == NO_SELECTION_ENTRY:
+            self.selected_profile = NO_SELECTION_ENTRY
+            return
+        if self.selected_profile:
+            self.profile_name.SetValue(self.selected_profile)
+        self.host.profile = profile_name  # FIXME: EXTREMELY DIRTY, needed for sat_frontends.tools.xmlui.XMLUI.submit
+        self.host.launchAction(C.AUTHENTICATE_PROFILE_ID, {'caller': 'profile_manager'}, profile_key=profile_name)
 
     def onConnectButton(self, event):
         """Called when the Connect button is pressed"""
         name = self.profile_name.GetValue()
-        if not name:
+        assert(name == self.selected_profile)  # if not, there's a bug somewhere...
+        if not name or name == NO_SELECTION_ENTRY:
             wx.MessageDialog(self, _("You must select a profile or create a new one before connecting"), _("No profile selected"), wx.ICON_ERROR).ShowModal()
             return
         if name[0]=='@':
@@ -145,11 +171,10 @@ class ProfileManager(wx.Panel):
         new_jid = self.login_jid.GetValue()
         new_pass = self.login_pass.GetValue()
         if old_jid != new_jid:
-            debug(_('Saving new JID and server'))
+            log.debug(_('Saving new JID and server'))
             self.host.bridge.setParam("JabberID", new_jid, "Connection", profile_key=profile)
-            self.host.bridge.setParam("Server", JID(new_jid).domain, "Connection", profile_key=profile)
         if old_pass != new_pass:
-            debug(_('Saving new password'))
+            log.debug(_('Saving new password'))
             self.host.bridge.setParam("Password", new_pass, "Connection", profile_key=profile)
         self.host.plug_profile(profile)
 
