@@ -17,12 +17,14 @@
 # You should have received a copy of the GNU Affero General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
+from sat.core.i18n import _
 from bridge import Bridge
 import dbus
 import dbus.service
 import dbus.mainloop.glib
 import inspect
-from logging import debug, info, error
+from sat.core.log import getLogger
+log = getLogger(__name__)
 from twisted.internet.defer import Deferred
 from sat.core.exceptions import BridgeInitError
 
@@ -55,21 +57,31 @@ class DeferredNotAsync(dbus.DBusException):
 
 class GenericException(dbus.DBusException):
     def __init__(self, twisted_error):
+        """
+
+        @param twisted_error (Failure): instance of twisted Failure
+        @return: DBusException
+        """
         super(GenericException, self).__init__()
         try:
-            error_name = str(twisted_error.value().__class__.__name__)
             # twisted_error.value is a class
+            class_ = twisted_error.value().__class__
         except TypeError:
             # twisted_error.value is an instance
-            error_name = str(twisted_error.value.__class__.__name__)
-        self._dbus_error_name = const_ERROR_PREFIX + "." + error_name
+            class_ = twisted_error.value.__class__
+            message = twisted_error.getErrorMessage()
+            try:
+                self.args = (message, twisted_error.value.condition)
+            except AttributeError:
+                self.args = (message,)
+        self._dbus_error_name = '.'.join([const_ERROR_PREFIX, class_.__module__, class_.__name__])
 
 
 class DbusObject(dbus.service.Object):
 
     def __init__(self, bus, path):
         dbus.service.Object.__init__(self, bus, path)
-        debug("Init DbusObject...")
+        log.debug("Init DbusObject...")
         self.cb = {}
 
     def register(self, name, cb):
@@ -84,7 +96,7 @@ class DbusObject(dbus.service.Object):
         if "callback" in kwargs:
             #we must have errback too
             if not "errback" in kwargs:
-                error("errback is missing in method call [%s]" % name)
+                log.error("errback is missing in method call [%s]" % name)
                 raise InternalError
             callback = kwargs.pop("callback")
             errback = kwargs.pop("errback")
@@ -94,13 +106,13 @@ class DbusObject(dbus.service.Object):
         result = self.cb[name](*args, **kwargs)
         if async:
             if not isinstance(result, Deferred):
-                error("Asynchronous method [%s] does not return a Deferred." % name)
+                log.error("Asynchronous method [%s] does not return a Deferred." % name)
                 raise AsyncNotDeferred
             result.addCallback(lambda result: callback() if result is None else callback(result))
             result.addErrback(lambda err: errback(GenericException(err)))
         else:
             if isinstance(result, Deferred):
-                error("Synchronous method [%s] return a Deferred." % name)
+                log.error("Synchronous method [%s] return a Deferred." % name)
                 raise DeferredNotAsync
             return result
     ### signals ###
@@ -133,11 +145,6 @@ class DbusObject(dbus.service.Object):
     def connected(self, profile):
         pass
 
-    @dbus.service.signal(const_INT_PREFIX+const_CORE_SUFFIX,
-                         signature='ss')
-    def connectionError(self, error_type, profile):
-        pass
-
     @dbus.service.signal(const_INT_PREFIX+const_CORE_SUFFIX,
                          signature='ss')
     def contactDeleted(self, entity_jid, profile):
@@ -192,16 +199,22 @@ class DbusObject(dbus.service.Object):
         return self._callback("addContact", unicode(entity_jid), unicode(profile_key))
 
     @dbus.service.method(const_INT_PREFIX+const_CORE_SUFFIX,
-                         in_signature='s', out_signature='',
+                         in_signature='ss', out_signature='b',
                          async_callbacks=('callback', 'errback'))
-    def asyncConnect(self, profile_key="@DEFAULT@", callback=None, errback=None):
-        return self._callback("asyncConnect", unicode(profile_key), callback=callback, errback=errback)
+    def asyncConnect(self, profile_key="@DEFAULT@", password='', callback=None, errback=None):
+        return self._callback("asyncConnect", unicode(profile_key), unicode(password), callback=callback, errback=errback)
+
+    @dbus.service.method(const_INT_PREFIX+const_CORE_SUFFIX,
+                         in_signature='ss', out_signature='',
+                         async_callbacks=('callback', 'errback'))
+    def asyncCreateProfile(self, profile, password='', callback=None, errback=None):
+        return self._callback("asyncCreateProfile", unicode(profile), unicode(password), callback=callback, errback=errback)
 
     @dbus.service.method(const_INT_PREFIX+const_CORE_SUFFIX,
                          in_signature='s', out_signature='',
                          async_callbacks=('callback', 'errback'))
-    def asyncCreateProfile(self, profile, callback=None, errback=None):
-        return self._callback("asyncCreateProfile", unicode(profile), callback=callback, errback=errback)
+    def asyncDeleteProfile(self, profile, callback=None, errback=None):
+        return self._callback("asyncDeleteProfile", unicode(profile), callback=callback, errback=errback)
 
     @dbus.service.method(const_INT_PREFIX+const_CORE_SUFFIX,
                          in_signature='sssis', out_signature='s',
@@ -215,18 +228,6 @@ class DbusObject(dbus.service.Object):
     def confirmationAnswer(self, id, accepted, data, profile):
         return self._callback("confirmationAnswer", unicode(id), accepted, data, unicode(profile))
 
-    @dbus.service.method(const_INT_PREFIX+const_CORE_SUFFIX,
-                         in_signature='s', out_signature='',
-                         async_callbacks=None)
-    def connect(self, profile_key="@DEFAULT@"):
-        return self._callback("connect", unicode(profile_key))
-
-    @dbus.service.method(const_INT_PREFIX+const_CORE_SUFFIX,
-                         in_signature='s', out_signature='i',
-                         async_callbacks=None)
-    def createProfile(self, profile):
-        return self._callback("createProfile", unicode(profile))
-
     @dbus.service.method(const_INT_PREFIX+const_CORE_SUFFIX,
                          in_signature='ss', out_signature='',
                          async_callbacks=None)
@@ -234,10 +235,16 @@ class DbusObject(dbus.service.Object):
         return self._callback("delContact", unicode(entity_jid), unicode(profile_key))
 
     @dbus.service.method(const_INT_PREFIX+const_CORE_SUFFIX,
-                         in_signature='s', out_signature='i',
-                         async_callbacks=None)
-    def deleteProfile(self, profile):
-        return self._callback("deleteProfile", unicode(profile))
+                         in_signature='ss', out_signature='(asa(sss))',
+                         async_callbacks=('callback', 'errback'))
+    def discoInfos(self, entity_jid, profile_key, callback=None, errback=None):
+        return self._callback("discoInfos", unicode(entity_jid), unicode(profile_key), callback=callback, errback=errback)
+
+    @dbus.service.method(const_INT_PREFIX+const_CORE_SUFFIX,
+                         in_signature='ss', out_signature='a(sss)',
+                         async_callbacks=('callback', 'errback'))
+    def discoItems(self, entity_jid, profile_key, callback=None, errback=None):
+        return self._callback("discoItems", unicode(entity_jid), unicode(profile_key), callback=callback, errback=errback)
 
     @dbus.service.method(const_INT_PREFIX+const_CORE_SUFFIX,
                          in_signature='s', out_signature='',
@@ -326,8 +333,8 @@ class DbusObject(dbus.service.Object):
     @dbus.service.method(const_INT_PREFIX+const_CORE_SUFFIX,
                          in_signature='s', out_signature='a{sa{s(sia{ss})}}',
                          async_callbacks=None)
-    def getPresenceStatus(self, profile_key="@DEFAULT@"):
-        return self._callback("getPresenceStatus", unicode(profile_key))
+    def getPresenceStatuses(self, profile_key="@DEFAULT@"):
+        return self._callback("getPresenceStatuses", unicode(profile_key))
 
     @dbus.service.method(const_INT_PREFIX+const_CORE_SUFFIX,
                          in_signature='s', out_signature='s',
@@ -347,6 +354,12 @@ class DbusObject(dbus.service.Object):
     def getProgress(self, id, profile):
         return self._callback("getProgress", unicode(id), unicode(profile))
 
+    @dbus.service.method(const_INT_PREFIX+const_CORE_SUFFIX,
+                         in_signature='', out_signature='',
+                         async_callbacks=('callback', 'errback'))
+    def getReady(self, callback=None, errback=None):
+        return self._callback("getReady", callback=callback, errback=errback)
+
     @dbus.service.method(const_INT_PREFIX+const_CORE_SUFFIX,
                          in_signature='', out_signature='s',
                          async_callbacks=None)
@@ -377,6 +390,12 @@ class DbusObject(dbus.service.Object):
     def launchAction(self, callback_id, data, profile_key="@DEFAULT@", callback=None, errback=None):
         return self._callback("launchAction", unicode(callback_id), data, unicode(profile_key), callback=callback, errback=errback)
 
+    @dbus.service.method(const_INT_PREFIX+const_CORE_SUFFIX,
+                         in_signature='s', out_signature='b',
+                         async_callbacks=None)
+    def loadParamsTemplate(self, filename):
+        return self._callback("loadParamsTemplate", unicode(filename))
+
     @dbus.service.method(const_INT_PREFIX+const_CORE_SUFFIX,
                          in_signature='sis', out_signature='',
                          async_callbacks=None)
@@ -384,10 +403,10 @@ class DbusObject(dbus.service.Object):
         return self._callback("paramsRegisterApp", unicode(xml), security_limit, unicode(app))
 
     @dbus.service.method(const_INT_PREFIX+const_CORE_SUFFIX,
-                         in_signature='ssssi', out_signature='s',
+                         in_signature='s', out_signature='b',
                          async_callbacks=None)
-    def registerNewAccount(self, login, password, email, host, port=5222):
-        return self._callback("registerNewAccount", unicode(login), unicode(password), unicode(email), unicode(host), port)
+    def saveParamsTemplate(self, filename):
+        return self._callback("saveParamsTemplate", unicode(filename))
 
     @dbus.service.method(const_INT_PREFIX+const_CORE_SUFFIX,
                          in_signature='ssssa{ss}s', out_signature='',
@@ -402,10 +421,10 @@ class DbusObject(dbus.service.Object):
         return self._callback("setParam", unicode(name), unicode(value), unicode(category), security_limit, unicode(profile_key))
 
     @dbus.service.method(const_INT_PREFIX+const_CORE_SUFFIX,
-                         in_signature='ssia{ss}s', out_signature='',
+                         in_signature='ssa{ss}s', out_signature='',
                          async_callbacks=None)
-    def setPresence(self, to_jid='', show='', priority=0, statuses={}, profile_key="@DEFAULT@"):
-        return self._callback("setPresence", unicode(to_jid), unicode(show), priority, statuses, unicode(profile_key))
+    def setPresence(self, to_jid='', show='', statuses={}, profile_key="@DEFAULT@"):
+        return self._callback("setPresence", unicode(to_jid), unicode(show), statuses, unicode(profile_key))
 
     @dbus.service.method(const_INT_PREFIX+const_CORE_SUFFIX,
                          in_signature='sss', out_signature='',
@@ -496,7 +515,7 @@ class DbusObject(dbus.service.Object):
         attributes = ', '.join(self.__attributes(signature))
         #TODO: use doc parameter to name attributes
 
-        #code = compile ('def '+name+' (self,'+attributes+'): debug ("'+name+' signal")', '<DBus bridge>','exec') #XXX: the debug is too annoying with xmllog
+        #code = compile ('def '+name+' (self,'+attributes+'): log.debug ("'+name+' signal")', '<DBus bridge>','exec') #XXX: the log.debug is too annoying with xmllog
         code = compile('def ' + name + ' (self,' + attributes + '): pass', '<DBus bridge>', 'exec')
         exec (code)
         signal = locals()[name]
@@ -511,12 +530,12 @@ class DBusBridge(Bridge):
     def __init__(self):
         dbus.mainloop.glib.DBusGMainLoop(set_as_default=True)
         Bridge.__init__(self)
-        info("Init DBus...")
+        log.info("Init DBus...")
         try:
             self.session_bus = dbus.SessionBus()
         except dbus.DBusException as e:
             if e._dbus_error_name == 'org.freedesktop.DBus.Error.NotSupported':
-                print u"D-Bus is not launched, please see README to see instructions on how to launch it"
+                log.error(_(u"D-Bus is not launched, please see README to see instructions on how to launch it"))
             raise BridgeInitError
         self.dbus_name = dbus.service.BusName(const_INT_PREFIX, self.session_bus)
         self.dbus_bridge = DbusObject(self.session_bus, const_OBJ_PATH)
@@ -533,9 +552,6 @@ class DBusBridge(Bridge):
     def connected(self, profile):
         self.dbus_bridge.connected(profile)
 
-    def connectionError(self, error_type, profile):
-        self.dbus_bridge.connectionError(error_type, profile)
-
     def contactDeleted(self, entity_jid, profile):
         self.dbus_bridge.contactDeleted(entity_jid, profile)
 
@@ -564,13 +580,13 @@ class DBusBridge(Bridge):
         self.dbus_bridge.subscribe(sub_type, entity_jid, profile)
 
     def register(self, name, callback):
-        debug("registering DBus bridge method [%s]", name)
+        log.debug("registering DBus bridge method [%s]" % name)
         self.dbus_bridge.register(name, callback)
 
     def addMethod(self, name, int_suffix, in_sign, out_sign, method, async=False, doc={}):
         """Dynamically add a method to Dbus Bridge"""
         #FIXME: doc parameter is kept only temporary, the time to remove it from calls
-        print ("Adding method [%s] to DBus bridge" % name)
+        log.debug("Adding method [%s] to DBus bridge" % name)
         self.dbus_bridge.addMethod(name, int_suffix, in_sign, out_sign, method, async)
         self.register(name, method)
 
