@@ -18,19 +18,45 @@
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 from sat.core.i18n import _
-from bridge_frontend import BridgeFrontend
+from bridge_frontend import BridgeFrontend, BridgeException
 import dbus
-from logging import debug, error
+from sat.core.log import getLogger
+log = getLogger(__name__)
 from sat.core.exceptions import BridgeExceptionNoService, BridgeInitError
 
 from dbus.mainloop.glib import DBusGMainLoop
 DBusGMainLoop(set_as_default=True)
 
+import ast
+
 const_INT_PREFIX = "org.goffi.SAT"  # Interface prefix
 const_ERROR_PREFIX = const_INT_PREFIX + ".error"
 const_OBJ_PATH = '/org/goffi/SAT/bridge'
 const_CORE_SUFFIX = ".core"
 const_PLUGIN_SUFFIX = ".plugin"
+const_TIMEOUT = 120
+
+
+def dbus_to_bridge_exception(dbus_e):
+    """Convert a DBusException to a BridgeException.
+
+    @param dbus_e (DBusException)
+    @return: BridgeException
+    """
+    full_name = dbus_e.get_dbus_name()
+    if full_name.startswith(const_ERROR_PREFIX):
+        name = dbus_e.get_dbus_name()[len(const_ERROR_PREFIX) + 1:]
+    else:
+        name = full_name
+    # XXX: dbus_e.args doesn't contain the original DBusException args, but we
+    # receive its serialized form in dbus_e.args[0]. From that we can rebuild
+    # the original arguments list thanks to ast.literal_eval (secure eval).
+    message = dbus_e.get_dbus_message()  # similar to dbus_e.args[0]
+    try:
+        message, condition = ast.literal_eval(message)
+    except (SyntaxError, ValueError, TypeError):
+        condition = ''
+    return BridgeException(name, message, condition)
 
 
 class DBusBridgeFrontend(BridgeFrontend):
@@ -47,7 +73,7 @@ class DBusBridgeFrontend(BridgeFrontend):
             if e._dbus_error_name == 'org.freedesktop.DBus.Error.ServiceUnknown':
                 raise BridgeExceptionNoService
             elif e._dbus_error_name == 'org.freedesktop.DBus.Error.NotSupported':
-                print u"D-Bus is not launched, please see README to see instructions on how to launch it"
+                log.error(_(u"D-Bus is not launched, please see README to see instructions on how to launch it"))
                 raise BridgeInitError
             else:
                 raise e
@@ -59,7 +85,7 @@ class DBusBridgeFrontend(BridgeFrontend):
         elif iface == "plugin":
             self.db_plugin_iface.connect_to_signal(functionName, handler)
         else:
-            error(_('Unknown interface'))
+            log.error(_('Unknown interface'))
 
     def __getattribute__(self, name):
         """ usual __getattribute__ if the method exists, else try to find a plugin method """
@@ -89,8 +115,9 @@ class DBusBridgeFrontend(BridgeFrontend):
                 method = getattr(self.db_plugin_iface, name)
 
                 if async:
+                    kwargs['timeout'] = const_TIMEOUT
                     kwargs['reply_handler'] = _callback
-                    kwargs['error_handler'] = lambda err: _errback(err._dbus_error_name[len(const_ERROR_PREFIX) + 1:])
+                    kwargs['error_handler'] = lambda err: _errback(dbus_to_bridge_exception(err))
 
                 return method(*args, **kwargs)
 
