@@ -18,6 +18,9 @@
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 
+from sat.core.log import getLogger
+log = getLogger(__name__)
+
 try:
 
     import gettext
@@ -32,7 +35,7 @@ try:
 
 except ImportError:
 
-    print "WARNING: gettext support disabled"
+    log.warning("gettext support disabled")
     _ = lambda msg: msg # Libervia doesn't support gettext
     def languageSwitch(lang=None):
         pass
