@@ -18,10 +18,12 @@
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 from sat.core.i18n import _
+from sat.core.constants import Const as C
 from twisted.internet import task, defer
 from twisted.words.protocols.jabber import jid, xmlstream
 from wokkel import client, disco, xmppim, generic, compat, delay, iwokkel
-from logging import debug, info, warning, error
+from sat.core.log import getLogger
+log = getLogger(__name__)
 from sat.core import exceptions
 from calendar import timegm
 from zope.interface import implements
@@ -34,13 +36,14 @@ except ImportError:
 class SatXMPPClient(client.XMPPClient):
     implements(iwokkel.IDisco)
 
-    def __init__(self, host_app, profile, user_jid, password, host=None, port=5222):
-        client.XMPPClient.__init__(self, user_jid, password, host, port)
+    def __init__(self, host_app, profile, user_jid, password, host=None, port=C.XMPP_C2S_PORT):
+        # XXX: DNS SRV records are checked when the host is not specified.
+        # If no SRV record is found, the host is directly extracted from the JID.
+        client.XMPPClient.__init__(self, user_jid, password, host or None, port or C.XMPP_C2S_PORT)
         self.factory.clientConnectionLost = self.connectionLost
         self.__connected = False
         self.profile = profile
         self.host_app = host_app
-        self.client_initialized = defer.Deferred()
         self.conn_deferred = defer.Deferred()
         self._waiting_conf = {}  # callback called when a confirmation is received
         self._progress_cb_map = {}  # callback called when a progress is requested (key = progress id)
@@ -54,13 +57,13 @@ class SatXMPPClient(client.XMPPClient):
             return
         client.XMPPClient._authd(self, xmlstream)
         self.__connected = True
-        info(_("********** [%s] CONNECTED **********") % self.profile)
+        log.info(_("********** [%s] CONNECTED **********") % self.profile)
         self.streamInitialized()
         self.host_app.bridge.connected(self.profile)  # we send the signal to the clients
 
     def streamInitialized(self):
         """Called after _authd"""
-        debug(_("XML stream is initialized"))
+        log.debug(_("XML stream is initialized"))
         self.keep_alife = task.LoopingCall(self.xmlstream.send, " ")  # Needed to avoid disconnection (specially with openfire)
         self.keep_alife.start(180)
 
@@ -68,46 +71,40 @@ class SatXMPPClient(client.XMPPClient):
         self.disco.setHandlerParent(self)
         self.discoHandler = disco.DiscoHandler()
         self.discoHandler.setHandlerParent(self)
+        disco_d = defer.succeed(None)
 
-        if not self.host_app.trigger.point("Disco Handled", self.profile):
+        if not self.host_app.trigger.point("Disco handled", disco_d, self.profile):
             return
 
-        self.roster.requestRoster()
+        def finish_connection(dummy):
+            self.roster.requestRoster()
+            self.presence.available()
+            self.conn_deferred.callback(None)
 
-        self.presence.available()
-
-        jid_ = self.getHostJid()
-        self.disco.requestInfo(jid_).addCallback(self.host_app.serverDisco, jid_, self.profile)  # FIXME: use these informations
-
-        self.disco.requestItems(jid_).addCallback(self.host_app.serverDiscoItems, self.disco, jid_, self.profile, self.client_initialized)
-        self.conn_deferred.callback(None)
+        disco_d.addCallback(finish_connection)
 
     def initializationFailed(self, reason):
-        print "initializationFailed: %s" % reason
-        self.host_app.bridge.connectionError("AUTH_ERROR", self.profile)
+        log.error(_("ERROR: XMPP connection failed for profile '%(profile)s': %(reason)s" % {'profile': self.profile, 'reason': reason}))
+        self.conn_deferred.errback(reason.value)
         try:
             client.XMPPClient.initializationFailed(self, reason)
         except:
-            # we already send an error signal, no need to raise an exception
+            # we already chained an errback, no need to raise an exception
             pass
-        self.conn_deferred.errback()
 
     def isConnected(self):
         return self.__connected
 
     def connectionLost(self, connector, unused_reason):
-        self.__connected = False
-        info(_("********** [%s] DISCONNECTED **********") % self.profile)
         try:
             self.keep_alife.stop()
         except AttributeError:
-            debug(_("No keep_alife"))
-        self.host_app.bridge.disconnected(self.profile)  # we send the signal to the clients
-        self.host_app.purgeClient(self.profile)  # and we remove references to this client
-
-    def getHostJid(self):
-        """@return: the jid of the local server"""
-        return jid.JID(self.jid.host)
+            log.debug(_("No keep_alife"))
+        if self.__connected:
+            log.info(_("********** [%s] DISCONNECTED **********") % self.profile)
+            self.host_app.bridge.disconnected(self.profile)  # we send the signal to the clients
+            self.host_app.purgeClient(self.profile)  # and we remove references to this client
+        self.__connected = False
 
 
 class SatMessageProtocol(xmppim.MessageProtocol):
@@ -117,7 +114,7 @@ class SatMessageProtocol(xmppim.MessageProtocol):
         self.host = host
 
     def onMessage(self, message):
-        debug(_(u"got message from: %s"), message["from"])
+        log.debug(_(u"got message from: %s") % message["from"])
         post_treat = defer.Deferred() # XXX: plugin can add their treatments to this deferred
 
         if not self.host.trigger.point("MessageReceived", message, post_treat, profile=self.parent.profile):
@@ -131,11 +128,17 @@ class SatMessageProtocol(xmppim.MessageProtocol):
         for e in message.elements():
             if e.name == "body":
                 data['body'] = e.children[0] if e.children else ""
-                break
+            elif e.name == "subject" and e.children:
+                data['extra']['subject'] = e.children[0]
 
         data['type'] = message['type'] if message.hasAttribute('type') else 'normal'
 
-        def after_treatments(data):
+        def bridgeSignal(data):
+            if data is not None:
+                self.host.bridge.newMessage(data['from'], data['body'], data['type'], data['to'], data['extra'], profile=self.parent.profile)
+            return data
+
+        def addToHistory(data):
             # set message body to empty string by default, and later
             # also forward message without body (chat state notification...)
             try:
@@ -147,9 +150,20 @@ class SatMessageProtocol(xmppim.MessageProtocol):
                     self.host.memory.addToHistory(jid.JID(data['from']), jid.JID(data['to']), data['body'], data['type'], data['extra'], timestamp, profile=self.parent.profile)
             except IndexError:
                 self.host.memory.addToHistory(jid.JID(data['from']), jid.JID(data['to']), data['body'], data['type'], data['extra'], profile=self.parent.profile)
-            self.host.bridge.newMessage(data['from'], data['body'], data['type'], data['to'], data['extra'], profile=self.parent.profile)
+            return data
 
-        post_treat.addCallback(after_treatments)
+        def treatmentsEb(failure):
+            failure.trap(exceptions.SkipHistory)
+            return data
+
+        def cancelErrorTrap(failure):
+            """A message sending can be cancelled by a plugin treatment"""
+            failure.trap(exceptions.CancelError)
+
+        post_treat.addCallback(addToHistory)
+        post_treat.addErrback(treatmentsEb)
+        post_treat.addCallback(bridgeSignal)
+        post_treat.addErrback(cancelErrorTrap)
         post_treat.callback(data)
 
 
@@ -169,7 +183,7 @@ class SatRosterProtocol(xmppim.RosterClientProtocol):
 
     def requestRoster(self):
         """ ask the server for Roster list """
-        debug("requestRoster")
+        log.debug("requestRoster")
         d = self.getRoster().addCallback(self.rosterCb)
         d.chainDeferred(self.got_roster)
 
@@ -223,7 +237,11 @@ class SatRosterProtocol(xmppim.RosterClientProtocol):
             # may change in the future
             self.removeItem(item.jid)
             return
-        info(_("new contact in roster list: %s"), item.jid.full())
+        log.info(_("new contact in roster list: %s") % item.jid.full())
+        if not item.subscriptionTo:
+            log.warning(_("You are not subscribed to this contact !"))
+        if not item.subscriptionFrom:
+            log.warning(_("This contact is not subscribed to you !"))
         #self.host.memory.addContact(item.jid, item_attr, item.groups, self.parent.profile)
 
         bare_jid = item.jid.userhostJID()
@@ -241,7 +259,7 @@ class SatRosterProtocol(xmppim.RosterClientProtocol):
         try:
             item = self._jids.pop(bare_jid)
         except KeyError:
-            warning("Received a roster remove event for an item not in cache")
+            log.warning("Received a roster remove event for an item not in cache")
             return
         for group in item.groups:
             try:
@@ -250,7 +268,7 @@ class SatRosterProtocol(xmppim.RosterClientProtocol):
                 if not jids_set:
                     del self._groups[group]
             except KeyError:
-                warning("there is not cache for the group [%(groups)s] of the removed roster item [%(jid)s]" %
+                log.warning("there is not cache for the group [%(groups)s] of the removed roster item [%(jid)s]" %
                         {"group": group, "jid": bare_jid})
 
         # then we send the bridge signal
@@ -291,8 +309,13 @@ class SatPresenceProtocol(xmppim.PresenceClientProtocol):
         xmppim.PresenceClientProtocol.__init__(self)
         self.host = host
 
+    def send(self, obj):
+        if not self.host.trigger.point("Presence send", self.parent, obj):
+            return
+        super(SatPresenceProtocol, self).send(obj)
+
     def availableReceived(self, entity, show=None, statuses=None, priority=0):
-        debug(_("presence update for [%(entity)s] (available, show=%(show)s statuses=%(statuses)s priority=%(priority)d)") % {'entity': entity, 'show': show, 'statuses': statuses, 'priority': priority})
+        log.debug(_("presence update for [%(entity)s] (available, show=%(show)s statuses=%(statuses)s priority=%(priority)d)") % {'entity': entity, 'show': show, 'statuses': statuses, 'priority': priority})
 
         if not statuses:
             statuses = {}
@@ -305,13 +328,17 @@ class SatPresenceProtocol(xmppim.PresenceClientProtocol):
                                            int(priority), statuses,
                                            self.parent.profile)
 
+        # uncomment these two lines if you need the trigger
+        #if not self.host.trigger.point("presenceReceived", entity, "unavailable", 0, statuses, self.parent.profile):
+        #    return
+
         # now it's time to notify frontends
         self.host.bridge.presenceUpdate(entity.full(), show or "",
                                         int(priority), statuses,
                                         self.parent.profile)
 
     def unavailableReceived(self, entity, statuses=None):
-        debug(_("presence update for [%(entity)s] (unavailable, statuses=%(statuses)s)") % {'entity': entity, 'statuses': statuses})
+        log.debug(_("presence update for [%(entity)s] (unavailable, statuses=%(statuses)s)") % {'entity': entity, 'statuses': statuses})
 
         if not statuses:
             statuses = {}
@@ -321,6 +348,9 @@ class SatPresenceProtocol(xmppim.PresenceClientProtocol):
             del statuses[None]
         self.host.memory.setPresenceStatus(entity, "unavailable", 0, statuses, self.parent.profile)
 
+        if not self.host.trigger.point("presenceReceived", entity, "unavailable", 0, statuses, self.parent.profile):
+            return
+
         # now it's time to notify frontends
         self.host.bridge.presenceUpdate(entity.full(), "unavailable", 0, statuses, self.parent.profile)
 
@@ -338,23 +368,25 @@ class SatPresenceProtocol(xmppim.PresenceClientProtocol):
         # so we must temporarily switch to wokkel's convention...
         if 'default' in statuses:
             statuses[None] = statuses['default']
+            del statuses['default']
 
-        xmppim.PresenceClientProtocol.available(self, entity, show, statuses, priority)
         presence_elt = xmppim.AvailablePresence(entity, show, statuses, priority)
-        if not self.host.trigger.point("presence_available", presence_elt, self.parent):
-            return
-        self.send(presence_elt)
 
         # ... before switching back
         if None in statuses:
+            statuses['default'] = statuses[None]
             del statuses[None]
 
+        if not self.host.trigger.point("presence_available", presence_elt, self.parent):
+            return
+        self.send(presence_elt)
+
     def subscribed(self, entity):
         xmppim.PresenceClientProtocol.subscribed(self, entity)
         self.host.memory.delWaitingSub(entity.userhost(), self.parent.profile)
         item = self.parent.roster.getItem(entity)
         if not item or not item.subscriptionTo:  # we automatically subscribe to 'to' presence
-            debug(_('sending automatic "from" subscription request'))
+            log.debug(_('sending automatic "from" subscription request'))
             self.subscribe(entity)
 
     def unsubscribed(self, entity):
@@ -362,29 +394,29 @@ class SatPresenceProtocol(xmppim.PresenceClientProtocol):
         self.host.memory.delWaitingSub(entity.userhost(), self.parent.profile)
 
     def subscribedReceived(self, entity):
-        debug(_("subscription approved for [%s]") % entity.userhost())
+        log.debug(_("subscription approved for [%s]") % entity.userhost())
         self.host.bridge.subscribe('subscribed', entity.userhost(), self.parent.profile)
 
     def unsubscribedReceived(self, entity):
-        debug(_("unsubscription confirmed for [%s]") % entity.userhost())
+        log.debug(_("unsubscription confirmed for [%s]") % entity.userhost())
         self.host.bridge.subscribe('unsubscribed', entity.userhost(), self.parent.profile)
 
     def subscribeReceived(self, entity):
-        debug(_("subscription request from [%s]") % entity.userhost())
+        log.debug(_("subscription request from [%s]") % entity.userhost())
         item = self.parent.roster.getItem(entity)
         if item and item.subscriptionTo:
             # We automatically accept subscription if we are already subscribed to contact presence
-            debug(_('sending automatic subscription acceptance'))
+            log.debug(_('sending automatic subscription acceptance'))
             self.subscribed(entity)
         else:
             self.host.memory.addWaitingSub('subscribe', entity.userhost(), self.parent.profile)
             self.host.bridge.subscribe('subscribe', entity.userhost(), self.parent.profile)
 
     def unsubscribeReceived(self, entity):
-        debug(_("unsubscription asked for [%s]") % entity.userhost())
+        log.debug(_("unsubscription asked for [%s]") % entity.userhost())
         item = self.parent.roster.getItem(entity)
         if item and item.subscriptionFrom:  # we automatically remove contact
-            debug(_('automatic contact deletion'))
+            log.debug(_('automatic contact deletion'))
             self.host.delContact(entity, self.parent.profile)
         self.host.bridge.subscribe('unsubscribe', entity.userhost(), self.parent.profile)
 
@@ -401,26 +433,25 @@ class SatFallbackHandler(generic.FallbackHandler):
     def iqFallback(self, iq):
         if iq.handled is True:
             return
-        debug(u"iqFallback: xml = [%s]" % (iq.toXml()))
+        log.debug(u"iqFallback: xml = [%s]" % (iq.toXml()))
         generic.FallbackHandler.iqFallback(self, iq)
 
 
 class RegisteringAuthenticator(xmlstream.ConnectAuthenticator):
 
-    def __init__(self, host, jabber_host, user_login, user_pass, email, answer_id, profile):
+    def __init__(self, host, jabber_host, user_login, user_pass, email, deferred, profile):
         xmlstream.ConnectAuthenticator.__init__(self, jabber_host)
         self.host = host
         self.jabber_host = jabber_host
         self.user_login = user_login
         self.user_pass = user_pass
         self.user_email = email
-        self.answer_id = answer_id
+        self.deferred = deferred
         self.profile = profile
-        print _("Registration asked for"), user_login, user_pass, jabber_host
+        log.debug(_("Registration asked for %(user)s@%(host)s") % {'user': user_login, 'host': jabber_host})
 
     def connectionMade(self):
-        print "connectionMade"
-
+        log.debug(_("Connection made with %s" % self.jabber_host))
         self.xmlstream.namespace = "jabber:client"
         self.xmlstream.sendHeader()
 
@@ -434,27 +465,17 @@ class RegisteringAuthenticator(xmlstream.ConnectAuthenticator):
         if self.user_email:
             _email = query.addElement('email')
             _email.addContent(self.user_email)
-        reg = iq.send(self.jabber_host).addCallbacks(self.registrationAnswer, self.registrationFailure)
+        d = iq.send(self.jabber_host).addCallbacks(self.registrationAnswer, self.registrationFailure)
+        d.chainDeferred(self.deferred)
 
     def registrationAnswer(self, answer):
-        debug(_("registration answer: %s") % answer.toXml())
-        answer_type = "SUCCESS"
-        answer_data = {"message": _("Registration successfull")}
-        self.host.bridge.actionResult(answer_type, self.answer_id, answer_data, self.profile)
+        log.debug(_("Registration answer: %s") % answer.toXml())
         self.xmlstream.sendFooter()
 
     def registrationFailure(self, failure):
-        info(_("Registration failure: %s") % str(failure.value))
-        answer_type = "ERROR"
-        answer_data = {}
-        if failure.value.condition == 'conflict':
-            answer_data['reason'] = 'conflict'
-            answer_data = {"message": _("Username already exists, please choose an other one")}
-        else:
-            answer_data['reason'] = 'unknown'
-            answer_data = {"message": _("Registration failed (%s)") % str(failure.value.condition)}
-        self.host.bridge.actionResult(answer_type, self.answer_id, answer_data, self.profile)
+        log.info(_("Registration failure: %s") % str(failure.value))
         self.xmlstream.sendFooter()
+        raise failure.value
 
 
 class SatVersionHandler(generic.VersionHandler):
@@ -475,7 +496,7 @@ class SatIdentityHandler(XMPPHandler):
     implements(iwokkel.IDisco)
 
     def getDiscoInfo(self, requestor, target, nodeIdentifier=''):
-        return [disco.DiscoIdentity(u"client", u"pc", u"Salut à Toi")]
+        return [disco.DiscoIdentity(u"client", u"pc", C.APP_NAME)]
 
     def getDiscoItems(self, requestor, target, nodeIdentifier=''):
         return []
