@@ -18,7 +18,8 @@
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 from sat.core.i18n import _
-from logging import debug, info, warning, error
+from sat.core.log import getLogger
+log = getLogger(__name__)
 
 
 class MemoryNotInitializedError(Exception):
@@ -35,19 +36,22 @@ class PersistentDict(object):
         """@param namespace: unique namespace for this dictionary
         @param profile: profile which *MUST* exists, or None for general values"""
         if not self.storage:
-            error(_("PersistentDict can't be used before memory initialisation"))
+            log.error(_("PersistentDict can't be used before memory initialisation"))
             raise MemoryNotInitializedError
         self._cache = {}
         self.namespace = namespace
         self.profile = profile
 
     def load(self):
-        """load persistent data from storage
+        """Load persistent data from storage.
+
+        @return: defers the PersistentDict instance itself
         """
         if not self.profile:
-            return self.storage.loadGenPrivates(self._cache, self.namespace)
+            d = self.storage.loadGenPrivates(self._cache, self.namespace)
         else:
-            return self.storage.loadIndPrivates(self._cache, self.namespace, self.profile)
+            d = self.storage.loadIndPrivates(self._cache, self.namespace, self.profile)
+        return d.addCallback(lambda dummy: self)
 
     def __repr__(self):
         return self._cache.__repr__()
@@ -85,6 +89,9 @@ class PersistentDict(object):
     def __contains__(self, key):
         return self._cache.__contains__(key)
 
+    def __iter__(self):
+        return self._cache.__iter__()
+
     def __getitem__(self, key):
         return self._cache.__getitem__(key)
 
