@@ -18,8 +18,12 @@
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 from sat.core.i18n import _
+from sat.core.constants import Const as C
 from sat.core import exceptions
-from logging import debug, info, warning, error
+from sat.core.log import getLogger
+log = getLogger(__name__)
+from sat.memory.crypto import BlockCipher, PasswordHasher
+from sat.tools.config import fixConfigOption
 from twisted.enterprise import adbapi
 from twisted.internet import defer
 from collections import OrderedDict
@@ -29,7 +33,7 @@ import os.path
 import cPickle as pickle
 import hashlib
 
-CURRENT_DB_VERSION = 1
+CURRENT_DB_VERSION = 2
 
 # XXX: DATABASE schemas are used in the following way:
 #      - 'current' key is for the actual database schema, for a new base
@@ -38,7 +42,7 @@ CURRENT_DB_VERSION = 1
 #      a 'current' data dict can contains the keys:
 #      - 'CREATE': it contains an Ordered dict with table to create as keys, and a len 2 tuple as value, where value[0] are the columns definitions and value[1] are the table constraints
 #      - 'INSERT': it contains an Ordered dict with table where values have to be inserted, and many tuples containing values to insert in the order of the rows (#TODO: manage named columns)
-#      an update data dict (the ones with a number) can contains the keys 'create', 'delete', 'cols create', 'cols delete', 'cols modify' or 'insert'. See Updater.generateUpdateData for more infos. This metho can be used to autogenerate update_data, to ease the work of the developers.
+#      an update data dict (the ones with a number) can contains the keys 'create', 'delete', 'cols create', 'cols delete', 'cols modify', 'insert' or 'specific'. See Updater.generateUpdateData for more infos. This method can be used to autogenerate update_data, to ease the work of the developers.
 
 DATABASE_SCHEMAS = {
         "current": {'CREATE': OrderedDict((
@@ -58,7 +62,6 @@ DATABASE_SCHEMAS = {
                                                    ("PRIMARY KEY (namespace, key, profile_id)", "FOREIGN KEY(profile_id) REFERENCES profiles(id) ON DELETE CASCADE"))),
                               ('private_gen_bin', (("namespace TEXT", "key TEXT", "value BLOB"),
                                                    ("PRIMARY KEY (namespace, key)",))),
-
                               ('private_ind_bin', (("namespace TEXT", "key TEXT", "profile_id INTEGER", "value BLOB"),
                                                    ("PRIMARY KEY (namespace, key, profile_id)", "FOREIGN KEY(profile_id) REFERENCES profiles(id) ON DELETE CASCADE")))
                               )),
@@ -66,6 +69,8 @@ DATABASE_SCHEMAS = {
                               ('message_types', (("'chat'",), ("'error'",), ("'groupchat'",), ("'headline'",), ("'normal'",))),
                               )),
                     },
+        2:         {'specific': 'update2raw_v2'
+                   },
         1:         {'cols create': {'history': ('extra BLOB',)}
                    },
         }
@@ -80,15 +85,22 @@ class SqliteStorage(object):
         self.initialized = defer.Deferred()  # triggered when memory is fully initialised and ready
         self.profiles = {}  # we keep cache for the profiles (key: profile name, value: profile id)
 
-        info(_("Connecting database"))
+        log.info(_("Connecting database"))
         new_base = not os.path.exists(db_filename)  # do we have to create the database ?
+        if new_base:  # the dir may not exist if it's not the XDG recommended one
+            dir_ = os.path.dirname(db_filename)
+            if not os.path.exists(dir_):
+                os.makedirs(dir_, 0700)
         self.dbpool = adbapi.ConnectionPool("sqlite3", db_filename, check_same_thread=False)
 
         # init_defer is the initialisation deferred, initialisation is ok when all its callbacks have been done
-        init_defer = self.dbpool.runOperation("PRAGMA foreign_keys = ON").addErrback(lambda x: error(_("Can't activate foreign keys")))
+        # XXX: foreign_keys activation doesn't seem to work, probably because of the multi-threading
+        # All the requests that need to use this feature should be run with runInteraction instead,
+        # so you can set the PRAGMA as it is done in self.deleteProfile
+        init_defer = self.dbpool.runOperation("PRAGMA foreign_keys = ON").addErrback(lambda x: log.error(_("Can't activate foreign keys")))
 
         def getNewBaseSql():
-            info(_("The database is new, creating the tables"))
+            log.info(_("The database is new, creating the tables"))
             database_creation = ["PRAGMA user_version=%d" % CURRENT_DB_VERSION]
             database_creation.extend(Updater.createData2Raw(DATABASE_SCHEMAS['current']['CREATE']))
             database_creation.extend(Updater.insertData2Raw(DATABASE_SCHEMAS['current']['INSERT']))
@@ -102,7 +114,7 @@ class SqliteStorage(object):
 
             if statements is None:
                 return defer.succeed(None)
-            debug("===== COMMITING STATEMENTS =====\n%s\n============\n\n" % '\n'.join(statements))
+            log.debug("===== COMMITING STATEMENTS =====\n%s\n============\n\n" % '\n'.join(statements))
             d = self.dbpool.runInteraction(self._updateDb, tuple(statements))
             return d
 
@@ -119,7 +131,6 @@ class SqliteStorage(object):
         for statement in statements:
             interaction.execute(statement)
 
-
     #Profiles
     def _profilesCache(self, profiles_result):
         """Fill the profiles cache
@@ -159,12 +170,17 @@ class SqliteStorage(object):
         @param name: name of the profile
         @return: deferred triggered once profile is actually deleted"""
         def deletionError(failure):
-            error(_("Can't delete profile [%s]") % name)
+            log.error(_("Can't delete profile [%s]") % name)
             return failure
 
-        del self.profiles[name]
-        d = self.dbpool.runQuery("DELETE FROM profiles WHERE name = ?", (name, ))
-        d.addCallback(lambda ignore: info(_("Profile [%s] deleted") % name))
+        def delete(txn):
+            del self.profiles[name]
+            txn.execute("PRAGMA foreign_keys = ON")
+            txn.execute("DELETE FROM profiles WHERE name = ?", (name,))
+            return None
+
+        d = self.dbpool.runInteraction(delete)
+        d.addCallback(lambda ignore: log.info(_("Profile [%s] deleted") % name))
         d.addErrback(deletionError)
         return d
 
@@ -178,7 +194,7 @@ class SqliteStorage(object):
             for param in result:
                 category, name, value = param
                 params_gen[(category, name)] = value
-        debug(_("loading general parameters from database"))
+        log.debug(_("loading general parameters from database"))
         return self.dbpool.runQuery("SELECT category,name,value FROM param_gen").addCallback(fillParams)
 
     def loadIndParams(self, params_ind, profile):
@@ -191,7 +207,7 @@ class SqliteStorage(object):
             for param in result:
                 category, name, value = param
                 params_ind[(category, name)] = value
-        debug(_("loading individual parameters from database"))
+        log.debug(_("loading individual parameters from database"))
         d = self.dbpool.runQuery("SELECT category,name,value FROM param_ind WHERE profile_id=?", (self.profiles[profile], ))
         d.addCallback(fillParams)
         return d
@@ -213,7 +229,7 @@ class SqliteStorage(object):
         @param value: value to set
         @return: deferred"""
         d = self.dbpool.runQuery("REPLACE INTO param_gen(category,name,value) VALUES (?,?,?)", (category, name, value))
-        d.addErrback(lambda ignore: error(_("Can't set general parameter (%(category)s/%(name)s) in database" % {"category": category, "name": name})))
+        d.addErrback(lambda ignore: log.error(_("Can't set general parameter (%(category)s/%(name)s) in database" % {"category": category, "name": name})))
         return d
 
     def setIndParam(self, category, name, value, profile):
@@ -224,7 +240,7 @@ class SqliteStorage(object):
         @param profile: a profile which *must* exist
         @return: deferred"""
         d = self.dbpool.runQuery("REPLACE INTO param_ind(category,name,profile_id,value) VALUES (?,?,?,?)", (category, name, self.profiles[profile], value))
-        d.addErrback(lambda ignore: error(_("Can't set individual parameter (%(category)s/%(name)s) for [%(profile)s] in database" % {"category": category, "name": name, "profile": profile})))
+        d.addErrback(lambda ignore: log.error(_("Can't set individual parameter (%(category)s/%(name)s) for [%(profile)s] in database" % {"category": category, "name": name, "profile": profile})))
         return d
 
     #History
@@ -244,7 +260,7 @@ class SqliteStorage(object):
         d = self.dbpool.runQuery("INSERT INTO history(source, source_res, dest, dest_res, timestamp, message, type, extra, profile_id) VALUES (?,?,?,?,?,?,?,?,?)",
                                  (from_jid.userhost(), from_jid.resource, to_jid.userhost(), to_jid.resource, timestamp or time(),
                                   message, _type, extra_, self.profiles[profile]))
-        d.addErrback(lambda ignore: error(_("Can't save following message in history: from [%(from_jid)s] to [%(to_jid)s] ==> [%(message)s]" %
+        d.addErrback(lambda ignore: log.error(_("Can't save following message in history: from [%(from_jid)s] to [%(to_jid)s] ==> [%(message)s]" %
                                           {"from_jid": from_jid.full(), "to_jid": to_jid.full(), "message": message})))
         return d
 
@@ -309,9 +325,9 @@ class SqliteStorage(object):
             for private in result:
                 key, value = private
                 private_gen[key] = value
-        debug(_("loading general private values [namespace: %s] from database") % (namespace, ))
+        log.debug(_("loading general private values [namespace: %s] from database") % (namespace, ))
         d = self.dbpool.runQuery("SELECT key,value FROM private_gen WHERE namespace=?", (namespace, )).addCallback(fillPrivates)
-        return d.addErrback(lambda x: debug(_("No data present in database for namespace %s") % namespace))
+        return d.addErrback(lambda x: log.debug(_("No data present in database for namespace %s") % namespace))
 
     def loadIndPrivates(self, private_ind, namespace, profile):
         """Load individual private values
@@ -324,10 +340,10 @@ class SqliteStorage(object):
             for private in result:
                 key, value = private
                 private_ind[key] = value
-        debug(_("loading individual private values [namespace: %s] from database") % (namespace, ))
+        log.debug(_("loading individual private values [namespace: %s] from database") % (namespace, ))
         d = self.dbpool.runQuery("SELECT key,value FROM private_ind WHERE namespace=? AND profile_id=?", (namespace, self.profiles[profile]))
         d.addCallback(fillPrivates)
-        return d.addErrback(lambda x: debug(_("No data present in database for namespace %s") % namespace))
+        return d.addErrback(lambda x: log.debug(_("No data present in database for namespace %s") % namespace))
 
     def setGenPrivate(self, namespace, key, value):
         """Save the general private value in database
@@ -336,7 +352,7 @@ class SqliteStorage(object):
         @param value: value to set
         @return: deferred"""
         d = self.dbpool.runQuery("REPLACE INTO private_gen(namespace,key,value) VALUES (?,?,?)", (namespace, key, value))
-        d.addErrback(lambda ignore: error(_("Can't set general private value (%(key)s) [namespace:%(namespace)s] in database" %
+        d.addErrback(lambda ignore: log.error(_("Can't set general private value (%(key)s) [namespace:%(namespace)s] in database" %
                      {"namespace": namespace, "key": key})))
         return d
 
@@ -348,7 +364,7 @@ class SqliteStorage(object):
         @param profile: a profile which *must* exist
         @return: deferred"""
         d = self.dbpool.runQuery("REPLACE INTO private_ind(namespace,key,profile_id,value) VALUES (?,?,?,?)", (namespace, key, self.profiles[profile], value))
-        d.addErrback(lambda ignore: error(_("Can't set individual private value (%(key)s) [namespace: %(namespace)s] for [%(profile)s] in database" %
+        d.addErrback(lambda ignore: log.error(_("Can't set individual private value (%(key)s) [namespace: %(namespace)s] for [%(profile)s] in database" %
                      {"namespace": namespace, "key": key, "profile": profile})))
         return d
 
@@ -358,7 +374,7 @@ class SqliteStorage(object):
         @param key: key of the private value
         @return: deferred"""
         d = self.dbpool.runQuery("DELETE FROM private_gen WHERE namespace=? AND key=?", (namespace, key))
-        d.addErrback(lambda ignore: error(_("Can't delete general private value (%(key)s) [namespace:%(namespace)s] in database" %
+        d.addErrback(lambda ignore: log.error(_("Can't delete general private value (%(key)s) [namespace:%(namespace)s] in database" %
                      {"namespace": namespace, "key": key})))
         return d
 
@@ -369,7 +385,7 @@ class SqliteStorage(object):
         @param profile: a profile which *must* exist
         @return: deferred"""
         d = self.dbpool.runQuery("DELETE FROM private_ind WHERE namespace=? AND key=? AND profile=?)", (namespace, key, self.profiles[profile]))
-        d.addErrback(lambda ignore: error(_("Can't delete individual private value (%(key)s) [namespace: %(namespace)s] for [%(profile)s] in database" %
+        d.addErrback(lambda ignore: log.error(_("Can't delete individual private value (%(key)s) [namespace: %(namespace)s] for [%(profile)s] in database" %
                      {"namespace": namespace, "key": key, "profile": profile})))
         return d
 
@@ -383,9 +399,9 @@ class SqliteStorage(object):
             for private in result:
                 key, value = private
                 private_gen[key] = pickle.loads(str(value))
-        debug(_("loading general private binary values [namespace: %s] from database") % (namespace, ))
+        log.debug(_("loading general private binary values [namespace: %s] from database") % (namespace, ))
         d = self.dbpool.runQuery("SELECT key,value FROM private_gen_bin WHERE namespace=?", (namespace, )).addCallback(fillPrivates)
-        return d.addErrback(lambda x: debug(_("No binary data present in database for namespace %s") % namespace))
+        return d.addErrback(lambda x: log.debug(_("No binary data present in database for namespace %s") % namespace))
 
     def loadIndPrivatesBinary(self, private_ind, namespace, profile):
         """Load individual private binary values
@@ -398,10 +414,10 @@ class SqliteStorage(object):
             for private in result:
                 key, value = private
                 private_ind[key] = pickle.loads(str(value))
-        debug(_("loading individual private binary values [namespace: %s] from database") % (namespace, ))
+        log.debug(_("loading individual private binary values [namespace: %s] from database") % (namespace, ))
         d = self.dbpool.runQuery("SELECT key,value FROM private_ind_bin WHERE namespace=? AND profile_id=?", (namespace, self.profiles[profile]))
         d.addCallback(fillPrivates)
-        return d.addErrback(lambda x: debug(_("No binary data present in database for namespace %s") % namespace))
+        return d.addErrback(lambda x: log.debug(_("No binary data present in database for namespace %s") % namespace))
 
     def setGenPrivateBinary(self, namespace, key, value):
         """Save the general private binary value in database
@@ -410,7 +426,7 @@ class SqliteStorage(object):
         @param value: value to set
         @return: deferred"""
         d = self.dbpool.runQuery("REPLACE INTO private_gen_bin(namespace,key,value) VALUES (?,?,?)", (namespace, key, pickle.dumps(value, 0)))
-        d.addErrback(lambda ignore: error(_("Can't set general private binary value (%(key)s) [namespace:%(namespace)s] in database" %
+        d.addErrback(lambda ignore: log.error(_("Can't set general private binary value (%(key)s) [namespace:%(namespace)s] in database" %
                      {"namespace": namespace, "key": key})))
         return d
 
@@ -422,7 +438,7 @@ class SqliteStorage(object):
         @param profile: a profile which *must* exist
         @return: deferred"""
         d = self.dbpool.runQuery("REPLACE INTO private_ind_bin(namespace,key,profile_id,value) VALUES (?,?,?,?)", (namespace, key, self.profiles[profile], pickle.dumps(value, 0)))
-        d.addErrback(lambda ignore: error(_("Can't set individual binary private value (%(key)s) [namespace: %(namespace)s] for [%(profile)s] in database" %
+        d.addErrback(lambda ignore: log.error(_("Can't set individual binary private value (%(key)s) [namespace: %(namespace)s] for [%(profile)s] in database" %
                      {"namespace": namespace, "key": key, "profile": profile})))
         return d
 
@@ -432,7 +448,7 @@ class SqliteStorage(object):
         @param key: key of the private value
         @return: deferred"""
         d = self.dbpool.runQuery("DELETE FROM private_gen_bin WHERE namespace=? AND key=?", (namespace, key))
-        d.addErrback(lambda ignore: error(_("Can't delete general private binary value (%(key)s) [namespace:%(namespace)s] in database" %
+        d.addErrback(lambda ignore: log.error(_("Can't delete general private binary value (%(key)s) [namespace:%(namespace)s] in database" %
                      {"namespace": namespace, "key": key})))
         return d
 
@@ -443,7 +459,7 @@ class SqliteStorage(object):
         @param profile: a profile which *must* exist
         @return: deferred"""
         d = self.dbpool.runQuery("DELETE FROM private_ind_bin WHERE namespace=? AND key=? AND profile=?)", (namespace, key, self.profiles[profile]))
-        d.addErrback(lambda ignore: error(_("Can't delete individual private binary value (%(key)s) [namespace: %(namespace)s] for [%(profile)s] in database" %
+        d.addErrback(lambda ignore: log.error(_("Can't delete individual private binary value (%(key)s) [namespace: %(namespace)s] for [%(profile)s] in database" %
                      {"namespace": namespace, "key": key, "profile": profile})))
         return d
     ##Helper methods##
@@ -496,7 +512,7 @@ class Updater(object):
 
     @defer.inlineCallbacks
     def checkUpdates(self):
-        """ Check is database schema update is needed, according to DATABASE_SCHEMAS
+        """ Check if a database schema/content update is needed, according to DATABASE_SCHEMAS
         @return: deferred which fire a list of SQL update statements, or None if no update is needed
 
         """
@@ -507,9 +523,13 @@ class Updater(object):
         local_hash = self.statementHash(local_sch)
         current_hash = self.statementHash(current_sch)
 
-        if local_hash == current_hash:
+        # Force the update if the schemas are unchanged but a specific update is needed
+        force_update = local_hash == current_hash and local_version < CURRENT_DB_VERSION \
+                        and 'specific' in DATABASE_SCHEMAS[CURRENT_DB_VERSION]
+
+        if local_hash == current_hash and not force_update:
             if local_version != CURRENT_DB_VERSION:
-                warning(_("Your local schema is up-to-date, but database versions mismatch, fixing it..."))
+                log.warning(_("Your local schema is up-to-date, but database versions mismatch, fixing it..."))
                 yield self._setLocalVersion(CURRENT_DB_VERSION)
         else:
             # an update is needed
@@ -519,24 +539,29 @@ class Updater(object):
                 if self._sat_version.endswith('D'):
                     # we are in a development version
                     update_data = self.generateUpdateData(local_sch, current_sch, False)
-                    warning(_("There is a schema mismatch, but as we are on a dev version, database will be updated"))
-                    update_raw = self.update2raw(update_data, True)
+                    log.warning(_("There is a schema mismatch, but as we are on a dev version, database will be updated"))
+                    update_raw = yield self.update2raw(update_data, True)
                     defer.returnValue(update_raw)
                 else:
-                    error(_(u"schema version is up-to-date, but local schema differ from expected current schema"))
+                    log.error(_(u"schema version is up-to-date, but local schema differ from expected current schema"))
                     update_data = self.generateUpdateData(local_sch, current_sch, True)
-                    warning(_(u"Here are the commands that should fix the situation, use at your own risk (do a backup before modifying database), you can go to SàT's MUC room at sat@chat.jabberfr.org for help\n### SQL###\n%s\n### END SQL ###\n") % u'\n'.join(("%s;" % statement for statement in self.update2raw(update_data))))
+                    update_raw = yield self.update2raw(update_data)
+                    log.warning(_(u"Here are the commands that should fix the situation, use at your own risk (do a backup before modifying database), you can go to SàT's MUC room at sat@chat.jabberfr.org for help\n### SQL###\n%s\n### END SQL ###\n") % u'\n'.join("%s;" % statement for statement in update_raw))
                     raise exceptions.DatabaseError("Database mismatch")
             else:
                 # Database is not up-to-date, we'll do the update
-                info(_("Database schema has changed, local database will be updated"))
+                if force_update:
+                    log.info(_("Database content needs a specific processing, local database will be updated"))
+                else:
+                    log.info(_("Database schema has changed, local database will be updated"))
                 update_raw = []
-                for version in xrange(local_version+1, CURRENT_DB_VERSION+1):
+                for version in xrange(local_version + 1, CURRENT_DB_VERSION + 1):
                     try:
                         update_data = DATABASE_SCHEMAS[version]
                     except KeyError:
                         raise exceptions.InternalError("Missing update definition (version %d)" % version)
-                    update_raw.extend(self.update2raw(update_data))
+                    update_raw_step = yield self.update2raw(update_data)
+                    update_raw.extend(update_raw_step)
                 update_raw.append("PRAGMA user_version=%d" % CURRENT_DB_VERSION)
                 defer.returnValue(update_raw)
 
@@ -600,12 +625,12 @@ class Updater(object):
         schema_dict = {}
         for create_statement in raw_statements:
             if not create_statement.startswith("CREATE TABLE "):
-                warning("Unexpected statement, ignoring it")
+                log.warning("Unexpected statement, ignoring it")
                 continue
             _create_statement = create_statement[13:]
             table, raw_col_stats = _create_statement.split(' ',1)
             if raw_col_stats[0] != '(' or raw_col_stats[-1] != ')':
-                warning("Unexpected statement structure, ignoring it")
+                log.warning("Unexpected statement structure, ignoring it")
                 continue
             col_stats = [stmt.strip() for stmt in self.stmnt_regex.findall(raw_col_stats[1:-1])]
             col_defs = []
@@ -656,7 +681,7 @@ class Updater(object):
             new_col_defs, new_constraints = new_data[table]
             for obj in old_col_defs, old_constraints, new_col_defs, new_constraints:
                 if not isinstance(obj, tuple):
-                    raise InternalError("Columns definitions must be tuples")
+                    raise exceptions.InternalError("Columns definitions must be tuples")
             defs_create, defs_delete, ignore = getChanges(set(old_col_defs), set(new_col_defs))
             constraints_create, constraints_delete, ignore = getChanges(set(old_constraints), set(new_constraints))
             created_col_names = set([name.split(' ',1)[0] for name in defs_create])
@@ -681,6 +706,7 @@ class Updater(object):
                 'cols modify': modify_cols_data
                 }
 
+    @defer.inlineCallbacks
     def update2raw(self, update, dev_version=False):
         """ Transform update data to raw SQLite statements
         @param upadte: update data as returned by generateUpdateData
@@ -694,7 +720,7 @@ class Updater(object):
             drop.append(self.DROP_SQL % table)
         if dev_version:
             if drop:
-                info("Dev version, SQL NOT EXECUTED:\n--\n%s\n--\n" % "\n".join(drop))
+                log.info("Dev version, SQL NOT EXECUTED:\n--\n%s\n--\n" % "\n".join(drop))
         else:
             ret.extend(drop)
 
@@ -705,7 +731,7 @@ class Updater(object):
 
         cols_delete = update.get('cols delete', {})
         for table in cols_delete:
-            info("Following columns in table [%s] are not needed anymore, but are kept for dev version: %s" % (table, ", ".join(cols_delete[table])))
+            log.info("Following columns in table [%s] are not needed anymore, but are kept for dev version: %s" % (table, ", ".join(cols_delete[table])))
 
         cols_modify = update.get('cols modify', {})
         for table in cols_modify:
@@ -719,4 +745,79 @@ class Updater(object):
         insert = update.get('insert', {})
         ret.extend(self.insertData2Raw(insert))
 
-        return ret
+        specific = update.get('specific', None)
+        if specific:
+            cmds = yield getattr(self, specific)()
+            ret.extend(cmds)
+        defer.returnValue(ret)
+
+    def update2raw_v2(self):
+        """Update the database from v1 to v2 (add passwords encryptions):
+            - the XMPP password value is re-used for the profile password (new parameter)
+            - the profile password is stored hashed
+            - the XMPP password is stored encrypted, with the profile password as key
+            - as there are no other stored passwords yet, it is enough, otherwise we
+              would need to encrypt the other passwords as it's done for XMPP password
+        """
+        xmpp_pass_path = ('Connection', 'Password')
+
+        def encrypt_values(values):
+            ret = []
+            list_ = []
+
+            def prepare_queries(result, xmpp_password):
+                try:
+                    id_ = result[0][0]
+                except IndexError:
+                    log.error("Profile of id %d is referenced in 'param_ind' but it doesn't exist!" % profile_id)
+                    return defer.succeed(None)
+
+                sat_password = xmpp_password
+                d1 = PasswordHasher.hash(sat_password)
+                personal_key = BlockCipher.getRandomKey(base64=True)
+                d2 = BlockCipher.encrypt(sat_password, personal_key)
+                d3 = BlockCipher.encrypt(personal_key, xmpp_password)
+
+                def gotValues(res):
+                    sat_cipher, personal_cipher, xmpp_cipher = res[0][1], res[1][1], res[2][1]
+                    ret.append("INSERT INTO param_ind(category,name,profile_id,value) VALUES ('%s','%s',%s,'%s')" %
+                               (C.PROFILE_PASS_PATH[0], C.PROFILE_PASS_PATH[1], id_, sat_cipher))
+
+                    ret.append("INSERT INTO private_ind(namespace,key,profile_id,value) VALUES ('%s','%s',%s,'%s')" %
+                               (C.MEMORY_CRYPTO_NAMESPACE, C.MEMORY_CRYPTO_KEY, id_, personal_cipher))
+
+                    ret.append("REPLACE INTO param_ind(category,name,profile_id,value) VALUES ('%s','%s',%s,'%s')" %
+                               (xmpp_pass_path[0], xmpp_pass_path[1], id_, xmpp_cipher))
+
+                return defer.DeferredList([d1, d2, d3]).addCallback(gotValues)
+
+            for profile_id, xmpp_password in values:
+                d = self.dbpool.runQuery("SELECT id FROM profiles WHERE id=?", (profile_id,))
+                d.addCallback(prepare_queries, xmpp_password)
+                list_.append(d)
+
+            d_list = defer.DeferredList(list_)
+            d_list.addCallback(lambda dummy: ret)
+            return d_list
+
+        def updateLiberviaConf(values):
+            try:
+                profile_id = values[0][0]
+            except IndexError:
+                return  # no profile called "libervia"
+
+            def cb(selected):
+                try:
+                    password = selected[0][0]
+                except IndexError:
+                    log.error("Libervia profile exists but no password is set! Update Libervia configuration will be skipped.")
+                    return
+                fixConfigOption('libervia', 'passphrase', password, False)
+            d = self.dbpool.runQuery("SELECT value FROM param_ind WHERE category=? AND name=? AND profile_id=?", xmpp_pass_path + (profile_id,))
+            return d.addCallback(cb)
+
+        d = self.dbpool.runQuery("SELECT id FROM profiles WHERE name='libervia'")
+        d.addCallback(updateLiberviaConf)
+        d.addCallback(lambda dummy: self.dbpool.runQuery("SELECT profile_id,value FROM param_ind WHERE category=? AND name=?", xmpp_pass_path))
+        d.addCallback(encrypt_values)
+        return d
