@@ -18,7 +18,9 @@
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 from sat.core.i18n import _
-from logging import debug, info, warning, error
+from sat.core.constants import Const as C
+from sat.core.log import getLogger
+log = getLogger(__name__)
 from twisted.words.protocols.jabber import jid
 
 from sat.core.exceptions import UnknownEntityError
@@ -30,6 +32,7 @@ PLUGIN_INFO = {
     "type": "EXP",
     "protocols": [],
     "dependencies": ["XEP-0045"],
+    "recommendations": [C.TEXT_CMDS],
     "main": "Exp_Parrot",
     "handler": "no",
     "description": _("""Implementation of parrot mode (repeat messages between 2 entities)""")
@@ -44,10 +47,14 @@ class Exp_Parrot(object):
     #     /unparrot command in text commands plugin.
 
     def __init__(self, host):
-        info(_("Plugin Parrot initialization"))
+        log.info(_("Plugin Parrot initialization"))
         self.host = host
         host.trigger.add("MessageReceived", self.MessageReceivedTrigger, priority=100)
         #host.trigger.add("sendMessage", self.sendMessageTrigger, priority=100)
+        try:
+            self.host.plugins[C.TEXT_CMDS].registerTextCommands(self)
+        except KeyError:
+            log.info(_("Text commands not available"))
 
     #def sendMessageTrigger(self, mess_data, treatments, profile):
     #    """ Deactivate other triggers if recipient is in parrot links """
@@ -58,7 +65,7 @@ class Exp_Parrot(object):
     #        return True
     #
     #    if mess_data['to'].userhostJID() in _links.values():
-    #        debug("Parrot link detected, skipping other triggers")
+    #        log.debug("Parrot link detected, skipping other triggers")
     #        raise SkipOtherTriggers
 
     def MessageReceivedTrigger(self, message, post_treat, profile):
@@ -95,7 +102,7 @@ class Exp_Parrot(object):
 
                 self.host.sendMessage(jid.JID(unicode(linked)), msg, None, "auto", no_trigger=True, profile_key=profile)
         else:
-            warning("No body element found in message, following normal behaviour")
+            log.warning("No body element found in message, following normal behaviour")
 
         return True
 
@@ -111,7 +118,7 @@ class Exp_Parrot(object):
             _links = client.parrot_links = {}
 
         _links[source_jid.userhostJID()] = dest_jid
-        info("Parrot mode: %s will be repeated to %s" % (source_jid.userhost(), unicode(dest_jid)))
+        log.info("Parrot mode: %s will be repeated to %s" % (source_jid.userhost(), unicode(dest_jid)))
 
     def removeParrot(self, source_jid, profile):
         """Remove parrot link
@@ -122,3 +129,47 @@ class Exp_Parrot(object):
             del client.parrot_links[source_jid.userhostJID()]
         except (AttributeError, KeyError):
             pass
+
+    def cmd_parrot(self, mess_data, profile):
+        """activate Parrot mode between 2 entities, in both directions."""
+        log.debug("Catched parrot command")
+        txt_cmd = self.host.plugins[C.TEXT_CMDS]
+
+        try:
+            link_left_jid = jid.JID(mess_data["unparsed"].strip())
+            if not link_left_jid.user or not link_left_jid.host:
+                raise jid.InvalidFormat
+        except jid.InvalidFormat:
+            txt_cmd.feedBack("Can't activate Parrot mode for invalid jid", mess_data, profile)
+            return False
+
+        link_right_jid = mess_data['to']
+
+        self.addParrot(link_left_jid, link_right_jid, profile)
+        self.addParrot(link_right_jid, link_left_jid, profile)
+
+        txt_cmd.feedBack("Parrot mode activated for %s" % (unicode(link_left_jid), ), mess_data, profile)
+
+        return False
+
+    def cmd_unparrot(self, mess_data, profile):
+        """remove Parrot mode between 2 entities, in both directions."""
+        log.debug("Catched unparrot command")
+        txt_cmd = self.host.plugins[C.TEXT_CMDS]
+
+        try:
+            link_left_jid = jid.JID(mess_data["unparsed"].strip())
+            if not link_left_jid.user or not link_left_jid.host:
+                raise jid.InvalidFormat
+        except jid.InvalidFormat:
+            txt_cmd.feedBack("Can't deactivate Parrot mode for invalid jid", mess_data, profile)
+            return False
+
+        link_right_jid = mess_data['to']
+
+        self.removeParrot(link_left_jid, profile)
+        self.removeParrot(link_right_jid, profile)
+
+        txt_cmd.feedBack("Parrot mode deactivated for %s and %s" % (unicode(link_left_jid), unicode(link_right_jid)), mess_data, profile)
+
+        return False
