@@ -18,10 +18,12 @@
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 from sat.core.i18n import _
-from logging import debug, info, warning, error
+from sat.core.constants import Const as C
+from sat.core.log import getLogger
+log = getLogger(__name__)
 from twisted.internet import defer
 from twisted.words.protocols.jabber import jid
-from twisted.words.xish.domish import Element
+from twisted.words.xish.domish import Element, generateElementsNamed
 from sat.core import exceptions
 from wokkel import disco, data_form, iwokkel
 from zope.interface import implements
@@ -52,7 +54,7 @@ ACCESS_TYPE_MAP = { 'PUBLIC': 'open',
 
 PLUGIN_INFO = {
     "name": "Group blogging throught collections",
-    "import_name": "groupblog",
+    "import_name": "GROUPBLOG",
     "type": "MISC",
     "protocols": [],
     "dependencies": ["XEP-0277"],
@@ -82,7 +84,7 @@ class GroupBlog(object):
     """This class use a SàT PubSub Service to manage access on microblog"""
 
     def __init__(self, host):
-        info(_("Group blog plugin initialization"))
+        log.info(_("Group blog plugin initialization"))
         self.host = host
 
         host.bridge.addMethod("sendGroupBlog", ".plugin", in_sign='sassa{ss}s', out_sign='',
@@ -101,6 +103,16 @@ class GroupBlog(object):
                               method=self.sendGroupBlogComment,
                               async=True)
 
+        host.bridge.addMethod("getGroupBlogs", ".plugin",
+                              in_sign='sass', out_sign='aa{ss}',
+                              method=self.getGroupBlogs,
+                              async=True)
+
+        host.bridge.addMethod("getGroupBlogsWithComments", ".plugin",
+                              in_sign='sass', out_sign='a(a{ss}aa{ss})',
+                              method=self.getGroupBlogsWithComments,
+                              async=True)
+
         host.bridge.addMethod("getLastGroupBlogs", ".plugin",
                               in_sign='sis', out_sign='aa{ss}',
                               method=self.getLastGroupBlogs,
@@ -143,32 +155,21 @@ class GroupBlog(object):
             raise exceptions.ProfileUnknownError
 
         client = self.host.getClient(profile)
-        if not client:
-            error(_('No client for this profile key: %s') % profile_key)
-            raise Exception("Unknown profile")
-        yield client.client_initialized  # we want to be sure that the client is initialized
 
         #we first check that we have a item-access pubsub server
         if not hasattr(client, "item_access_pubsub"):
-            debug(_('Looking for item-access power pubsub server'))
+            log.debug(_('Looking for item-access powered pubsub server'))
             #we don't have any pubsub server featuring item access yet
-            client.item_access_pubsub = None
-            client._item_access_pubsub_pending = defer.Deferred()
-            for entity in self.host.memory.getServerServiceEntities("pubsub", "service", profile=profile):
-                _disco = yield client.disco.requestInfo(entity)
-                #if set([NS_PUBSUB_ITEM_ACCESS, NS_PUBSUB_AUTO_CREATE, NS_PUBSUB_CREATOR_JID_CHECK]).issubset(_disco.features):
-                if set([NS_PUBSUB_AUTO_CREATE, NS_PUBSUB_CREATOR_JID_CHECK]).issubset(_disco.features):
-                    info(_("item-access powered pubsub service found: [%s]") % entity.full())
-                    client.item_access_pubsub = entity
-            client._item_access_pubsub_pending.callback(None)
-
-        if hasattr(client, "_item_access_pubsub_pending"):
-            #XXX: we need to wait for item access pubsub service check
-            yield client._item_access_pubsub_pending
-            del client._item_access_pubsub_pending
+            item_access_pubsubs = yield self.host.findFeaturesSet((NS_PUBSUB_AUTO_CREATE, NS_PUBSUB_CREATOR_JID_CHECK), "pubsub", "service", profile_key=profile)
+            # item_access_pubsubs = yield self.host.findFeaturesSet((NS_PUBSUB_ITEM_ACCESS, NS_PUBSUB_AUTO_CREATE, NS_PUBSUB_CREATOR_JID_CHECK), "pubsub", "service", profile_key=profile)
+            try:
+                client.item_access_pubsub = item_access_pubsubs.pop()
+                log.info(_("item-access powered pubsub service found: [%s]") % client.item_access_pubsub.full())
+            except KeyError:
+                client.item_access_pubsub = None
 
         if not client.item_access_pubsub:
-            error(_("No item-access powered pubsub server found, can't use group blog"))
+            log.error(_("No item-access powered pubsub server found, can't use group blog"))
             raise NoCompatiblePubSubServerFound
 
         defer.returnValue((profile, client))
@@ -186,7 +187,7 @@ class GroupBlog(object):
             if (not (origin_host)
                     or len(event_host) < len(origin_host)
                     or event_host[-len(origin_host):] != origin_host):
-                warning("Host incoherence between %s and %s (hack attempt ?)" % (unicode(event.sender),
+                log.warning("Host incoherence between %s and %s (hack attempt ?)" % (unicode(event.sender),
                                                                                  unicode(publisher)))
                 return False
 
@@ -243,7 +244,7 @@ class GroupBlog(object):
                     try:
                         microblog_data["groups"] = '\n'.join(form.fields[P.OPT_ROSTER_GROUPS_ALLOWED].values)
                     except KeyError:
-                        warning("No group found for roster access-model")
+                        log.warning("No group found for roster access-model")
                         microblog_data["groups"] = ''
 
                 break
@@ -298,7 +299,7 @@ class GroupBlog(object):
             # FIXME: check comments node creation success, at the moment this is a potential security risk (if the node
             #        already exists, the creation will silently fail, but the comments link will stay the same, linking to a
             #        node owned by somebody else)
-            defer_blog = self.host.plugins["XEP-0060"].createNode(service, comments_node, _options, profile_key=client.profile)
+            self.host.plugins["XEP-0060"].createNode(service, comments_node, _options, profile_key=client.profile)
 
         def itemCreated(mblog_item):
             form = data_form.Form('submit', formNamespace=NS_PUBSUB_ITEM_CONFIG)
@@ -317,7 +318,7 @@ class GroupBlog(object):
             elif access_type == "JID":
                 raise NotImplementedError
             else:
-                error(_("Unknown access_type"))
+                log.error(_("Unknown access_type"))
                 raise BadAccessTypeError
 
             defer_blog = self.host.plugins["XEP-0060"].publish(service, node_name, items=[mblog_item], profile_key=client.profile)
@@ -347,7 +348,7 @@ class GroupBlog(object):
         #TODO
         return failure
 
-    def sendGroupBlog(self, access_type, access_list, message, extra, profile_key='@NONE@'):
+    def sendGroupBlog(self, access_type, access_list, message, extra, profile_key=C.PROF_KEY_NONE):
         """Publish a microblog with given item access
         @param access_type: one of "PUBLIC", "GROUP", "JID"
         @param access_list: list of authorized entity (empty list for PUBLIC ACCESS,
@@ -373,12 +374,12 @@ class GroupBlog(object):
             elif access_type == "JID":
                 raise NotImplementedError
             else:
-                error(_("Unknown access type"))
+                log.error(_("Unknown access type"))
                 raise BadAccessTypeError
 
         return self._initialise(profile_key).addCallback(initialised)
 
-    def deleteGroupBlog(self, pub_data, comments, profile_key='@NONE@'):
+    def deleteGroupBlog(self, pub_data, comments, profile_key=C.PROF_KEY_NONE):
         """Delete a microblog item from a node.
         @param pub_data: a tuple (service, node identifier, item identifier)
         @param comments: comments node identifier (for main item) or empty string
@@ -395,17 +396,17 @@ class GroupBlog(object):
                 # remove the associated comments node
                 comments_service, comments_node = self.host.plugins["XEP-0277"].parseCommentUrl(comments)
                 d = self.host.plugins["XEP-0060"].deleteNode(comments_service, comments_node, profile_key=profile)
-                d.addErrback(lambda failure: error("Deletion of node %s failed: %s" % (comments_node, failure.getErrorMessage())))
+                d.addErrback(lambda failure: log.error("Deletion of node %s failed: %s" % (comments_node, failure.getErrorMessage())))
             # remove the item itself
             d = self.host.plugins["XEP-0060"].retractItems(service_jid, node, [item_id], profile_key=profile)
-            d.addErrback(lambda failure: error("Deletion of item %s from %s failed: %s" % (item_id, node, failure.getErrorMessage())))
+            d.addErrback(lambda failure: log.error("Deletion of item %s from %s failed: %s" % (item_id, node, failure.getErrorMessage())))
             return d
 
         def notify(d):
             # TODO: this works only on the same host, and notifications for item deletion should be
             # implemented according to http://xmpp.org/extensions/xep-0060.html#publisher-delete-success-notify
-            # instead. The notification mechanisms implemented in sat_pubsub and wokkel seem not compatible,
-            # see wokkel.pubsub.PubSubClient._onEvent_items and sat_pubsub.backend._doNotifyRetraction
+            # instead. The notification mechanism implemented in sat_pubsub and wokkel have apriori
+            # a problem with retrieving the subscriptions, or something else.
             service, node, item_id = pub_data
             publisher = self.host.getJidNStream(profile_key)[0]
             profile = self.host.memory.getProfileName(profile_key)
@@ -415,7 +416,7 @@ class GroupBlog(object):
 
         return self._initialise(profile_key).addCallback(initialised).addCallback(notify)
 
-    def updateGroupBlog(self, pub_data, comments, message, extra, profile_key='@NONE@'):
+    def updateGroupBlog(self, pub_data, comments, message, extra, profile_key=C.PROF_KEY_NONE):
         """Modify a microblog node
         @param pub_data: a tuple (service, node identifier, item identifier)
         @param comments: comments node identifier (for main item) or empty string
@@ -447,12 +448,12 @@ class GroupBlog(object):
                 self.__fillCommentsElement(mblog_data, entry_id, node, service_jid)
             entry_d = self.host.plugins["XEP-0277"].data2entry(mblog_data, profile)
             entry_d.addCallback(lambda mblog_item: self.host.plugins["XEP-0060"].publish(service_jid, node, items=[mblog_item], profile_key=profile))
-            entry_d.addErrback(lambda failure: error("Modification of %s failed: %s" % (pub_data, failure.getErrorMessage())))
+            entry_d.addErrback(lambda failure: log.error("Modification of %s failed: %s" % (pub_data, failure.getErrorMessage())))
             return entry_d
 
         return self._initialise(profile_key).addCallback(initialised)
 
-    def sendGroupBlogComment(self, node_url, message, extra, profile_key='@NONE@'):
+    def sendGroupBlogComment(self, node_url, message, extra, profile_key=C.PROF_KEY_NONE):
         """Publish a comment in the given node
         @param node url: link to the comments node as specified in XEP-0277 and given in microblog data's comments key
         @param message: comment
@@ -501,13 +502,14 @@ class GroupBlog(object):
                         self.host.plugins["XEP-0060"].subscribe(jid.JID(gbdata["comments_service"]), gbdata["comments_node"],
                                                                 profile_key=client.profile)
                 except KeyError:
-                    warning("Missing key for comments")
+                    log.warning("Missing key for comments")
         defer.returnValue(ret)
 
-    def getLastGroupBlogs(self, pub_jid_s, max_items=10, profile_key='@NONE@'):
-        """Get the last published microblogs
+    def __getGroupBlogs(self, pub_jid_s, max_items=10, item_ids=None, profile_key=C.PROF_KEY_NONE):
+        """Retrieve previously published items from a publish subscribe node.
         @param pub_jid_s: jid of the publisher
         @param max_items: how many microblogs we want to get (see XEP-0060 #6.5.7)
+        @param item_ids: list of microblogs items IDs
         @param profile_key: profile key
         @return: list of microblog data (dict)
         """
@@ -516,7 +518,7 @@ class GroupBlog(object):
         def initialised(result):
             profile, client = result
             d = self.host.plugins["XEP-0060"].getItems(client.item_access_pubsub, self.getNodeName(pub_jid),
-                                                       max_items=max_items, profile_key=profile_key)
+                                                       max_items=max_items, item_ids=item_ids, profile_key=profile_key)
             d.addCallback(self._itemsConstruction, pub_jid, client)
             d.addErrback(lambda ignore: {})  # TODO: more complete error management (log !)
             return d
@@ -524,7 +526,52 @@ class GroupBlog(object):
         #TODO: we need to use the server corresponding the the host of the jid
         return self._initialise(profile_key).addCallback(initialised)
 
-    def getLastGroupBlogsAtom(self, pub_jid_s, max_items=10, profile_key='@NONE@'):
+    def getGroupBlogs(self, pub_jid_s, item_ids=None, profile_key=C.PROF_KEY_NONE):
+        """Get the published microblogs of the specified IDs. If item_ids is
+        None, the result would be the same than calling getLastGroupBlogs
+        with the default value for the attribute max_items.
+        @param pub_jid_s: jid of the publisher
+        @param item_ids: list of microblogs items IDs
+        @param profile_key: profile key
+        @return: list of microblog data (dict)
+        """
+        return self.__getGroupBlogs(pub_jid_s, item_ids=item_ids, profile_key=profile_key)
+
+    def getGroupBlogsWithComments(self, pub_jid_s, item_ids=None, profile_key=C.PROF_KEY_NONE):
+        """Get the published microblogs of the specified IDs and their comments. If
+        item_ids is None, returns the last published microblogs and their comments.
+        @param pub_jid_s: jid of the publisher
+        @param item_ids: list of microblogs items IDs
+        @param profile_key: profile key
+        @return: list of couple (microblog data, list of microblog data)
+        """
+        def get_comments(data):
+            d_list = []
+            for entry in data:
+                if entry.get('comments', False):
+                    d = self.getGroupBlogComments(entry['comments_service'], entry['comments_node'], profile_key=profile_key)
+                    d.addCallback(lambda data: (entry, data))
+                    d_list.append(d)
+                else:
+                    d_list.append(defer.succeed((entry, [])))
+            deferred_list = defer.DeferredList(d_list)
+            deferred_list.addCallback(lambda result: [value for (success, value) in result if success])
+            return deferred_list
+
+        d = self.__getGroupBlogs(pub_jid_s, item_ids=item_ids, profile_key=profile_key)
+        d.addCallback(get_comments)
+        return d
+
+    def getLastGroupBlogs(self, pub_jid_s, max_items=10, profile_key=C.PROF_KEY_NONE):
+        """Get the last published microblogs
+        @param pub_jid_s: jid of the publisher
+        @param max_items: how many microblogs we want to get (see XEP-0060 #6.5.7)
+        @param profile_key: profile key
+        @return: list of microblog data (dict)
+        """
+        return self.__getGroupBlogs(pub_jid_s, max_items=max_items, profile_key=profile_key)
+
+    def getLastGroupBlogsAtom(self, pub_jid_s, max_items=10, profile_key=C.PROF_KEY_NONE):
         """Get the atom feed of the last published microblogs
         @param pub_jid: jid of the publisher
         @param max_items: how many microblogs we want to get (see XEP-0060 #6.5.7)
@@ -565,13 +612,13 @@ class GroupBlog(object):
             d = self.host.plugins["XEP-0060"].getItems(client.item_access_pubsub, self.getNodeName(pub_jid),
                                                        max_items=max_items, profile_key=profile_key)
             d.addCallback(items2feed, pub_jid, client)
-            d.addErrback(lambda ignore: {})  # TODO: more complete error management (log !)
+            d.addErrback(lambda ignore: '')  # TODO: more complete error management (log !)
             return d
 
         #TODO: we need to use the server corresponding the the host of the jid
         return self._initialise(profile_key).addCallback(initialised)
 
-    def getGroupBlogComments(self, service_s, node, profile_key='@NONE@'):
+    def getGroupBlogComments(self, service_s, node, profile_key=C.PROF_KEY_NONE):
         """Get all comments of given node
         @param service_s: service hosting the node
         @param node: comments node
@@ -591,14 +638,14 @@ class GroupBlog(object):
         #TODO: we need to use the server corresponding the the host of the jid
         return self._initialise(profile_key).addCallback(initialised)
 
-    def _getMassiveLastGroupBlogs(self, publishers_type, publishers, max_items=10, profile_key='@NONE@'):
+    def _getMassiveLastGroupBlogs(self, publishers_type, publishers, max_items=10, profile_key=C.PROF_KEY_NONE):
         if publishers_type == 'JID':
             publishers_jids = [jid.JID(publisher) for publisher in publishers]
         else:
             publishers_jids = publishers
         return self.getMassiveLastGroupBlogs(publishers_type, publishers_jids, max_items, profile_key)
 
-    def getMassiveLastGroupBlogs(self, publishers_type, publishers, max_items=10, profile_key='@NONE@'):
+    def getMassiveLastGroupBlogs(self, publishers_type, publishers, max_items=10, profile_key=C.PROF_KEY_NONE):
         """Get the last published microblogs for a list of groups or jids
         @param publishers_type: type of the list of publishers (one of "GROUP" or "JID" or "ALL")
         @param publishers: list of publishers, according to "publishers_type" (list of groups or list of jids)
@@ -607,7 +654,7 @@ class GroupBlog(object):
         """
 
         def sendResult(result):
-            """send result of DeferredList (list of microblogs to the calling method"""
+            """send result of DeferredList (dict of jid => microblogs) to the calling method"""
 
             ret = {}
 
@@ -635,7 +682,6 @@ class GroupBlog(object):
 
             mblogs = []
 
-
             for jid_ in jids:
                 d = self.host.plugins["XEP-0060"].getItems(client.item_access_pubsub, self.getNodeName(jid_),
                                                            max_items=max_items, profile_key=profile_key)
@@ -658,7 +704,7 @@ class GroupBlog(object):
         return self._initialise(profile_key).addCallback(initialised)
         #TODO: we need to use the server corresponding the the host of the jid
 
-    def subscribeGroupBlog(self, pub_jid, profile_key='@NONE@'):
+    def subscribeGroupBlog(self, pub_jid, profile_key=C.PROF_KEY_NONE):
         def initialised(result):
             profile, client = result
             d = self.host.plugins["XEP-0060"].subscribe(client.item_access_pubsub, self.getNodeName(jid.JID(pub_jid)),
@@ -668,14 +714,14 @@ class GroupBlog(object):
         #TODO: we need to use the server corresponding the the host of the jid
         return self._initialise(profile_key).addCallback(initialised)
 
-    def _massiveSubscribeGroupBlogs(self, publishers_type, publishers, profile_key='@NONE@'):
+    def _massiveSubscribeGroupBlogs(self, publishers_type, publishers, profile_key=C.PROF_KEY_NONE):
         if publishers_type == 'JID':
             publishers_jids = [jid.JID(publisher) for publisher in publishers]
         else:
             publishers_jids = publishers
         return self.massiveSubscribeGroupBlogs(publishers_type, publishers_jids, profile_key)
 
-    def massiveSubscribeGroupBlogs(self, publishers_type, publishers, profile_key='@NONE@'):
+    def massiveSubscribeGroupBlogs(self, publishers_type, publishers, profile_key=C.PROF_KEY_NONE):
         """Subscribe microblogs for a list of groups or jids
         @param publishers_type: type of the list of publishers (one of "GROUP" or "JID" or "ALL")
         @param publishers: list of publishers, according to "publishers_type" (list of groups or list of jids)
@@ -715,6 +761,109 @@ class GroupBlog(object):
         return self._initialise(profile_key).addCallback(initialised)
         #TODO: we need to use the server corresponding the the host of the jid
 
+    def deleteAllGroupBlogsAndComments(self, profile_key=C.PROF_KEY_NONE):
+        """Delete absolutely all the microblog data that the user has posted"""
+        calls = [self.deleteAllGroupBlogs(profile_key), self.deleteAllGroupBlogsComments(profile_key)]
+        return defer.DeferredList(calls)
+
+    def deleteAllGroupBlogs(self, profile_key=C.PROF_KEY_NONE):
+        """Delete all the main items and their comments that the user has posted
+        """
+        def initialised(result):
+            profile, client = result
+            service = client.item_access_pubsub
+            jid_ = client.jid
+
+            main_node = self.getNodeName(jid_)
+            d = self.host.plugins["XEP-0060"].deleteNode(service, main_node, profile_key=profile)
+            d.addCallback(lambda dummy: log.info(_("All microblog's main items from %s have been deleted!") % jid_.userhost()))
+            d.addErrback(lambda failure: log.error(_("Deletion of node %(node)s failed: %(message)s") %
+                                               {'node': main_node, 'message': failure.getErrorMessage()}))
+            return d
+
+        return self._initialise(profile_key).addCallback(initialised)
+
+    def deleteAllGroupBlogsComments(self, profile_key=C.PROF_KEY_NONE):
+        """Delete all the comments that the user posted on other's main items.
+        We avoid the conversions from item to microblog data as we only need
+        to retrieve some attributes, no need to convert text syntax...
+        """
+        def initialised(result):
+            """Get all the main items from our contact list
+            @return: a DeferredList
+            """
+            profile, client = result
+            service = client.item_access_pubsub
+            jids = [contact.jid.userhostJID() for contact in client.roster.getItems()]
+            blogs = []
+            for jid_ in jids:
+                main_node = self.getNodeName(jid_)
+                d = self.host.plugins["XEP-0060"].getItems(service, main_node, profile_key=profile)
+                d.addCallback(getComments, client)
+                d.addErrback(lambda failure, main_node: log.error(_("Retrieval of items for node %(node)s failed: %(message)s") %
+                                                              {'node': main_node, 'message': failure.getErrorMessage()}), main_node)
+                blogs.append(d)
+
+            return defer.DeferredList(blogs)
+
+        def getComments(items, client):
+            """Get all the comments for a list of items
+            @param items: a list of main items for one user
+            @param client: the client of the user
+            @return: a DeferredList
+            """
+            comments = []
+            for item in items:
+                try:
+                    entry = generateElementsNamed(item.elements(), 'entry').next()
+                    link = generateElementsNamed(entry.elements(), 'link').next()
+                except StopIteration:
+                    continue
+                href = link.getAttribute('href')
+                service, node = self.host.plugins['XEP-0277'].parseCommentUrl(href)
+                d = self.host.plugins["XEP-0060"].getItems(service, node, profile_key=profile_key)
+                d.addCallback(lambda items, service, node: (service, node, items), service, node)
+                d.addErrback(lambda failure, node: log.error(_("Retrieval of comments for node %(node)s failed: %(message)s") %
+                                                         {'node': node, 'message': failure.getErrorMessage()}), node)
+                comments.append(d)
+            dlist = defer.DeferredList(comments)
+            dlist.addCallback(deleteComments, client)
+            return dlist
+
+        def deleteComments(result, client):
+            """Delete all the comments of the user that are found in result
+            @param result: a list of couple (success, value) with success a
+            boolean and value a tuple (service as JID, node_id, comment_items)
+            @param client: the client of the user
+            @return: a DeferredList with the deletions result
+            """
+            user_jid_s = client.jid.userhost()
+            for (success, value) in result:
+                if not success:
+                    continue
+                service, node_id, comment_items = value
+                item_ids = []
+                for comment_item in comment_items:  # for all the comments on one post
+                    try:
+                        entry = generateElementsNamed(comment_item.elements(), 'entry').next()
+                        author = generateElementsNamed(entry.elements(), 'author').next()
+                        name = generateElementsNamed(author.elements(), 'name').next()
+                    except StopIteration:
+                        continue
+                    if name.children[0] == user_jid_s:
+                        item_ids.append(comment_item.getAttribute('id'))
+                deletions = []
+                if item_ids:  # remove the comments of the user on the given post
+                    d = self.host.plugins['XEP-0060'].retractItems(service, node_id, item_ids, profile_key=profile_key)
+                    d.addCallback(lambda dummy, node_id: log.debug(_('Comments of user %(user)s in node %(node)s have been retracted') %
+                                                      {'user': user_jid_s, 'node': node_id}), node_id)
+                    d.addErrback(lambda failure, node_id: log.error(_("Retraction of comments from %(user)s in node %(node)s failed: %(message)s") %
+                                                       {'user': user_jid_s, 'node': node_id, 'message': failure.getErrorMessage()}), node_id)
+                    deletions.append(d)
+            return defer.DeferredList(deletions)
+
+        return self._initialise(profile_key).addCallback(initialised)
+
 
 class GroupBlog_handler(XMPPHandler):
     implements(iwokkel.IDisco)
