@@ -18,11 +18,13 @@
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 from sat.core.i18n import _
-from logging import debug, info, error
+from sat.core.constants import Const as C
+from sat.core.log import getLogger
+log = getLogger(__name__)
 import warnings
 warnings.filterwarnings('ignore', 'the MimeWriter', DeprecationWarning, 'twisted')  # FIXME: to be removed, see http://twistedmatrix.com/trac/ticket/4038
 from twisted.internet import protocol
-from twisted.words.protocols.jabber import error as jab_error
+from twisted.words.protocols import jabber
 from twisted.cred import portal, checkers
 from twisted.mail import imap4, maildir
 from email.parser import Parser
@@ -57,7 +59,7 @@ class MaildirError(Exception):
 class MaildirBox(object):
 
     def __init__(self, host):
-        info(_("Plugin Maildir initialization"))
+        log.info(_("Plugin Maildir initialization"))
         self.host = host
 
         self.__observed = {}
@@ -100,7 +102,7 @@ class MaildirBox(object):
                 self.accessMessageBox("INBOX", profile_key=profile).addMessage(message)
                 return False
 
-    def accessMessageBox(self, boxname, observer=None, profile_key='@DEFAULT@'):
+    def accessMessageBox(self, boxname, observer=None, profile_key=C.PROF_KEY_NONE):
         """Create and return a MailboxUser instance
         @param boxname: name of the box
         @param observer: method to call when a NewMessage arrive"""
@@ -125,7 +127,7 @@ class MaildirBox(object):
         @param mailboxUser: MailboxUser instance"""
         if boxname not in self.__mailboxes:
             err_msg = _("Trying to remove an mailboxUser not referenced")
-            error(_("INTERNAL ERROR: ") + err_msg)
+            log.error(_("INTERNAL ERROR: ") + err_msg)
             raise MaildirError(err_msg)
         assert self.__mailboxes[profile][boxname] == mailboxUser
         del self.__mailboxes[profile][boxname]
@@ -144,7 +146,7 @@ class MaildirBox(object):
             return self.data[profile][boxname]  # the boxname MUST exist in the data
         except KeyError:
             err_msg = _("Boxname doesn't exist in internal data")
-            error(_("INTERNAL ERROR: ") + err_msg)
+            log.error(_("INTERNAL ERROR: ") + err_msg)
             raise MaildirError(err_msg)
 
     def getUid(self, boxname, message_id, profile):
@@ -283,21 +285,21 @@ class MaildirBox(object):
         @param signal: which signal was observed by the caller"""
         if (profile, boxname) not in self.__observed:
             err_msg = _("Trying to remove an observer for an inexistant mailbox")
-            error(_("INTERNAL ERROR: ") + err_msg)
+            log.error(_("INTERNAL ERROR: ") + err_msg)
             raise MaildirError(err_msg)
         if signal not in self.__observed[(profile, boxname)]:
             err_msg = _("Trying to remove an inexistant observer, no observer for this signal")
-            error(_("INTERNAL ERROR: ") + err_msg)
+            log.error(_("INTERNAL ERROR: ") + err_msg)
             raise MaildirError(err_msg)
         if not callback in self.__observed[(profile, boxname)][signal]:
             err_msg = _("Trying to remove an inexistant observer")
-            error(_("INTERNAL ERROR: ") + err_msg)
+            log.error(_("INTERNAL ERROR: ") + err_msg)
             raise MaildirError(err_msg)
         self.__observed[(profile, boxname)][signal].remove(callback)
 
     def emitSignal(self, profile, boxname, signal_name):
         """Emit the signal to observer"""
-        debug('emitSignal %s %s %s' % (profile, boxname, signal_name))
+        log.debug('emitSignal %s %s %s' % (profile, boxname, signal_name))
         try:
             for observer_cb in self.__observed[(profile, boxname)][signal_name]:
                 observer_cb()
@@ -327,13 +329,13 @@ class MailboxUser(object):
                 mail['Subject'] = e.children[0].encode('utf-8')
         return mail.as_string()
 
-    def __init__(self, _maildir, name, observer=None, profile="@NONE@"):
+    def __init__(self, _maildir, name, observer=None, profile=C.PROF_KEY_NONE):
         """@param _maildir: the main MaildirBox instance
            @param name: name of the mailbox
            @param profile: real profile (ie not a profile_key)
            THIS OBJECT MUST NOT BE USED DIRECTLY: use MaildirBox.accessMessageBox instead"""
         if _maildir._checkBoxReference(name, profile):
-            error("INTERNAL ERROR: MailboxUser MUST NOT be instancied directly")
+            log.error("INTERNAL ERROR: MailboxUser MUST NOT be instancied directly")
             raise MaildirError('double MailboxUser instanciation')
         if name != "INBOX":
             raise NotImplementedError
@@ -351,7 +353,7 @@ class MailboxUser(object):
         self.__uid_table_update()
 
         if observer:
-            debug("adding observer for %s (%s)" % (name, profile))
+            log.debug("adding observer for %s (%s)" % (name, profile))
             self.maildir.addObserver(observer, profile, name, "NEW_MESSAGE")
 
     def __uid_table_update(self):
@@ -364,7 +366,7 @@ class MailboxUser(object):
 
     def __del__(self):
         if self.observer:
-            debug("removing observer for %s" % self.name)
+            log.debug("removing observer for %s" % self.name)
             self._maildir.removeObserver(self.observer, self.name, "NEW_MESSAGE")
         self.maildir._removeBoxAccess(self.name, self, profile=self.profile)
 
