@@ -18,7 +18,9 @@
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 from sat.core.i18n import _
-from logging import debug, info, warning, error
+from sat.core.constants import Const as C
+from sat.core.log import getLogger
+log = getLogger(__name__)
 from twisted.words.xish import domish
 from twisted.words.protocols.jabber import jid
 from twisted.internet import defer
@@ -27,7 +29,6 @@ from wokkel import data_form
 from sat.memory import memory
 from sat.tools import xml_tools
 from sat_frontends.tools.games import TarotCard
-from time import time
 import random
 
 
@@ -54,7 +55,7 @@ class Tarot(object):
         self.__class__ = type(self.__class__.__name__, (self.__class__, RoomGame, object), {})
 
     def __init__(self, host):
-        info(_("Plugin Tarot initialization"))
+        log.info(_("Plugin Tarot initialization"))
         self._sessions = memory.Sessions()
         self.inheritFromRoomGame(host)
         RoomGame._init_(self, host, PLUGIN_INFO, (NS_CG, CG_TAG),
@@ -198,7 +199,7 @@ class Tarot(object):
                             pl_waiting = players_data[player]['wait_for_low']
                             played.remove(card)
                             players_data[pl_waiting]['levees'].append(card)
-                            debug(_('Player %(excuse_owner)s give %(card_waited)s to %(player_waiting)s for Excuse compensation') % {"excuse_owner": player, "card_waited": card, "player_waiting": pl_waiting})
+                            log.debug(_('Player %(excuse_owner)s give %(card_waited)s to %(player_waiting)s for Excuse compensation') % {"excuse_owner": player, "card_waited": card, "player_waiting": pl_waiting})
                             return
                 return
 
@@ -229,12 +230,12 @@ class Tarot(object):
                 low_card = owner_levees[card_idx]
                 del owner_levees[card_idx]
                 players_data[winner]['levees'].append(low_card)
-                debug(_('Player %(excuse_owner)s give %(card_waited)s to %(player_waiting)s for Excuse compensation') % {"excuse_owner": excuse_player, "card_waited": low_card, "player_waiting": winner})
+                log.debug(_('Player %(excuse_owner)s give %(card_waited)s to %(player_waiting)s for Excuse compensation') % {"excuse_owner": excuse_player, "card_waited": low_card, "player_waiting": winner})
                 break
         if not low_card:  # The player has no low card yet
             #TODO: manage case when player never win a trick with low card
             players_data[excuse_player]['wait_for_low'] = winner
-            debug(_("%(excuse_owner)s keep the Excuse but has not card to give, %(winner)s is waiting for one") % {'excuse_owner': excuse_player, 'winner': winner})
+            log.debug(_("%(excuse_owner)s keep the Excuse but has not card to give, %(winner)s is waiting for one") % {'excuse_owner': excuse_player, 'winner': winner})
 
     def __draw_game(self, game_data):
         """The game is draw, no score change
@@ -245,7 +246,7 @@ class Tarot(object):
         scores_str += '\n'
         for player in game_data['players']:
             scores_str += _("\n--\n%(player)s:\nscore for this game ==> %(score_game)i\ntotal score ==> %(total_score)i") % {'player': player, 'score_game': 0, 'total_score': players_data[player]['score']}
-        debug(scores_str)
+        log.debug(scores_str)
 
         return (scores_str, [], [])
 
@@ -294,7 +295,7 @@ class Tarot(object):
         elif game_data['contrat'] == 'Garde Contre':
             contrat_mult = 6
         else:
-            error(_('INTERNAL ERROR: contrat not managed (mispelled ?)'))
+            log.error(_('INTERNAL ERROR: contrat not managed (mispelled ?)'))
             assert(False)
 
         victory = (score >= point_limit)
@@ -317,7 +318,7 @@ class Tarot(object):
         scores_str += '\n'
         for player in game_data['players']:
             scores_str += _("\n--\n%(player)s:\nscore for this game ==> %(score_game)i\ntotal score ==> %(total_score)i") % {'player': player, 'score_game': player_score[player], 'total_score': players_data[player]['score']}
-        debug(scores_str)
+        log.debug(scores_str)
 
         return (scores_str, winners, loosers)
 
@@ -374,7 +375,7 @@ class Tarot(object):
             if card.suit == "atout" and card < biggest_atout and biggest_hand_atout > biggest_atout and card.value != "excuse":
                 forbidden_cards.append(card)
         else:
-            error(_('Internal error: unmanaged game stage'))
+            log.error(_('Internal error: unmanaged game stage'))
         return forbidden_cards
 
     def __start_play(self, room_jid, game_data, profile):
@@ -393,7 +394,7 @@ class Tarot(object):
         try:
             session_data = self._sessions.profileGet(raw_data["session_id"], profile)
         except KeyError:
-            warning(_("session id doesn't exist, session has probably expired"))
+            log.warning(_("session id doesn't exist, session has probably expired"))
             # TODO: send error dialog
             return defer.succeed({})
 
@@ -402,7 +403,7 @@ class Tarot(object):
         player = self.host.plugins["XEP-0045"].getRoomNick(room_jid_s, profile)
         data = xml_tools.XMLUIResult2DataFormResult(raw_data)
         contrat = data['contrat']
-        debug(_('contrat [%(contrat)s] choosed by %(profile)s') % {'contrat': contrat, 'profile': profile})
+        log.debug(_('contrat [%(contrat)s] choosed by %(profile)s') % {'contrat': contrat, 'profile': profile})
         d = self.send(jid.JID(referee), ('', 'contrat_choosed'), {'player': player}, content=contrat, profile=profile)
         d.addCallback(lambda ignore: {})
         del self._sessions[raw_data["session_id"]]
@@ -416,7 +417,7 @@ class Tarot(object):
         try:
             session_data = self._sessions.profileGet(raw_data["session_id"], profile)
         except KeyError:
-            warning(_("session id doesn't exist, session has probably expired"))
+            log.warning(_("session id doesn't exist, session has probably expired"))
             # TODO: send error dialog
             return defer.succeed({})
 
@@ -426,7 +427,7 @@ class Tarot(object):
         del self._sessions[raw_data["session_id"]]
         return defer.succeed({})
 
-    def play_cards(self, player, referee, cards, profile_key='@NONE@'):
+    def play_cards(self, player, referee, cards, profile_key=C.PROF_KEY_NONE):
         """Must be call by player when the contrat is selected
         @param player: player's name
         @param referee: arbiter jid
@@ -435,9 +436,9 @@ class Tarot(object):
         """
         profile = self.host.memory.getProfileName(profile_key)
         if not profile:
-            error(_("profile %s is unknown") % profile_key)
+            log.error(_("profile %s is unknown") % profile_key)
             return
-        debug(_('Cards played by %(profile)s: [%(cards)s]') % {'profile': profile, 'cards': cards})
+        log.debug(_('Cards played by %(profile)s: [%(cards)s]') % {'profile': profile, 'cards': cards})
         elem = self.__card_list_to_xml(TarotCard.from_tuples(cards), 'cards_played')
         self.send(jid.JID(referee), elem, {'player': player}, profile=profile)
 
@@ -503,7 +504,7 @@ class Tarot(object):
                 status = self.games[room_jid.userhost()]['status']
                 nb_players = len(self.games[room_jid.userhost()]['players'])
                 status[player] = 'ready'
-                debug(_('Player %(player)s is ready to start [status: %(status)s]') % {'player': player, 'status': status})
+                log.debug(_('Player %(player)s is ready to start [status: %(status)s]') % {'player': player, 'status': status})
                 if status.values().count('ready') == nb_players:  # everybody is ready, we can start the game
                     self.newRound(room_jid, profile)
 
@@ -538,14 +539,14 @@ class Tarot(object):
                             best_contrat[0] = player
                             best_contrat[1] = contrat
                     if best_contrat[1] == "Passe":
-                        debug(_("Everybody is passing, round ended"))
+                        log.debug(_("Everybody is passing, round ended"))
                         to_jid = jid.JID(room_jid.userhost())
                         self.send(to_jid, self.__give_scores(*self.__draw_game(game_data)), profile=profile)
                         game_data['init_player'] = (game_data['init_player'] + 1) % len(game_data['players'])  # we change the dealer
                         for player in game_data['players']:
                             game_data['status'][player] = "init"
                         return
-                    debug(_("%(player)s win the bid with %(contrat)s") % {'player': best_contrat[0], 'contrat': best_contrat[1]})
+                    log.debug(_("%(player)s win the bid with %(contrat)s") % {'player': best_contrat[0], 'contrat': best_contrat[1]})
                     game_data['contrat'] = best_contrat[1]
 
                     if game_data['contrat'] == "Garde Sans" or game_data['contrat'] == "Garde Contre":
@@ -566,7 +567,7 @@ class Tarot(object):
                         del game_data['chien'][:]
 
             elif elt.name == 'chien':  # we have received the chien
-                debug(_("tarot: chien received"))
+                log.debug(_("tarot: chien received"))
                 data = {"attaquant": elt['attaquant']}
                 game_data['stage'] = "ecart"
                 game_data['attaquant'] = elt['attaquant']
@@ -617,7 +618,7 @@ class Tarot(object):
                         if all(played):
                             #everybody has played
                             winner = self.__winner(game_data)
-                            debug(_('The winner of this trick is %s') % winner)
+                            log.debug(_('The winner of this trick is %s') % winner)
                             #the winner win the trick
                             self.__excuse_hack(game_data, played, winner)
                             players_data[elt['player']]['levees'].extend(played)
@@ -663,9 +664,9 @@ class Tarot(object):
                     invalid_cards = self.__xml_to_list(elt.elements(name='invalid', uri=NS_CG).next())
                     self.host.bridge.tarotGameInvalidCards(room_jid.userhost(), elt['phase'], played_cards, invalid_cards, profile)
                 else:
-                    error(_('Unmanaged error type: %s') % elt['type'])
+                    log.error(_('Unmanaged error type: %s') % elt['type'])
             else:
-                error(_('Unmanaged card game element: %s') % elt.name)
+                log.error(_('Unmanaged card game element: %s') % elt.name)
 
     def getSyncDataForPlayer(self, room_jid_s, nick):
         return []
