@@ -18,12 +18,11 @@
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 from sat.core.i18n import _
-from logging import debug, info, warning, error
+from sat.core.log import getLogger
+log = getLogger(__name__)
 from twisted.words.protocols.jabber import client as jabber_client, jid
 from twisted.words.xish import domish
-import twisted.internet.error
 from twisted.internet import reactor
-from sat.core.exceptions import ProfileNotInCacheError
 
 from wokkel import disco, iwokkel
 
@@ -61,7 +60,7 @@ class XEP_0047(object):
     NAMESPACE = NS_IBB
 
     def __init__(self, host):
-        info(_("In-Band Bytestreams plugin initialization"))
+        log.info(_("In-Band Bytestreams plugin initialization"))
         self.host = host
 
     def getHandler(self, profile):
@@ -69,14 +68,12 @@ class XEP_0047(object):
 
     def profileConnected(self, profile):
         client = self.host.getClient(profile)
-        if not client:
-            raise ProfileNotInCacheError
         client.xep_0047_current_stream = {}  # key: stream_id, value: data(dict)
 
     def _timeOut(self, sid, profile):
         """Delecte current_stream id, called after timeout
         @param id: id of client.xep_0047_current_stream"""
-        info(_("In-Band Bytestream: TimeOut reached for id %(sid)s [%(profile)s]")
+        log.info(_("In-Band Bytestream: TimeOut reached for id %(sid)s [%(profile)s]")
              % {"sid": sid, "profile": profile})
         self._killId(sid, False, "TIMEOUT", profile)
 
@@ -85,11 +82,8 @@ class XEP_0047(object):
         @param sid: id of client.xep_0047_current_stream"""
         assert(profile)
         client = self.host.getClient(profile)
-        if not client:
-            warning(_("Client no more in cache"))
-            return
         if sid not in client.xep_0047_current_stream:
-            warning(_("kill id called on a non existant id"))
+            log.warning(_("kill id called on a non existant id"))
             return
         if "observer_cb" in client.xep_0047_current_stream[sid]:
             client.xmlstream.removeObserver(client.xep_0047_current_stream[sid]["event_data"], client.xep_0047_current_stream[sid]["observer_cb"])
@@ -112,8 +106,6 @@ class XEP_0047(object):
     def getProgress(self, sid, data, profile):
         """Fill data with position of current transfer"""
         client = self.host.getClient(profile)
-        if not client:
-            raise ProfileNotInCacheError
         try:
             file_obj = client.xep_0047_current_stream[sid]["file_obj"]
             data["position"] = str(file_obj.tell())
@@ -131,8 +123,6 @@ class XEP_0047(object):
         @param failure_cb: method to call when something goes wrong
         @param profile: %(doc_profile)s"""
         client = self.host.getClient(profile)
-        if not client:
-            raise ProfileNotInCacheError
         data = client.xep_0047_current_stream[sid] = {}
         data["from"] = from_jid
         data["file_obj"] = file_obj
@@ -145,25 +135,23 @@ class XEP_0047(object):
         data["failure_cb"] = failure_cb
 
     def streamOpening(self, IQ, profile):
-        debug(_("IBB stream opening"))
+        log.debug(_("IBB stream opening"))
         IQ.handled = True
         client = self.host.getClient(profile)
-        if not client:
-            raise ProfileNotInCacheError
         open_elt = IQ.firstChildElement()
         block_size = open_elt.getAttribute('block-size')
         sid = open_elt.getAttribute('sid')
         stanza = open_elt.getAttribute('stanza', 'iq')
         if not sid or not block_size or int(block_size) > 65535:
-            warning(_("malformed IBB transfer: %s" % IQ['id']))
+            log.warning(_("malformed IBB transfer: %s" % IQ['id']))
             self.sendNotAcceptableError(IQ['id'], IQ['from'], client.xmlstream)
             return
         if not sid in client.xep_0047_current_stream:
-            warning(_("Ignoring unexpected IBB transfer: %s" % sid))
+            log.warning(_("Ignoring unexpected IBB transfer: %s" % sid))
             self.sendNotAcceptableError(IQ['id'], IQ['from'], client.xmlstream)
             return
         if client.xep_0047_current_stream[sid]["from"] != jid.JID(IQ['from']):
-            warning(_("sended jid inconsistency (man in the middle attack attempt ?)"))
+            log.warning(_("sended jid inconsistency (man in the middle attack attempt ?)"))
             self.sendNotAcceptableError(IQ['id'], IQ['from'], client.xmlstream)
             self._killId(sid, False, "PROTOCOL_ERROR", profile=profile)
             return
@@ -190,9 +178,7 @@ class XEP_0047(object):
     def streamClosing(self, IQ, profile):
         IQ.handled = True
         client = self.host.getClient(profile)
-        if not client:
-            raise ProfileNotInCacheError
-        debug(_("IBB stream closing"))
+        log.debug(_("IBB stream closing"))
         data_elt = IQ.firstChildElement()
         sid = data_elt.getAttribute('sid')
         result = domish.Element((None, 'iq'))
@@ -205,8 +191,6 @@ class XEP_0047(object):
     def iqData(self, IQ, profile):
         IQ.handled = True
         client = self.host.getClient(profile)
-        if not client:
-            raise ProfileNotInCacheError
         data_elt = IQ.firstChildElement()
 
         if self._manageDataElt(data_elt, 'iq', IQ['id'], jid.JID(IQ['from']), profile):
@@ -219,7 +203,6 @@ class XEP_0047(object):
             client.xmlstream.send(result)
 
     def messageData(self, message_elt, profile):
-        data_elt = message_elt.firstChildElement()
         sid = message_elt.getAttribute('id', '')
         self._manageDataElt(message_elt, 'message', sid, jid.JID(message_elt['from']), profile)
 
@@ -228,25 +211,23 @@ class XEP_0047(object):
         @param data_elt: "data" domish element
         @return: True if success"""
         client = self.host.getClient(profile)
-        if not client:
-            raise ProfileNotInCacheError
         sid = data_elt.getAttribute('sid')
         if sid not in client.xep_0047_current_stream:
-            error(_("Received data for an unknown session id"))
+            log.error(_("Received data for an unknown session id"))
             return False
 
         from_jid = client.xep_0047_current_stream[sid]["from"]
         file_obj = client.xep_0047_current_stream[sid]["file_obj"]
 
         if stanza_from_jid != from_jid:
-            warning(_("sended jid inconsistency (man in the middle attack attempt ?)"))
+            log.warning(_("sended jid inconsistency (man in the middle attack attempt ?)"))
             if stanza == 'iq':
                 self.sendNotAcceptableError(sid, from_jid, client.xmlstream)
             return False
 
         client.xep_0047_current_stream[sid]["seq"] += 1
         if int(data_elt.getAttribute("seq", -1)) != client.xep_0047_current_stream[sid]["seq"]:
-            warning(_("Sequence error"))
+            log.warning(_("Sequence error"))
             if stanza == 'iq':
                 self.sendNotAcceptableError(data_elt["id"], from_jid, client.xmlstream)
             return False
@@ -259,7 +240,7 @@ class XEP_0047(object):
             file_obj.write(base64.b64decode(str(data_elt)))
         except TypeError:
             #The base64 data is invalid
-            warning(_("Invalid base64 data"))
+            log.warning(_("Invalid base64 data"))
             if stanza == 'iq':
                 self.sendNotAcceptableError(data_elt["id"], from_jid, client.xmlstream)
             return False
@@ -289,10 +270,8 @@ class XEP_0047(object):
         @param failureCb: method to call when something goes wrong
         @param profile: %(doc_profile)s"""
         client = self.host.getClient(profile)
-        if not client:
-            raise ProfileNotInCacheError
         if length is not None:
-            error(_('stream length not managed yet'))
+            log.error(_('stream length not managed yet'))
             return
         data = client.xep_0047_current_stream[sid] = {}
         data["timer"] = reactor.callLater(TIMEOUT, self._timeOut, sid)
@@ -317,11 +296,9 @@ class XEP_0047(object):
     def iqResult(self, sid, seq, length, profile, iq_elt):
         """Called when the result of open iq is received"""
         client = self.host.getClient(profile)
-        if not client:
-            raise ProfileNotInCacheError
         data = client.xep_0047_current_stream[sid]
         if iq_elt["type"] == "error":
-            warning(_("Transfer failed"))
+            log.warning(_("Transfer failed"))
             self.terminateStream(sid, "IQ_ERROR")
             return
 
@@ -351,8 +328,6 @@ class XEP_0047(object):
         @param callback: method to call after finishing
         @param failure_reason: reason of the failure, or None if steam was successful"""
         client = self.host.getClient(profile)
-        if not client:
-            raise ProfileNotInCacheError
         data = client.xep_0047_current_stream[sid]
         iq_elt = jabber_client.IQ(client.xmlstream, 'set')
         iq_elt['to'] = data["to"].full()
