@@ -18,11 +18,13 @@
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 from sat.core.i18n import _, D_
-from logging import debug, info, warning, error
+from sat.core.constants import Const as C
+from sat.core.log import getLogger
+log = getLogger(__name__)
 from twisted.words.protocols.jabber import jid
-from twisted.words.protocols.jabber import error as xmpp_error
+from twisted.words.protocols import jabber
 from twisted.words.xish import domish
-from twisted.internet import defer, reactor
+from twisted.internet import defer
 from wokkel import disco, iwokkel, data_form, compat
 from sat.core import exceptions
 from sat.memory.memory import Sessions
@@ -105,13 +107,13 @@ class AdHocCommand(XMPPHandler):
             try:
                 allowed.update(self.client.roster.getJidsFromGroup(group))
             except exceptions.UnknownGroupError:
-                warning(_("The groups [%(group)s] is unknown for profile [%(profile)s])" % {'group':group, 'profile':self.client.profile}))
+                log.warning(_("The groups [%(group)s] is unknown for profile [%(profile)s])" % {'group':group, 'profile':self.client.profile}))
         if requestor.userhostJID() in allowed:
             return True
         return False
 
     def getDiscoInfo(self, requestor, target, nodeIdentifier=''):
-        identities = [ID_CMD_LIST if self.node == NS_COMMANDS else ID_CMD_NODE]
+        # identities = [ID_CMD_LIST if self.node == NS_COMMANDS else ID_CMD_NODE] # FIXME
         return [disco.DiscoFeature(NS_COMMANDS)] + self.features
 
     def getDiscoItems(self, requestor, target, nodeIdentifier=''):
@@ -168,7 +170,7 @@ class AdHocCommand(XMPPHandler):
         @param request: original request (domish.Element)
         """
         xmpp_condition, cmd_condition = error_constant
-        iq_elt = xmpp_error.StanzaError(xmpp_condition).toResponse(request)
+        iq_elt = jabber.error.StanzaError(xmpp_condition).toResponse(request)
         if cmd_condition:
             error_elt = iq_elt.elements(None, "error").next()
             error_elt.addElement(cmd_condition, NS_COMMANDS)
@@ -204,10 +206,10 @@ class XEP_0050(object):
                                  'FORBIDDEN', 'ITEM_NOT_FOUND', 'FEATURE_NOT_IMPLEMENTED', 'INTERNAL'))(('bad-request', 'malformed-action'),
                                  ('bad-request', 'bad-action'), ('bad-request', 'bad-locale'), ('bad-request','bad-payload'),
                                  ('bad-request','bad-sessionid'), ('not-allowed','session-expired'), ('forbidden', None),
-                                 ('item-not-found', None), ('feature-not-implemented', None), ('internal-server-error', None)) # XEP-0050 §4.6 Table 5
+                                 ('item-not-found', None), ('feature-not-implemented', None), ('internal-server-error', None)) # XEP-0050 §4.4 Table 5
 
     def __init__(self, host):
-        info(_("plugin XEP-0050 initialization"))
+        log.info(_("plugin XEP-0050 initialization"))
         self.host = host
         self.requesting = Sessions()
         self.answering = {}
@@ -240,6 +242,33 @@ class XEP_0050(object):
         form_ui.addList("node", options)
         return form_ui
 
+    def _getDataLvl(self, type_):
+        """Return the constant corresponding to <note/> type attribute value
+
+        @param type_: note type (see XEP-0050 §4.3)
+        @return: a C.XMLUI_DATA_LVL_* constant
+        """
+        if type_ == 'error':
+            return C.XMLUI_DATA_LVL_ERROR
+        elif type_ == 'warn':
+            return C.XMLUI_DATA_LVL_WARNING
+        else:
+            if type_ != 'info':
+                log.warning(_("Invalid note type [%s], using info") % type_)
+            return C.XMLUI_DATA_LVL_INFO
+
+    def _mergeNotes(self, notes):
+        """Merge notes with level prefix (e.g. "ERROR: the message")
+
+        @param notes (list): list of tuple (level, message)
+        @return: list of messages
+        """
+        lvl_map = {C.XMLUI_DATA_LVL_INFO: '',
+                   C.XMLUI_DATA_LVL_WARNING: "%s: " % _("WARNING"),
+                   C.XMLUI_DATA_LVL_ERROR: "%s: " % _("ERROR")
+                  }
+        return [u"%s%s" % (lvl_map[lvl], msg) for lvl, msg in notes]
+
     def _commandsAnswer2XMLUI(self, iq_elt, session_id, session_data):
         """
         Convert command answer to an ui for frontend
@@ -253,13 +282,51 @@ class XEP_0050(object):
         if status in [XEP_0050.STATUS.COMPLETED, XEP_0050.STATUS.CANCELED]:
             # the command session is finished, we purge our session
             del self.requesting[session_id]
-            return None
+            if status == XEP_0050.STATUS.COMPLETED:
+                session_id = None
+            else:
+                return None
         remote_session_id = command_elt.getAttribute('sessionid')
         if remote_session_id:
             session_data['remote_id'] = remote_session_id
-        data_elt = command_elt.elements(data_form.NS_X_DATA, 'x').next()
+        notes = []
+        for note_elt in command_elt.elements(NS_COMMANDS, 'note'):
+            notes.append((self._getDataLvl(note_elt.getAttribute('type', 'info')),
+                          unicode(note_elt)))
+        try:
+            data_elt = command_elt.elements(data_form.NS_X_DATA, 'x').next()
+        except StopIteration:
+            if status != XEP_0050.STATUS.COMPLETED:
+                log.warning(_("No known payload found in ad-hoc command result, aborting"))
+                del self.requesting[session_id]
+                return xml_tools.XMLUI(C.XMLUI_DIALOG,
+                                       dialog_opt = {C.XMLUI_DATA_TYPE: C.XMLUI_DIALOG_NOTE,
+                                                     C.XMLUI_DATA_MESS: _("No payload found"),
+                                                     C.XMLUI_DATA_LVL: C.XMLUI_DATA_LVL_ERROR,
+                                                    }
+                                      )
+            if not notes:
+                # the status is completed, and we have no note to show
+                return None
+
+            # if we have only one note, we show a dialog with the level of the note
+            # if we have more, we show a dialog with "info" level, and all notes merged
+            dlg_level = notes[0][0] if len(notes) == 1 else C.XMLUI_DATA_LVL_INFO
+            return xml_tools.XMLUI(
+                                   C.XMLUI_DIALOG,
+                                   dialog_opt = {C.XMLUI_DATA_TYPE: C.XMLUI_DIALOG_NOTE,
+                                                 C.XMLUI_DATA_MESS: u'\n'.join(self._mergeNotes(notes)),
+                                                 C.XMLUI_DATA_LVL: dlg_level,
+                                                },
+                                   session_id = session_id
+                                  )
+
+        if session_id is None:
+            return xml_tools.dataFormResult2XMLUI(data_elt)
         form = data_form.Form.fromElement(data_elt)
-        return  xml_tools.dataForm2XMLUI(form, self.__requesting_id, session_id=session_id)
+        # we add any present note to the instructions
+        form.instructions.extend(self._mergeNotes(notes))
+        return xml_tools.dataForm2XMLUI(form, self.__requesting_id, session_id=session_id)
 
     def _requestingEntity(self, data, profile):
         """
@@ -289,21 +356,19 @@ class XEP_0050(object):
             try:
                 session_data = self.requesting.profileGet(data["session_id"], profile)
             except KeyError:
-                warning ("session id doesn't exist, session has probably expired")
+                log.warning ("session id doesn't exist, session has probably expired")
                 # TODO: send error dialog
                 return defer.succeed({})
             session_id = data["session_id"]
             entity = session_data['jid']
             try:
-                node = session_data['node']
+                session_data['node']
                 # node has already been received
             except KeyError:
                 # it's the first time we know the node, we save it in session data
-                node = session_data['node'] = data[xml_tools.SAT_FORM_PREFIX+'node']
+                session_data['node'] = data[xml_tools.SAT_FORM_PREFIX+'node']
 
             client = self.host.getClient(profile)
-            if not client:
-                raise exceptions.ProfileUnknownError
 
             # we request execute node's command
             iq_elt = compat.IQ(client.xmlstream, 'set')
@@ -359,7 +424,7 @@ class XEP_0050(object):
                 x_elt = command_elt.elements(data_form.NS_X_DATA,'x').next()
                 answer_form = data_form.Form.fromElement(x_elt)
                 show = answer_form['show']
-            except KeyError, StopIteration:
+            except (KeyError, StopIteration):
                 raise AdHocError(XEP_0050.ERROR.BAD_PAYLOAD)
             if show not in SHOWS:
                 raise AdHocError(XEP_0050.ERROR.BAD_PAYLOAD)
@@ -396,10 +461,8 @@ class XEP_0050(object):
         return d
 
     def addAdHocCommand(self, callback, label, node="", features = None, timeout = 600, allowed_jids = None, allowed_groups = None,
-                        allowed_magics = None, forbidden_jids = None, forbidden_groups = None, profile_key="@NONE@"):
-        """
-
-        Add an ad-hoc command for the current profile
+                        allowed_magics = None, forbidden_jids = None, forbidden_groups = None, profile_key=C.PROF_KEY_NONE):
+        """Add an ad-hoc command for the current profile
 
         @param callback: method associated with this ad-hoc command which return the payload data (see AdHocCommand._sendAnswer), can return a deferred
         @param label: label associated with this command on the main menu
