@@ -19,9 +19,10 @@
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 from sat.core.i18n import _
-from logging import debug, info, error
-from twisted.internet import threads
-from twisted.internet.defer import inlineCallbacks, returnValue
+from sat.core.constants import Const as C
+from sat.core.log import getLogger
+log = getLogger(__name__)
+from twisted.internet import threads, defer
 from twisted.words.protocols.jabber import jid
 from twisted.words.protocols.jabber.xmlstream import IQ
 from twisted.words.xish import domish
@@ -45,6 +46,7 @@ except ImportError:
     from wokkel.subprotocols import XMPPHandler
 
 AVATAR_PATH = "avatars"
+AVATAR_DIM = (64, 64)
 
 IQ_GET = '/iq[@type="get"]'
 NS_VCARD = 'vcard-temp'
@@ -72,7 +74,7 @@ class XEP_0054(object):
     #      - get missing values
 
     def __init__(self, host):
-        info(_("Plugin XEP_0054 initialization"))
+        log.info(_("Plugin XEP_0054 initialization"))
         self.host = host
         self.avatar_path = os.path.join(self.host.memory.getConfig('', 'local_dir'), AVATAR_PATH)
         if not os.path.exists(self.avatar_path):
@@ -125,14 +127,14 @@ class XEP_0054(object):
             if name == "avatar":
                 self.avatars_cache[jid.userhost()] = value
 
-    def get_cache(self, jid, name, profile):
+    def get_cache(self, entity_jid, name, profile):
         """return cached value for jid
-        @param jid: target contact
+        @param entity_jid: target contact
         @param name: name of the value ('nick' or 'avatar')
         @param profile: %(doc_profile)s
         @return: wanted value or None"""
         try:
-            data = self.host.memory.getEntityData(jid, [name], profile)
+            data = self.host.memory.getEntityData(entity_jid, [name], profile)
         except exceptions.UnknownEntityError:
             return None
         return data.get(name)
@@ -141,24 +143,24 @@ class XEP_0054(object):
         """Parse a <PHOTO> elem and save the picture"""
         for elem in photo_xml.elements():
             if elem.name == 'TYPE':
-                info(_('Photo of type [%s] found') % str(elem))
+                log.info(_('Photo of type [%s] found') % str(elem))
             if elem.name == 'BINVAL':
-                debug(_('Decoding binary'))
+                log.debug(_('Decoding binary'))
                 decoded = b64decode(str(elem))
                 image_hash = sha1(decoded).hexdigest()
                 filename = self.avatar_path + '/' + image_hash
                 if not os.path.exists(filename):
-                    with open(filename, 'wb') as file:
-                        file.write(decoded)
-                    debug(_("file saved to %s") % image_hash)
+                    with open(filename, 'wb') as file_:
+                        file_.write(decoded)
+                    log.debug(_("file saved to %s") % image_hash)
                 else:
-                    debug(_("file [%s] already in cache") % image_hash)
+                    log.debug(_("file [%s] already in cache") % image_hash)
                 return image_hash
 
-    @inlineCallbacks
+    @defer.inlineCallbacks
     def vCard2Dict(self, vcard, target, profile):
         """Convert a VCard to a dict, and save binaries"""
-        debug(_("parsing vcard"))
+        log.debug(_("parsing vcard"))
         dictionary = {}
 
         for elem in vcard.elements():
@@ -180,13 +182,13 @@ class XEP_0054(object):
                 else:
                     self.update_cache(target, 'avatar', dictionary['avatar'], profile)
             else:
-                info(_('FIXME: [%s] VCard tag is not managed yet') % elem.name)
+                log.info(_('FIXME: [%s] VCard tag is not managed yet') % elem.name)
 
-        returnValue(dictionary)
+        defer.returnValue(dictionary)
 
     def vcard_ok(self, answer, profile):
         """Called after the first get IQ"""
-        debug(_("VCard found"))
+        log.debug(_("VCard found"))
 
         if answer.firstChildElement().name == "vCard":
             _jid, steam = self.host.getJidNStream(profile)
@@ -197,26 +199,26 @@ class XEP_0054(object):
             d = self.vCard2Dict(answer.firstChildElement(), from_jid, profile)
             d.addCallback(lambda data: self.host.bridge.actionResult("RESULT", answer['id'], data, profile))
         else:
-            error(_("FIXME: vCard not found as first child element"))
+            log.error(_("FIXME: vCard not found as first child element"))
             self.host.bridge.actionResult("SUPPRESS", answer['id'], {}, profile)  # FIXME: maybe an error message would be better
 
     def vcard_err(self, failure, profile):
         """Called when something is wrong with registration"""
         if failure.value.stanza.hasAttribute("from"):
-            error(_("Can't find VCard of %s") % failure.value.stanza['from'])
+            log.error(_("Can't find VCard of %s") % failure.value.stanza['from'])
         self.host.bridge.actionResult("SUPPRESS", failure.value.stanza['id'], {}, profile)  # FIXME: maybe an error message would be better
 
-    def getCard(self, target_s, profile_key='@DEFAULT@'):
+    def getCard(self, target_s, profile_key=C.PROF_KEY_NONE):
         """Ask server for VCard
         @param target_s: jid from which we want the VCard
         @result: id to retrieve the profile"""
         current_jid, xmlstream = self.host.getJidNStream(profile_key)
         if not xmlstream:
-            error(_('Asking vcard for a non-existant or not connected profile'))
+            log.error(_('Asking vcard for a non-existant or not connected profile'))
             return ""
         profile = self.host.memory.getProfileName(profile_key)
         to_jid = jid.JID(target_s)
-        debug(_("Asking for %s's VCard") % to_jid.userhost())
+        log.debug(_("Asking for %s's VCard") % to_jid.userhost())
         reg_request = IQ(xmlstream, 'get')
         reg_request["from"] = current_jid.full()
         reg_request["to"] = to_jid.userhost()
@@ -231,7 +233,7 @@ class XEP_0054(object):
         """
         filename = self.avatar_path + '/' + avatar_hash
         if not os.path.exists(filename):
-            error(_("Asking for an uncached avatar [%s]") % avatar_hash)
+            log.error(_("Asking for an uncached avatar [%s]") % avatar_hash)
             return ""
         return filename
 
@@ -241,8 +243,19 @@ class XEP_0054(object):
         except IOError:
             return Failure(exceptions.DataError("Can't open image"))
 
-        if img.size != (64, 64):
-            img = img.resize((64, 64))
+        if img.size != AVATAR_DIM:
+            img.thumbnail(AVATAR_DIM, Image.ANTIALIAS)
+            if img.size[0] != img.size[1]:  # we need to crop first
+                left, upper = (0, 0)
+                right, lower = img.size
+                offset = abs(right - lower) / 2
+                if right == min(img.size):
+                    upper += offset
+                    lower -= offset
+                else:
+                    left += offset
+                    right -= offset
+                img = img.crop((left, upper, right, lower))
         img_buf = StringIO()
         img.save(img_buf, 'PNG')
 
@@ -253,14 +266,12 @@ class XEP_0054(object):
         img_hash = sha1(img_buf.getvalue()).hexdigest()
         return (vcard_set, img_hash)
 
-    def setAvatar(self, filepath, profile_key='@DEFAULT@'):
+    def setAvatar(self, filepath, profile_key=C.PROF_KEY_NONE):
         """Set avatar of the profile
         @param filepath: path of the image of the avatar"""
-        #TODO: This is a temporary way of setting avatar, as other VCard informations are not managed.
+        #TODO: This is a temporary way of setting the avatar, as other VCard information is not managed.
         #      A proper full VCard management should be done (and more generaly a public/private profile)
         client = self.host.getClient(profile_key)
-        if not client:
-            raise exceptions.NotConnectedProfileError(_('Trying to set avatar for a non-existant or not connected profile'))
 
         vcard_set = IQ(client.xmlstream, 'set')
         d = threads.deferToThread(self._buildSetAvatar, vcard_set, filepath)
@@ -305,5 +316,5 @@ class XEP_0054_handler(XMPPHandler):
                 _hash = str(elem)
                 old_avatar = self.plugin_parent.get_cache(from_jid, 'avatar', self.parent.profile)
                 if not old_avatar or old_avatar != _hash:
-                    debug(_('New avatar found, requesting vcard'))
+                    log.debug(_('New avatar found, requesting vcard'))
                     self.plugin_parent.getCard(from_jid.userhost(), self.parent.profile)
