@@ -18,13 +18,11 @@
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 from sat.core.i18n import _
-from logging import debug, info, error
+from sat.core.constants import Const as C
+from sat.core.log import getLogger
+log = getLogger(__name__)
 from twisted.words.xish import domish
-from twisted.internet import protocol
-from twisted.words.protocols.jabber import client, jid
-from twisted.words.protocols.jabber import error as jab_error
-import os.path
-from twisted.internet import reactor
+from twisted.words.protocols.jabber import client
 import uuid
 
 from zope.interface import implements
@@ -55,7 +53,7 @@ PLUGIN_INFO = {
 class XEP_0095(object):
 
     def __init__(self, host):
-        info(_("Plugin XEP_0095 initialization"))
+        log.info(_("Plugin XEP_0095 initialization"))
         self.host = host
         self.si_profiles = {}  # key: SI profile, value: callback
 
@@ -72,7 +70,7 @@ class XEP_0095(object):
         """This method is called on stream initiation (XEP-0095 #3.2)
         @param iq_el: IQ element
         @param profile: %(doc_profile)s"""
-        info(_("XEP-0095 Stream initiation"))
+        log.info(_("XEP-0095 Stream initiation"))
         iq_el.handled = True
         si_el = iq_el.firstChildElement()
         si_id = si_el.getAttribute('id')
@@ -86,7 +84,7 @@ class XEP_0095(object):
             #We don't know this profile, we send an error
             self.sendBadProfileError(iq_el['id'], iq_el['from'], profile)
 
-    def sendRejectedError(self, iq_id, to_jid, reason='Offer Declined', profile='@NONE@'):
+    def sendRejectedError(self, iq_id, to_jid, reason='Offer Declined', profile=C.PROF_KEY_NONE):
         """Helper method to send when the stream is rejected
         @param iq_id: IQ id
         @param to_jid: recipient
@@ -113,9 +111,9 @@ class XEP_0095(object):
         @param iq_id: IQ id
         @param to_jid: recipient
         @param profile: %(doc_profile)s"""
-        self.sendError(iq_id, to_jid, 500, 'cancel', {'custom': 'failed'}, profile=profile)  # as there is no error code for failed transfer, we use 500 (undefined-condition)
+        self.sendError(iq_id, to_jid, 500, 'cancel', {'custom': 'failed'}, profile=profile)  # as there is no lerror code for failed transfer, we use 500 (undefined-condition)
 
-    def sendError(self, iq_id, to_jid, err_code, err_type='cancel', data={}, profile='@NONE@'):
+    def sendError(self, iq_id, to_jid, err_code, err_type='cancel', data={}, profile=C.PROF_KEY_NONE):
         """Send IQ error as a result
         @param iq_id: IQ id
         @param to_jid: recipient
@@ -124,8 +122,7 @@ class XEP_0095(object):
         @param data: error specific data (dictionary)
         @param profile: %(doc_profile)s
         """
-        _client = self.host.getClient(profile)
-        assert(_client)
+        client_ = self.host.getClient(profile)
         result = domish.Element((None, 'iq'))
         result['type'] = 'result'
         result['id'] = iq_id
@@ -148,9 +145,9 @@ class XEP_0095(object):
             if 'custom' in data and data['custom'] == 'failed':
                 condition_el.addContent('Stream failed')
 
-        _client.xmlstream.send(result)
+        client_.xmlstream.send(result)
 
-    def acceptStream(self, iq_id, to_jid, feature_elt, misc_elts=[], profile='@NONE@'):
+    def acceptStream(self, iq_id, to_jid, feature_elt, misc_elts=[], profile=C.PROF_KEY_NONE):
         """Send the accept stream initiation answer
         @param iq_id: IQ id
         @param feature_elt: domish element 'feature' containing stream method to use
@@ -158,7 +155,7 @@ class XEP_0095(object):
         @param profile: %(doc_profile)s"""
         _client = self.host.getClient(profile)
         assert(_client)
-        info(_("sending stream initiation accept answer"))
+        log.info(_("sending stream initiation accept answer"))
         result = domish.Element((None, 'iq'))
         result['type'] = 'result'
         result['id'] = iq_id
@@ -169,7 +166,7 @@ class XEP_0095(object):
             si.addChild(elt)
         _client.xmlstream.send(result)
 
-    def proposeStream(self, to_jid, si_profile, feature_elt, misc_elts, mime_type='application/octet-stream', profile_key='@NONE@'):
+    def proposeStream(self, to_jid, si_profile, feature_elt, misc_elts, mime_type='application/octet-stream', profile_key=C.PROF_KEY_NONE):
         """Propose a stream initiation
         @param to_jid: recipient (JID)
         @param si_profile: Stream initiation profile (XEP-0095)
@@ -180,12 +177,12 @@ class XEP_0095(object):
         @return: session id, offer"""
         current_jid, xmlstream = self.host.getJidNStream(profile_key)
         if not xmlstream:
-            error(_('Asking for an non-existant or not connected profile'))
+            log.error(_('Asking for an non-existant or not connected profile'))
             return ""
 
         offer = client.IQ(xmlstream, 'set')
         sid = str(uuid.uuid4())
-        debug(_("Stream Session ID: %s") % offer["id"])
+        log.debug(_("Stream Session ID: %s") % offer["id"])
 
         offer["from"] = current_jid.full()
         offer["to"] = to_jid.full()
