@@ -18,11 +18,12 @@
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 from sat.core.i18n import _, D_
+from sat.core.constants import Const as C
 from sat.core import exceptions
 from sat.tools import xml_tools
-from logging import debug, info, warning, error
-from twisted.words.protocols.jabber import client as jabber_client, jid
-from twisted.words.protocols.jabber import error as jab_error
+from sat.core.log import getLogger
+log = getLogger(__name__)
+from twisted.words.protocols.jabber import jid
 from twisted.internet import reactor, defer
 
 PLUGIN_INFO = {
@@ -38,24 +39,24 @@ PLUGIN_INFO = {
 WARNING_MSG = D_(u"""Be careful ! Gateways allow you to use an external IM (legacy IM), so you can see your contact as XMPP contacts.
 But when you do this, all your messages go throught the external legacy IM server, it is a huge privacy issue (i.e.: all your messages throught the gateway can be monitored, recorded, analysed by the external server, most of time a private company).""")
 
-GATEWAY_TIMEOUT = 10 # time to wait before cancelling a gateway disco info, in seconds
+GATEWAY_TIMEOUT = 10  # time to wait before cancelling a gateway disco info, in seconds
 
-TYPE_DESCRIPTIONS = { 'irc': D_("Internet Relay Chat"),
-                      'xmpp': D_("XMPP"),
-                      'qq': D_("Tencent QQ"),
-                      'simple': D_("SIP/SIMPLE"),
-                      'icq': D_("ICQ"),
-                      'yahoo': D_("Yahoo! Messenger"),
-                      'gadu-gadu': D_("Gadu-Gadu"),
-                      'aim': D_("AOL Instant Messenger"),
-                      'msn': D_("Windows Live Messenger"),
+TYPE_DESCRIPTIONS = {'irc': D_("Internet Relay Chat"),
+                     'xmpp': D_("XMPP"),
+                     'qq': D_("Tencent QQ"),
+                     'simple': D_("SIP/SIMPLE"),
+                     'icq': D_("ICQ"),
+                     'yahoo': D_("Yahoo! Messenger"),
+                     'gadu-gadu': D_("Gadu-Gadu"),
+                     'aim': D_("AOL Instant Messenger"),
+                     'msn': D_("Windows Live Messenger"),
                     }
 
 
 class XEP_0100(object):
 
     def __init__(self, host):
-        info(_("Gateways plugin initialization"))
+        log.info(_("Gateways plugin initialization"))
         self.host = host
         self.__gateways = {}  # dict used to construct the answer to findGateways. Key = target jid
         host.bridge.addMethod("findGateways", ".plugin", in_sign='ss', out_sign='s', method=self._findGateways)
@@ -64,7 +65,6 @@ class XEP_0100(object):
         self.__selected_id = host.registerCallback(self._gatewaySelectedCb, with_data=True)
         host.importMenu((D_("Service"), D_("gateways")), self._gatewaysMenu, security_limit=2, help_string=D_("Find gateways"))
 
-
     def _gatewaysMenu(self, data, profile):
         """ XMLUI activated by menu: return Gateways UI
         @param profile: %(doc_profile)s
@@ -111,7 +111,7 @@ class XEP_0100(object):
         try:
             target_jid = jid.JID(data['index'])
         except (KeyError, RuntimeError):
-            warning(_("No gateway index selected"))
+            log.warning(_("No gateway index selected"))
             return {}
 
         d = self.gatewayRegister(target_jid, profile)
@@ -125,7 +125,7 @@ class XEP_0100(object):
         """
         category, type_ = identity
         if category != 'gateway':
-            error(_('INTERNAL ERROR: identity category should always be "gateway" in _getTypeString, got "%s"') % category)
+            log.error(_('INTERNAL ERROR: identity category should always be "gateway" in _getTypeString, got "%s"') % category)
         try:
             return _(TYPE_DESCRIPTIONS[type_])
         except KeyError:
@@ -133,16 +133,16 @@ class XEP_0100(object):
 
     def _registrationSuccessful(self, jid_, profile):
         """Called when in_band registration is ok, we must now follow the rest of procedure"""
-        debug(_("Registration successful, doing the rest"))
+        log.debug(_("Registration successful, doing the rest"))
         self.host.addContact(jid_, profile_key=profile)
         self.host.setPresence(jid_, profile_key=profile)
 
-    def _gatewayRegister(self, target_jid_s, profile_key='@NONE@'):
+    def _gatewayRegister(self, target_jid_s, profile_key=C.PROF_KEY_NONE):
         d = self.gatewayRegister(jid.JID(target_jid_s), profile_key)
         d.addCallback(lambda xmlui: xmlui.toXml())
         return d
 
-    def gatewayRegister(self, target_jid, profile_key='@NONE@'):
+    def gatewayRegister(self, target_jid, profile_key=C.PROF_KEY_NONE):
         """Register gateway using in-band registration, then log-in to gateway"""
         profile = self.host.memory.getProfileName(profile_key)
         assert(profile)
@@ -160,36 +160,35 @@ class XEP_0100(object):
                 else:
                     try:
                         msg = result.value.condition
-                    except AttibuteError:
+                    except AttributeError:
                         msg = str(result)
                 ret.append((success, (msg, items[idx])))
             else:
                 entity = items[idx].entity
                 gateways = [(identity, result.identities[identity]) for identity in result.identities if identity[0] == 'gateway']
                 if gateways:
-                    info (_("Found gateway [%(jid)s]: %(identity_name)s") % {'jid': entity.full(), 'identity_name': ' - '.join([gateway[1] for gateway in gateways])})
+                    log.info(_("Found gateway [%(jid)s]: %(identity_name)s") % {'jid': entity.full(), 'identity_name': ' - '.join([gateway[1] for gateway in gateways])})
                     ret.append((success, (entity, gateways)))
                 else:
-                    info(_("Skipping [%(jid)s] which is not a gateway") % {'jid': entity.full()})
+                    log.info(_("Skipping [%(jid)s] which is not a gateway") % {'jid': entity.full()})
         return ret
 
     def _itemsReceived(self, disco, target, client):
         """Look for items with disco protocol, and ask infos for each one"""
 
         if len(disco._items) == 0:
-            debug(_("No gateway found"))
+            log.debug(_("No gateway found"))
             return []
 
         _defers = []
         for item in disco._items:
-            debug(_("item found: %s"), item.entity)
+            log.debug(_("item found: %s") % item.entity)
             _defers.append(client.disco.requestInfo(item.entity))
         dl = defer.DeferredList(_defers)
         dl.addCallback(self._infosReceived, items=disco._items, target=target, client=client)
         reactor.callLater(GATEWAY_TIMEOUT, dl.cancel)
         return dl
 
-
     def _findGateways(self, target_jid_s, profile_key):
         target_jid = jid.JID(target_jid_s)
         profile = self.host.memory.getProfileName(profile_key)
@@ -200,13 +199,11 @@ class XEP_0100(object):
         d.addCallback(lambda xmlui: xmlui.toXml())
         return d
 
-
     def findGateways(self, target, profile):
         """Find gateways in the target JID, using discovery protocol
         """
         client = self.host.getClient(profile)
-        assert(client)
-        debug(_("find gateways (target = %(target)s, profile = %(profile)s)") % {'target': target.full(), 'profile': profile})
+        log.debug(_("find gateways (target = %(target)s, profile = %(profile)s)") % {'target': target.full(), 'profile': profile})
         d = client.disco.requestItems(target)
-        d.addCallback(self._itemsReceived , target=target, client=client)
+        d.addCallback(self._itemsReceived, target=target, client=client)
         return d
