@@ -18,14 +18,15 @@
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 from sat.core.i18n import _
-from logging import debug, info, warning, error
+from sat.core.constants import Const as C
+from sat.core.log import getLogger
+log = getLogger(__name__)
 from twisted.words.xish import domish
-from twisted.internet import protocol, defer
-from twisted.words.protocols.jabber import client, jid, xmlstream
+from twisted.words.protocols.jabber import jid
 
 from zope.interface import implements
 
-from wokkel import disco, iwokkel, data_form
+from wokkel import disco, iwokkel
 
 
 try:
@@ -46,6 +47,7 @@ PLUGIN_INFO = {
     "type": "XEP",
     "protocols": ["XEP-0249"],
     "dependencies": ["XEP-0045"],
+    "recommendations": [C.TEXT_CMDS],
     "main": "XEP_0249",
     "handler": "yes",
     "description": _("""Implementation of Direct MUC Invitations""")
@@ -58,7 +60,7 @@ class XEP_0249(object):
     <params>
     <individual>
     <category name="%(category_name)s" label="%(category_label)s">
-        <param name="%(param_name)s" label="%(param_label)s" value="%(param_default)s" type="list" security="0">
+        <param name="%(param_name)s" label="%(param_label)s" type="list" security="0">
             %(param_options)s
         </param>
      </category>
@@ -69,20 +71,25 @@ class XEP_0249(object):
         'category_label': _("Misc"),
         'param_name': AUTOJOIN_NAME,
         'param_label': _("Auto-join MUC on invitation"),
-        'param_default': AUTOJOIN_VALUES[0],
-        'param_options': ['<option value="%s"/>' % value for value in AUTOJOIN_VALUES]
+        'param_options': '\n'.join(['<option value="%s" %s/>' % \
+                                   (value, 'selected="true"' if value == AUTOJOIN_VALUES[0] else '') \
+                                   for value in AUTOJOIN_VALUES])
     }
 
     def __init__(self, host):
-        info(_("Plugin XEP_0249 initialization"))
+        log.info(_("Plugin XEP_0249 initialization"))
         self.host = host
         host.memory.updateParams(self.params)
         host.bridge.addMethod("inviteMUC", ".plugin", in_sign='sssa{ss}s', out_sign='', method=self._invite)
+        try:
+            self.host.plugins[C.TEXT_CMDS].registerTextCommands(self)
+        except KeyError:
+            log.info(_("Text commands not available"))
 
     def getHandler(self, profile):
         return XEP_0249_handler(self)
 
-    def invite(self, target, room, options={}, profile_key='@DEFAULT@'):
+    def invite(self, target, room, options={}, profile_key=C.PROF_KEY_NONE):
         """
         Invite a user to a room
         @param target: jid of the user to invite
@@ -92,7 +99,7 @@ class XEP_0249(object):
         """
         profile = self.host.memory.getProfileName(profile_key)
         if not profile:
-            error(_("Profile doesn't exists !"))
+            log.error(_("Profile doesn't exists !"))
             return
         message = domish.Element((None, 'message'))
         message["to"] = target.full()
@@ -102,7 +109,7 @@ class XEP_0249(object):
             x_elt[opt] = options[opt]
         self.host.profiles[profile].xmlstream.send(message)
 
-    def _invite(self, target, service, roomId, options={}, profile_key='@DEFAULT@'):
+    def _invite(self, target, service, roomId, options={}, profile_key=C.PROF_KEY_NONE):
         """
         Invite an user to a room
         @param target: jid of the user to invite
@@ -113,18 +120,19 @@ class XEP_0249(object):
         #TODO: check parameters validity
         self.invite(jid.JID(target), jid.JID("%s@%s" % (roomId, service)), options, profile_key)
 
-    def _accept(self, room, profile_key='@DEFAULT@'):
+    def _accept(self, room, profile_key=C.PROF_KEY_NONE):
         """
         Accept the invitation to join a MUC
         @param room: room jid as string
         """
         profile = self.host.memory.getProfileName(profile_key)
         if not profile:
-            error(_("Profile doesn't exists !"))
+            log.error(_("Profile doesn't exists !"))
             return
-        info(_('Invitation accepted for room %(room)s [%(profile)s]') % {'room': room, 'profile': profile})
+        log.info(_('Invitation accepted for room %(room)s [%(profile)s]') % {'room': room, 'profile': profile})
         _jid, xmlstream = self.host.getJidNStream(profile)
         d = self.host.plugins["XEP-0045"].join(jid.JID(room), _jid.user, {}, profile)
+        return d
 
     def onInvitation(self, message, profile):
         """
@@ -134,13 +142,13 @@ class XEP_0249(object):
         """
         try:
             room = message.firstChildElement()['jid']
-            info(_('Invitation received for room %(room)s [%(profile)s]') % {'room': room, 'profile': profile})
+            log.info(_('Invitation received for room %(room)s [%(profile)s]') % {'room': room, 'profile': profile})
         except:
-            error(_('Error while parsing invitation'))
+            log.error(_('Error while parsing invitation'))
             return
         from_ = message["from"]
         if room in self.host.plugins["XEP-0045"].clients[profile].joined_rooms:
-            info(_("Invitation silently discarded because user is already in the room."))
+            log.info(_("Invitation silently discarded because user is already in the room."))
             return
         autojoin = self.host.memory.getParamA(AUTOJOIN_NAME, AUTOJOIN_KEY, profile_key=profile)
 
@@ -150,11 +158,37 @@ class XEP_0249(object):
 
         if autojoin == "always":
             self._accept(room, profile)
-        elif autojoin == "ask":
+        elif autojoin == "never":
+            self.host.bridge.newAlert(_("An invitation from %(user)s to join the room %(room)s has been declined according to your personal settings.") % {'user': from_, 'room': room}, _("MUC invitation"), "INFO", profile)
+        else:  # leave the default value here
             data = {"message": _("You have been invited by %(user)s to join the room %(room)s. Do you accept?") % {'user': from_, 'room': room}, "title": _("MUC invitation")}
             self.host.askConfirmation(room, "YES/NO", data, accept_cb, profile)
-        else:
-            self.host.bridge.newAlert(_("An invitation from %(user)s to join the room %(room)s has been declined according to your personal settings.") % {'user': from_, 'room': room}, _("MUC invitation"), "INFO", profile)
+
+    def cmd_invite(self, mess_data, profile):
+        """invite someone in the room
+
+        @command (group): (JID)
+            - JID: the JID of the person to invite
+        """
+        log.debug("Catched invite command")
+
+        if mess_data['type'] != "groupchat":
+            self.host.plugins[C.TEXT_CMDS].feedBackWrongContext('invite', 'groupchat', mess_data, profile)
+            return False
+
+        jid_s = mess_data["unparsed"].strip()
+        try:
+            assert(jid_s)
+            jid_ = jid.JID(jid_s)
+            assert(jid_.user)
+            assert(jid_.host)
+        except (jid.InvalidFormat, AssertionError):
+            feedback = _(u"You must provide a valid JID to invite, like in '/invite contact@example.net'")
+            self.host.plugins[C.TEXT_CMDS].feedBack(feedback, mess_data, profile)
+            return False
+
+        self.invite(jid_, mess_data["to"], {}, profile)
+        return False
 
 
 class XEP_0249_handler(XMPPHandler):
