@@ -17,7 +17,11 @@
 # You should have received a copy of the GNU Affero General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
-from sat.core.i18n import _
+
+## logging configuration for tests ##
+from sat.core import log_config
+log_config.satConfigure()
+
 from sat.core import exceptions
 from constants import Const
 from wokkel.xmppim import RosterItem
@@ -77,49 +81,43 @@ class FakeSAT(object):
     def registerCallback(self, callback, *args, **kwargs):
         pass
 
-    def registerNewAccountCB(self, data, profile):
-        pass
-
     def sendMessage(self, to_s, msg, subject=None, mess_type='auto', extra={}, profile_key='@NONE@'):
         self.sendAndStoreMessage({"to": JID(to_s)})
 
-    def sendAndStoreMessage(self, mess_data, skip_send=False, profile=None):
-        """Save the information to check later to whom messages have been sent and
-        if entries have been added to the history"""
-        if not skip_send:
-            self.sent_messages.append(mess_data["to"])
-        self.stored_messages.append(mess_data["to"])
-        pass
+    def _sendMessageToStream(self, mess_data, client):
+        """Save the information to check later to whom messages have been sent.
 
-    def requestServerDisco(self, feature, jid_=None, cache_only=False, profile_key="@NONE"):
-        """Discover if a server or its items offer a given feature
-        @param feature: the feature to check
-        @param jid_: the jid of the server, local server if None
-        @param cache_only: expect the result to be in cache and don't actually
-        make any request. This can be used anytime for requesting a feature on
-        the local server because the data are cached for sure.
-        @result: the Deferred entity jid offering the feature, or None
+        @param mess_data: message data dictionnary
+        @param client: profile's client
         """
-        profile = self.memory.getProfileName(profile_key)
-        self.memory.server_features.setdefault(profile, {})
-        if jid_ is None:
-            jid_ = self.getClientHostJid(profile_key)
-        # call FakeMemory.init and FakeMemory.addServerFeature
-        # in your tests to change the return value of this method
-        return defer.succeed(jid_ if self.memory.hasServerFeature(feature, jid_, profile_key) else None)
+        self.sent_messages.append(mess_data["to"])
+        return mess_data
 
-    def getClientHostJid(self, profile_key):
-        return Const.PROFILE_DICT[profile_key].host
+    def _storeMessage(self, mess_data, client):
+        """Save the information to check later if entries have been added to the history.
+
+        @param mess_data: message data dictionnary
+        @param client: profile's client
+        """
+        self.stored_messages.append(mess_data["to"])
+        return mess_data
+
+    def sendMessageToBridge(self, mess_data, client):
+        """Simulate the message being sent to the frontends.
+
+        @param mess_data: message data dictionnary
+        @param client: profile's client
+        """
+        return mess_data  # TODO
 
     def getClient(self, profile_key):
         """Convenient method to get client from profile key
         @return: client or None if it doesn't exist"""
         profile = self.memory.getProfileName(profile_key)
         if not profile:
-            return None
+            raise exceptions.ProfileKeyUnknownError
         if profile not in self.profiles:
             self.profiles[profile] = FakeClient(self, profile)
-            self.profiles[profile].client_initialized.callback(None)
         return self.profiles[profile]
 
     def getJidNStream(self, profile_key):
@@ -148,6 +146,33 @@ class FakeSAT(object):
         entry = self.getSentMessageRaw(profile_index)
         return entry.toXml() if entry else None
 
+    def findFeaturesSet(self, features, category=None, type_=None, jid_=None, profile_key=None):
+        """Call self.addFeature from your tests to change the return value.
+
+        @return: a set of entities
+        """
+        client = self.getClient(profile_key)
+        if jid_ is None:
+            jid_ = JID(client.jid.host)
+        try:
+            if set(features).issubset(client.features[jid_]):
+                return defer.succeed(set([jid_]))
+        except (TypeError, AttributeError, KeyError):
+            pass
+        return defer.succeed(set())
+
+    def addFeature(self, jid_, feature, profile_key):
+        """Add a feature to an entity.
+
+        To be called from your tests when needed.
+        """
+        client = self.getClient(profile_key)
+        if not hasattr(client, 'features'):
+            client.features = {}
+        if jid_ not in client.features:
+            client.features[jid_] = set()
+        client.features[jid_].add(feature)
+
 
 class FakeBridge(object):
     """Class to simulate and test bridge calls"""
@@ -242,7 +267,6 @@ class FakeMemory(Memory):
         self.params.params.clear()
         self.params.frontends_cache = []
         self.entities_data = {}
-        self.server_features = {}
 
     def getProfileName(self, profile_key, return_profile_keys=False):
         return self.params.getProfileName(profile_key, return_profile_keys)
@@ -324,7 +348,6 @@ class FakeClient(object):
         self.profile = profile if profile else Const.PROFILE[0]
         self.jid = Const.PROFILE_DICT[self.profile]
         self.roster = FakeRosterProtocol(host, self)
-        self.client_initialized = defer.Deferred()
         self.xmlstream = FakeXmlStream()
 
     def send(self, obj):
