@@ -20,9 +20,7 @@
 from sat.test import helpers
 from constants import Const
 from twisted.trial import unittest
-from sat.core.sat_main import SAT
 from sat.core import xmpp
-from twisted.internet import defer
 from twisted.words.protocols.jabber.jid import JID
 from wokkel.generic import parseXml
 from wokkel.xmppim import RosterItem
@@ -39,7 +37,6 @@ class SatXMPPClientTest(unittest.TestCase):
         self.assertEqual(self.client.profile, Const.PROFILE[0])
         print self.client.host
         self.assertEqual(self.client.host_app, self.host)
-        self.assertTrue(isinstance(self.client.client_initialized, defer.Deferred))
 
 
 class SatMessageProtocolTest(unittest.TestCase):
