@@ -18,12 +18,15 @@
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 from sat.core.i18n import _
-from logging import debug, info, error, warning
+from sat.core.constants import Const as C
+from sat.core.log import getLogger
+log = getLogger(__name__)
 from xml.dom import minidom, NotFoundErr
 from wokkel import data_form
 from twisted.words.xish import domish
 from sat.core import exceptions
 
+
 """This library help manage XML used in SàT (parameters, registration, etc) """
 
 SAT_FORM_PREFIX = "SAT_FORM_"
@@ -32,10 +35,11 @@ SAT_PARAM_SEPARATOR = "_XMLUI_PARAM_" # used to have unique elements names
 
 # Helper functions
 
-def _dataFormField2XMLUIData(field):
+def _dataFormField2XMLUIData(field, read_only=False):
     """ Get data needed to create an XMLUI's Widget from Wokkel's data_form's Field
     field can be modified (if it's fixed and it has no value)
     @param field: data_form.Field (it uses field.value, field.fieldType, field.label and field.var)
+    @param read_only: if True and it make sens, create a read only input widget
     @return: widget_type, widget_args, widget_kwargs
 
     """
@@ -45,7 +49,7 @@ def _dataFormField2XMLUIData(field):
         widget_type = 'text'
         if field.value is None:
             if field.label is None:
-                warning(_("Fixed field has neither value nor label, ignoring it"))
+                log.warning(_("Fixed field has neither value nor label, ignoring it"))
                 field.value = ""
             else:
                 field.value = field.label
@@ -53,41 +57,48 @@ def _dataFormField2XMLUIData(field):
             widget_args[0] = field.value
     elif field.fieldType == 'text-single':
         widget_type = "string"
+        widget_kwargs['read_only'] = read_only
+    elif field.fieldType == 'text-multi':
+        widget_type = "textbox"
+        widget_args[0] = u'\n'.join(field.values)
+        widget_kwargs['read_only'] = read_only
     elif field.fieldType == 'text-private':
         widget_type = "password"
+        widget_kwargs['read_only'] = read_only
     elif field.fieldType == 'boolean':
         widget_type = "bool"
         if widget_args[0] is None:
             widget_args[0] = 'false'
+        widget_kwargs['read_only'] = read_only
     elif field.fieldType == 'list-single':
         widget_type = "list"
-        del widget_args[0]
         widget_kwargs["options"] = [(option.value, option.label or option.value) for option in field.options]
+        widget_kwargs["selected"] = widget_args
+        widget_args = []
     else:
-        error(u"FIXME FIXME FIXME: Type [%s] is not managed yet by SàT" % field.fieldType)
+        log.error(u"FIXME FIXME FIXME: Type [%s] is not managed yet by SàT" % field.fieldType)
         widget_type = "string"
+        widget_kwargs['read_only'] = read_only
 
     if field.var:
         widget_kwargs["name"] = field.var
 
     return widget_type, widget_args, widget_kwargs
 
+def dataForm2Widgets(form_ui, form, read_only=False):
+    """Complete an existing XMLUI with widget converted frot XEP-0004 data forms
 
-def dataForm2XMLUI(form, submit_id, session_id=None):
-    """Take a data form (xep-0004, Wokkel's implementation) and convert it to a SàT XML
-    @param submit_id: callback id to call when submitting form
-    @param session_id: id to return with the data
-
+    @param form_ui: XMLUI instance
+    @param form: Wokkel's implementation of data form
+    @return: completed xml_ui
     """
-    form_ui = XMLUI("form", "vertical", submit_id=submit_id, session_id=session_id)
-
     if form.instructions:
         form_ui.addText('\n'.join(form.instructions), 'instructions')
 
     form_ui.changeContainer("pairs")
 
     for field in form.fieldList:
-        widget_type, widget_args, widget_kwargs = _dataFormField2XMLUIData(field)
+        widget_type, widget_args, widget_kwargs = _dataFormField2XMLUIData(field, read_only)
         label = field.label or field.var
         if label:
             form_ui.addLabel(label)
@@ -98,6 +109,15 @@ def dataForm2XMLUI(form, submit_id, session_id=None):
 
     return form_ui
 
+def dataForm2XMLUI(form, submit_id, session_id=None, read_only=False):
+    """Take a data form (xep-0004, Wokkel's implementation) and convert it to a SàT XML
+
+    @param submit_id: callback id to call when submitting form
+    @param session_id: id to return with the data
+    """
+    form_ui = XMLUI("form", "vertical", submit_id=submit_id, session_id=session_id)
+    return dataForm2Widgets(form_ui, form, read_only=read_only)
+
 def dataFormResult2AdvancedList(xmlui, form_xml):
     """Take a raw data form (not parsed by XEP-0004) and convert it to an advanced list
     raw data form is used because Wokkel doesn't manage result items parsing yet
@@ -130,7 +150,7 @@ def dataFormResult2AdvancedList(xmlui, form_xml):
     for item_elt in item_elts:
         for elt in item_elt.elements():
             if elt.name != 'field':
-                warning("Unexpected tag (%s)" % elt.name)
+                log.warning("Unexpected tag (%s)" % elt.name)
                 continue
             field = data_form.Field.fromElement(elt)
 
@@ -139,23 +159,39 @@ def dataFormResult2AdvancedList(xmlui, form_xml):
 
     return xmlui
 
-def dataFormResult2XMLUI(form_xml, session_id=None):
+def dataFormResult2XMLUI(form_elt, session_id=None):
     """Take a raw data form (not parsed by XEP-0004) and convert it to a SàT XMLUI
     raw data form is used because Wokkel doesn't manage result items parsing yet
-    @param form_xml: domish.Element of the data form
+    @param form_elt: domish.Element of the data form
     @return: XMLUI interface
     """
 
-    xmlui = XMLUI("window", "vertical", session_id=session_id)
-    dataFormResult2AdvancedList(xmlui, form_xml)
-    return xmlui
+    xml_ui = XMLUI("window", "vertical", session_id=session_id)
+    try:
+        dataFormResult2AdvancedList(xml_ui, form_elt)
+    except exceptions.DataError:
+        parsed_form = data_form.Form.fromElement(form_elt)
+        dataForm2Widgets(xml_ui, parsed_form, read_only=True)
+    return xml_ui
+
+def _cleanValue(value):
+    """Workaround method to avoid DBus types with D-Bus bridge
+
+    @param value: value to clean
+    @return: value in a non DBus type (only clean string yet)
+    """
+    # XXX: must be removed when DBus types will no cause problems anymore
+    # FIXME: should be cleaned inside D-Bus bridge itself
+    if isinstance(value, basestring):
+        return unicode(value)
+    return value
 
 def XMLUIResult2DataFormResult(xmlui_data):
     """ Extract form data from a XMLUI return
     @xmlui_data: data returned by frontends for XMLUI form
     @return: dict of data usable by Wokkel's dataform
     """
-    return {key[len(SAT_FORM_PREFIX):]: value for key, value in xmlui_data.iteritems() if key.startswith(SAT_FORM_PREFIX)}
+    return {key[len(SAT_FORM_PREFIX):]: _cleanValue(value) for key, value in xmlui_data.iteritems() if key.startswith(SAT_FORM_PREFIX)}
 
 def formEscape(name):
     """ Return escaped name for forms """
@@ -207,8 +243,9 @@ def paramsXML2XMLUI(xml):
             callback_id = param.getAttribute('callback_id') or None
 
             if type_ == 'list':
-                options = _getParamListOptions(param)
+                options, selected = _getParamListOptions(param)
                 widget_kwargs['options'] = options
+                widget_kwargs['selected'] = selected
 
             if type_ in ("button", "text"):
                 param_ui.addEmpty()
@@ -221,6 +258,10 @@ def paramsXML2XMLUI(xml):
 
             if callback_id:
                 widget_kwargs['callback_id'] = callback_id
+                others = ["%s%s%s" % (category_name, SAT_PARAM_SEPARATOR, other.getAttribute('name'))
+                          for other in category.getElementsByTagName('param')
+                          if other.getAttribute('type') != 'button']
+                widget_kwargs['fields_back'] = others
 
             widget_kwargs['name'] = "%s%s%s" % (category_name, SAT_PARAM_SEPARATOR, param_name)
 
@@ -230,16 +271,16 @@ def paramsXML2XMLUI(xml):
 
 
 def _getParamListOptions(param):
-    """Retrieve the options for list element. Allow listing the <option/>
-    tags directly in <param/> or in an intermediate <options/> tag."""
-    elems = param.getElementsByTagName("options")
-    if len(elems) == 0:
-        elems = param.getElementsByTagName("option")
-    else:
-        elems = elems.item(0).getElementsByTagName("option")
+    """Retrieve the options for list element. The <option/> tags
+    must be direct children of <param/>."""
+    if len(param.getElementsByTagName("options")) > 0:
+        raise exceptions.DataError(_("The 'options' tag is not allowed in parameter of type 'list'!"))
+    elems = param.getElementsByTagName("option")
     if len(elems) == 0:
         return []
-    return [elem.getAttribute("value") for elem in elems]
+    options = [elem.getAttribute("value") for elem in elems]
+    selected = [elem.getAttribute("value") for elem in elems if elem.getAttribute("selected") == 'true']
+    return  (options, selected)
 
 
 ## XMLUI Elements
@@ -298,19 +339,49 @@ class FieldBackElement(Element):
         self.elem.setAttribute('name', name)
 
 
+class InternalFieldElement(Element):
+    """ Used by internal callbacks to indicate which fields are manipulated """
+    type = 'internal_field'
+
+    def __init__(self, parent, name):
+        super(InternalFieldElement, self).__init__(parent.xmlui, parent)
+        self.elem.setAttribute('name', name)
+
+
+class InternalDataElement(Element):
+    """ Used by internal callbacks to retrieve extra data """
+    type = 'internal_data'
+
+    def __init__(self, parent, children):
+        super(InternalDataElement, self).__init__(parent.xmlui, parent)
+        assert(isinstance(children, list))
+        for child in children:
+            self.elem.childNodes.append(child)
+
+
 class OptionElement(Element):
     """" Used by ListWidget to specify options """
     type = 'option'
 
-    def __init__(self, parent, option):
+    def __init__(self, parent, option, selected=False):
+        """
+
+        @param parent
+        @param option (string, tuple)
+        @param selected (boolean)
+        """
         assert(isinstance(parent, ListWidget))
         super(OptionElement, self).__init__(parent.xmlui, parent)
         if isinstance(option, basestring):
             value, label = option, option
         elif isinstance(option, tuple):
             value, label = option
+        else:
+            raise NotImplementedError
         self.elem.setAttribute('value', value)
         self.elem.setAttribute('label', label)
+        if selected:
+            self.elem.setAttribute('selected', 'true')
 
 
 class RowElement(Element):
@@ -410,7 +481,7 @@ class AdvancedListContainer(Container):
 
     def __init__(self, xmlui, callback_id=None, name=None, headers=None, items=None, columns=None, selectable = 'no', auto_index = False, parent=None):
         """Create an advanced list
-        @param headers: optional headers informations
+        @param headers: optional headers information
         @param callback_id: id of the method to call when selection is done
         @param items: list of widgets to add (just the first row)
         @param columns: number of columns in this table, or None to autodetect
@@ -501,25 +572,54 @@ class Widget(Element):
             self.elem.setAttribute('name', name)
         self.elem.setAttribute('type', self.type)
 
+    def setInternalCallback(self, callback, fields, data_elts=None):
+        """Set an internal UI callback when the widget value is changed.
 
-class InputWidget(Widget):
-    pass
+        The internal callbacks are NO callback ids, they are strings from
+        a predefined set of actions that are running in the scope of XMLUI.
+
+        @param callback (string): a value from:
+            - 'copy': process the widgets given in 'fields' two by two, by
+                copying the values of one widget to the other. Target widgets
+                of type List do not accept the empty value.
+            - 'move': same than copy but moves the values if the source widget
+                is not a List.
+            - 'groups_of_contact': process the widgets two by two, assume A is
+                is a list of JID and B a list of groups, select in B the groups
+                to which the JID selected in A belongs.
+            - more operation to be added when necessary...
+        @param fields (list): a list of widget names (string)
+        @param data_elts (list[Element]): extra data elements
+        """
+        self.elem.setAttribute('internal_callback', callback)
+        if fields:
+            for field in fields:
+                InternalFieldElement(self, field)
+        if data_elts:
+            InternalDataElement(self, data_elts)
 
 
 class EmptyWidget(Widget):
+    """Place holder widget"""
     type = 'empty'
 
 
 class TextWidget(Widget):
+    """Used for blob of text"""
     type = 'text'
 
     def __init__(self, xmlui, value, name=None, parent=None):
         super(TextWidget, self).__init__(xmlui, name, parent)
+        value_elt = self.xmlui.doc.createElement('value')
         text = self.xmlui.doc.createTextNode(value)
-        self.elem.appendChild(text)
+        value_elt.appendChild(text)
+        self.elem.appendChild(value_elt)
 
 
 class LabelWidget(Widget):
+    """Used for one line blob of text,
+    most of time to display the desciption or name of the next widget
+    """
     type='label'
 
     def __init__(self, xmlui, label, name=None, parent=None):
@@ -528,6 +628,7 @@ class LabelWidget(Widget):
 
 
 class JidWidget(Widget):
+    """Used to display a Jabber ID, some specific methods can be added"""
     type='jid'
 
     def __init__(self, xmlui, jid, name=None, parent=None):
@@ -558,13 +659,29 @@ class DividerWidget(Widget):
         self.elem.setAttribute('style', style)
 
 
+### Inputs ###
+
+
+class InputWidget(Widget):
+    """Input widget are widget which can accept user inputs,
+    used mainly in forms
+    """
+    def __init__(self, xmlui, name=None, parent=None, read_only=False):
+        super(InputWidget, self).__init__(xmlui, name, parent)
+        if read_only:
+            self.elem.setAttribute('read_only', 'true')
+
+
 class StringWidget(InputWidget):
     type = 'string'
 
-    def __init__(self, xmlui, value=None, name=None, parent=None):
-        super(StringWidget, self).__init__(xmlui, name, parent)
+    def __init__(self, xmlui, value=None, name=None, parent=None, read_only=False):
+        super(StringWidget, self).__init__(xmlui, name, parent, read_only=read_only)
         if value:
-            self.elem.setAttribute('value', value)
+            value_elt = self.xmlui.doc.createElement('value')
+            text = self.xmlui.doc.createTextNode(value)
+            value_elt.appendChild(text)
+            self.elem.appendChild(value_elt)
 
 
 class PasswordWidget(StringWidget):
@@ -575,17 +692,23 @@ class TextBoxWidget(StringWidget):
     type = 'textbox'
 
 
+class JidInputWidget(StringWidget):
+    type = 'jid_input'
+
+
 class BoolWidget(InputWidget):
     type = 'bool'
 
-    def __init__(self, xmlui, value='false', name=None, parent=None):
-        if value == '0':
+    def __init__(self, xmlui, value='false', name=None, parent=None, read_only=False):
+        if isinstance(value, bool):
+            value = 'true' if value else 'false'
+        elif value == '0':
             value='false'
         elif value == '1':
             value='true'
         if not value in ('true', 'false'):
             raise exceptions.DataError(_("Value must be 0, 1, false or true"))
-        super(BoolWidget, self).__init__(xmlui, name, parent)
+        super(BoolWidget, self).__init__(xmlui, name, parent, read_only=read_only)
         self.elem.setAttribute('value', value)
 
 
@@ -607,31 +730,96 @@ class ButtonWidget(Widget):
         if value:
             self.elem.setAttribute('value', value)
         for field in fields_back:
-            fback_el = FieldBackElement(self, field)
+            FieldBackElement(self, field)
 
 
 class ListWidget(InputWidget):
     type = 'list'
 
-    def __init__(self, xmlui, options, value=None, style=None, name=None, parent=None):
+    def __init__(self, xmlui, options, selected=None, style=None, name=None, parent=None):
+        """
+
+        @param xmlui
+        @param options (list[option]): each option can be given as:
+            - a single string if the label and the value are the same
+            - a tuple with a couple of string (value,label) if the label and the value differ
+        @param selected (list[string]): list of the selected values
+        @param style (string)
+        @param name (string)
+        @param parent
+        """
         if style is None:
             style = set()
         styles = set(style)
         if not options:
-            warning(_('empty "options" list'))
+            log.warning(_('empty "options" list'))
         if not styles.issubset(['multi']):
             raise exceptions.DataError(_("invalid styles"))
         super(ListWidget, self).__init__(xmlui, name, parent)
-        self.addOptions(options)
-        if value:
-            self.elem.setAttribute('value', value)
+        self.addOptions(options, selected)
         for style in styles:
             self.elem.setAttribute(style, 'yes')
 
-    def addOptions(self, options):
-        """i Add options to a multi-values element (e.g. list) """
+    def addOptions(self, options, selected=None):
+        """Add options to a multi-values element (e.g. list) """
+        if selected:
+            if isinstance(selected, basestring):
+                selected = [selected]
+        else:
+            selected = []
         for option in options:
-            OptionElement(self, option)
+            assert(isinstance(option, basestring) or isinstance(option, tuple))
+            value = option if isinstance(option, basestring) else option[0]
+            OptionElement(self, option, value in selected)
+
+
+## Dialog Elements ##
+
+
+class DialogElement(Element):
+    """Main dialog element """
+    type = 'dialog'
+
+    def __init__(self, parent, type_, level=None):
+        if not isinstance(parent, TopElement):
+            raise exceptions.DataError(_("DialogElement must be a direct child of TopElement"))
+        super(DialogElement, self).__init__(parent.xmlui, parent)
+        self.elem.setAttribute(C.XMLUI_DATA_TYPE, type_)
+        self.elem.setAttribute(C.XMLUI_DATA_LVL, level or C.XMLUI_DATA_LVL_DEFAULT)
+
+
+class MessageElement(Element):
+    """Element with the instruction message"""
+    type = C.XMLUI_DATA_MESS
+
+    def __init__(self, parent, message):
+        if not isinstance(parent, DialogElement):
+            raise exceptions.DataError(_("MessageElement must be a direct child of DialogElement"))
+        super(MessageElement, self).__init__(parent.xmlui, parent)
+        message_txt = self.xmlui.doc.createTextNode(message)
+        self.elem.appendChild(message_txt)
+
+
+class ButtonsElement(Element):
+    """Buttons element which indicate which set to use"""
+    type = 'buttons'
+
+    def __init__(self, parent, set_):
+        if not isinstance(parent, DialogElement):
+            raise exceptions.DataError(_("ButtonsElement must be a direct child of DialogElement"))
+        super(ButtonsElement, self).__init__(parent.xmlui, parent)
+        self.elem.setAttribute('set', set_)
+
+
+class FileElement(Element):
+    """File element used for FileDialog"""
+    type = 'file'
+
+    def __init__(self, parent, type_):
+        if not isinstance(parent, DialogElement):
+            raise exceptions.DataError(_("FileElement must be a direct child of DialogElement"))
+        super(FileElement, self).__init__(parent.xmlui, parent)
+        self.elem.setAttribute('type', type_)
 
 
 ## XMLUI main class
@@ -640,29 +828,53 @@ class ListWidget(InputWidget):
 class XMLUI(object):
     """This class is used to create a user interface (form/window/parameters/etc) using SàT XML"""
 
-    def __init__(self, panel_type="window", container="vertical", title=None, submit_id=None, session_id=None):
+    def __init__(self, panel_type="window", container="vertical", dialog_opt=None, title=None, submit_id=None, session_id=None):
         """Init SàT XML Panel
+
         @param panel_type: one of
-            - window (new window)
-            - popup
-            - form (formulaire, depend of the frontend, usually a panel with cancel/submit buttons)
-            - param (parameters, presentation depend of the frontend)
+            - C.XMLUI_WINDOW (new window)
+            - C.XMLUI_POPUP
+            - C.XMLUI_FORM (formulaire, depend of the frontend, usually a panel with cancel/submit buttons)
+            - C.XMLUI_PARAM (parameters, presentation depend of the frontend)
+            - C.XMLUI_DIALOG (one common dialog, presentation depend of frontend)
         @param container: disposition of elements, one of:
             - vertical: elements are disposed up to bottom
             - horizontal: elements are disposed left to right
             - pairs: elements come on two aligned columns
               (usually one for a label, the next for the element)
             - tabs: elemens are in categories with tabs (notebook)
+        @param dialog_opt: only used if panel_type == C.XMLUI_DIALOG. Dictionnary (string/string) where key can be:
+            - C.XMLUI_DATA_TYPE: type of dialog, value can be:
+                - C.XMLUI_DIALOG_MESSAGE (default): an information/error message. Action of user is necessary to close the dialog. Usually the frontend display a classic popup
+                - C.XMLUI_DIALOG_NOTE: like a C.XMLUI_DIALOG_MESSAGE, but action of user is not necessary to close, at frontend choice (it can be closed after a timeout). Usually the frontend display as a timed out notification
+                - C.XMLUI_DIALOG_CONFIRM: dialog with 2 choices (usualy "Ok"/"Cancel").
+                    returned data can contain:
+                        - "answer": "true" if answer is "ok", "yes" or equivalent, "false" else
+                - C.XLMUI_DIALOG_FILE: a file selection dialog
+                    returned data can contain:
+                        - "cancelled": "true" if dialog has been cancelled, not present or "false" else
+                        - "path": path of the choosed file/dir
+            - C.XMLUI_DATA_MESS: message shown in dialog
+            - C.XMLUI_DATA_LVL: one of:
+                - C.XMLUI_DATA_LVL_INFO (default): normal message
+                - C.XMLUI_DATA_LVL_WARNING: attention of user is important
+                - C.XMLUI_DATA_LVL_ERROR: something went wrong
+            - C.XMLUI_DATA_BTNS_SET: one of:
+                - C.XMLUI_DATA_BTNS_SET_OKCANCEL (default): classical "OK" and "Cancel" set
+                - C.XMLUI_DATA_BTNS_SET_YESNO: a translated "yes" for OK, and "no" for Cancel
         @param title: title or default if None
-        @param submit_id: callback id to call for panel_type we can submit (form, param)
+        @param submit_id: callback id to call for panel_type we can submit (form, param, dialog)
+        @param session_id: use to keep a session attached to the dialog, must be returned by frontends
         """
         self._introspect()
-        if panel_type not in ['window', 'form', 'param', 'popup']:
+        if panel_type not in [C.XMLUI_WINDOW, C.XMLUI_FORM, C.XMLUI_PARAM, C.XMLUI_POPUP, C.XMLUI_DIALOG]:
             raise exceptions.DataError(_("Unknown panel type [%s]") % panel_type)
-        if panel_type == 'form' and submit_id is None:
+        if panel_type == C.XMLUI_FORM and submit_id is None:
             raise exceptions.DataError(_("form XMLUI need a submit_id"))
         if not isinstance(container, basestring):
             raise exceptions.DataError(_("container argument must be a string"))
+        if dialog_opt is not None and panel_type != C.XMLUI_DIALOG:
+            raise exceptions.DataError(_("dialog_opt can only be used with dialog panels"))
         self.type = panel_type
         impl = minidom.getDOMImplementation()
 
@@ -673,6 +885,11 @@ class XMLUI(object):
             top_element.setAttribute("title", title)
         self.submit_id = submit_id
         self.session_id = session_id
+        if panel_type == C.XMLUI_DIALOG:
+            if dialog_opt is None:
+                dialog_opt = {}
+            self._createDialog(dialog_opt)
+            return
         self.main_container  = self._createContainer(container, TopElement(self))
         self.current_container = self.main_container
 
@@ -698,6 +915,8 @@ class XMLUI(object):
 
     def __getattr__(self, name):
         if name.startswith("add") and name not in ('addWidget',): # addWidgetName(...) create an instance of WidgetName
+            if self.type == C.XMLUI_DIALOG:
+                raise exceptions.InternalError(_("addXXX can't be used with dialogs"))
             class_name = name[3:]+"Widget"
             if class_name in globals():
                 cls = globals()[class_name]
@@ -748,6 +967,29 @@ class XMLUI(object):
         else:
             raise exceptions.DataError("session_id can't be empty")
 
+    def _createDialog(self, dialog_opt):
+        dialog_type = dialog_opt.setdefault(C.XMLUI_DATA_TYPE, C.XMLUI_DIALOG_MESSAGE)
+        if dialog_type in [C.XMLUI_DIALOG_CONFIRM, C.XMLUI_DIALOG_FILE] and self.submit_id is None:
+            raise exceptions.InternalError(_("Submit ID must be filled for this kind of dialog"))
+        top_element = TopElement(self)
+        level = dialog_opt.get(C.XMLUI_DATA_LVL)
+        dialog_elt = DialogElement(top_element, dialog_type, level)
+
+        try:
+            MessageElement(dialog_elt, dialog_opt[C.XMLUI_DATA_MESS])
+        except KeyError:
+            pass
+
+        try:
+            ButtonsElement(dialog_elt, dialog_opt[C.XMLUI_DATA_BTNS_SET])
+        except KeyError:
+            pass
+
+        try:
+            FileElement(dialog_elt, dialog_opt[C.XMLUI_DATA_FILETYPE])
+        except KeyError:
+            pass
+
     def _createContainer(self, container, parent=None, **kwargs):
         """Create a container element
         @param type: container type (cf init doc)
@@ -784,6 +1026,23 @@ class XMLUI(object):
         return self.doc.toxml()
 
 
+# Some sugar for XMLUI dialogs
+
+def note(message, title='', level=C.XMLUI_DATA_LVL_INFO):
+    """Sugar to easily create a Note Dialog
+
+    @param message: body of the note
+    @param title: title of the note
+    @param level: one of C.XMLUI_DATA_LVL_*
+    """
+    note_xmlui = XMLUI(C.XMLUI_DIALOG, dialog_opt = {
+                       C.XMLUI_DATA_TYPE: C.XMLUI_DIALOG_NOTE,
+                       C.XMLUI_DATA_MESS: message,
+                       C.XMLUI_DATA_LVL: level},
+                       title=title
+                      )
+    return note_xmlui
+
 # Misc other funtions
 
 
@@ -792,7 +1051,11 @@ class ElementParser(object):
     Found at http://stackoverflow.com/questions/2093400/how-to-create-twisted-words-xish-domish-element-entirely-from-raw-xml/2095942#2095942
     (c) Karl Anderson"""
 
-    def __call__(self, string):
+    def __call__(self, string, force_spaces=False):
+        """
+        @param string: the XML string
+        @param force_spaces: if True, replace occurrences of 'n' and '\t' with ' '.
+        """
         self.result = None
 
         def onStart(elem):
@@ -809,6 +1072,8 @@ class ElementParser(object):
         parser.ElementEvent = onElement
         parser.DocumentEndEvent = onEnd
         tmp = domish.Element((None, "s"))
-        tmp.addRawXml(string.replace('\n', ' ').replace('\t', ' '))
+        if force_spaces:
+            string = string.replace('\n', ' ').replace('\t', ' ')
+        tmp.addRawXml(string)
         parser.parse(tmp.toXml().encode('utf-8'))
         return self.result.firstChildElement()
